# Copyright 2014 Canonical
# All Rights Reserved

"""Rest Scopes server."""

import importlib
import json
import os
import urlparse

import yaml

import utils

# this module is generated by a "make build"
from versioninfo import version_info

from translator import translator


REVNO_HEADER = ('X-Bzr-Revision-Number', str(version_info['revno']))

# load general config
with open(os.environ['REST_SCOPES_CONFIG']) as fh:
    config = yaml.load(fh)


def get_scopes_config():
    """Get the info for the scopes from the config(s) file(s)."""
    # check if we have the simple case (just the name) or a list with names
    scopes_config = config['scopes_config']
    if isinstance(scopes_config, basestring):
        scopes_config = [scopes_config]

    # compose the configs, by just updating a dict
    info = {}
    for fname in scopes_config:
        with open(os.path.join(os.environ['ROOTDIR'], fname)) as fh:
            info.update(yaml.load(fh))
    return info


def _get_scopes_secrets():
    """Get the secrets for the scopes from the config file."""
    scopes_config_file = os.path.join(os.environ['ROOTDIR'],
                                      config['scopes_secrets'])
    with open(scopes_config_file) as fh:
        secrets = yaml.load(fh) or {}
    return secrets


class App(object):
    """The WSGI application.

    It can receive a dict with extra config per scope, that will overwrite
    whatever the config in disk would be.
    """

    def __init__(self, extra_scope_config=None):
        # load scope info and modules
        scope_info = []
        self.scope_apps = {}
        info = get_scopes_config()
        scopes_secrets = _get_scopes_secrets()
        for scope_id, data in info.items():
            base_url = data['base_url']
            if base_url[0] == '/':
                # internal path
                base_url = config['server'] + base_url
                internal = True
            else:
                internal = False

            fqdn = data.get('fqdn')
            try:
                icon_fname = data['icon']
            except KeyError:
                icon_img = None
            else:
                icon_img = os.path.join(config['base_image_url'], icon_fname)
            try:
                art_fname = data['art']
            except KeyError:
                art_img = None
            else:
                art_img = os.path.join(config['base_image_url'], art_fname)

            # get appearance info, and fix logo path if needed
            appearance = data.get('appearance', {})
            _pageheader = appearance.get('page-header', {})
            if 'logo' in _pageheader:
                _pageheader['logo'] = os.path.join(config['base_image_url'],
                                                   _pageheader['logo'])

            scope_info.append({
                'id': fqdn if fqdn else scope_id,
                'name': data['name'].strip(),
                'description': data['description'].strip(),
                'base_url': base_url,
                'invisible': data['invisible'],
                'icon': icon_img,
                'art': art_img,
                'author': data['author'],
                'appearance': appearance,
            })
            if internal:
                package = data.get('mod_package', config['scopes_package'])
                mod_path = "%s.%s" % (package, scope_id)
                mod = importlib.import_module(mod_path)

                # build the config
                secrets = scopes_secrets.get(scope_id, {})
                scope_config = config.copy()
                scope_config['secrets'] = secrets
                if extra_scope_config is not None:
                    scope_config.update(extra_scope_config.get(scope_id, {}))

                self.scope_apps[scope_id] = mod.App(scope_config)
        self.scope_info = {None: scope_info}

    def _generic_response(self, status, start_response,
                          message="", **extra_headers):
        """Answer a generic response."""
        response_headers = [
            ('Content-type', "application/json"),
            ('Content-Length', str(len(message))),
            REVNO_HEADER,
        ]
        response_headers.extend(extra_headers.items())
        start_response(status, response_headers)
        return [message]

    def _error(self, status, start_response, message="",
               **extra_headers):
        """Answer an error response."""
        data = {"error": status}
        if message:
            data['message'] = message
        return self._generic_response(status, start_response,
                                      json.dumps(data, indent=4),
                                      **extra_headers)

    def __call__(self, environ, start_response):
        """Serve the request."""
        path = environ["PATH_INFO"].rstrip("/")
        if path.startswith("/_static"):
            return self.serve_static(environ, start_response, path)
        if path == "/remote-scopes":
            return self.list_remote_scopes(environ, start_response)

        # may be a scope!
        scope_name = path.split("/")[1]
        try:
            scope = self.scope_apps[scope_name]
        except KeyError:
            return self._error("404 NOT FOUND", start_response)

        scope_path = path[len(scope_name) + 1:]  # +1 because of initial /
        if len(scope_path) < 2:
            msg = "Malformed URL: need an action after the scope name"
            return self._error("400 BAD REQUEST", start_response, msg)

        # fix environment and serve
        environ["PATH_INFO"] = scope_path
        return scope(environ, start_response)

    def curate_list(self, country_blacklist, scopes):
        """Remove blacklisted scopes."""
        curated_list = []
        for scope in scopes:
            if scope['id'] in country_blacklist:
                continue
            curated_list.append(scope)
        return curated_list

    def list_remote_scopes(self, environ, start_response):
        """List all the configured scopes."""
        qs = environ.get("QUERY_STRING", "")
        qd = dict(urlparse.parse_qsl(qs, keep_blank_values=True))
        locale = utils.sanitize_locale(qd.get("locale"))
        country = qd.get('country')
        try:
            scope_list = self.scope_info[locale]
        except KeyError:
            # not built yet, let's do it
            scope_info = []
            for item in self.scope_info[None]:
                loc = item.copy()
                loc['description'] = translator(item['description'], locale)
                loc['name'] = translator(item['name'], locale)
                scope_info.append(loc)
            self.scope_info[locale] = scope_list = scope_info
        # Remove blacklisted scopes
        country_blacklist = config['blacklist'].get(country)
        if country_blacklist:
            curated_list = self.curate_list(country_blacklist, scope_list)
        else:
            curated_list = scope_list
        json_list = json.dumps(curated_list)
        return self._generic_response("200 OK", start_response, json_list)

    def serve_static(self, environ, start_response, path):
        """Serve static files from inside the project."""
        filepath = os.path.join(os.environ["ROOTDIR"], path[1:])
        if not os.access(filepath, os.R_OK):
            return self._error("404 NOT FOUND", environ, start_response)
        img_size = os.stat(filepath).st_size

        def stream():
            """Generator to return the data streamed."""
            with open(filepath, 'rb') as fh:
                while True:
                    chunk = fh.read(4096)
                    if not chunk:
                        break
                    yield chunk

        response_headers = [
            ('Content-Length', str(img_size)),
        ]
        start_response('200 OK', response_headers)
        return stream()
