# Copyright 2014 Canonical
# All Rights Reserved

"""A base Simple Application to provide common functionality to scopes.

When building a new builtin scope, just create a Python class (inheriting
this SimpleApp) and put it in a file in the scopes directory. Check the
src/scopes/wikipedia.py scope as a fully working example and the SimpleApp
docstring for more info.

Of course, if you want more complex needs and want to write the whole
stack, you can do it.
"""

import logging
import urlparse
import json

from .. import utils

# some defaults for the requests
DEFAULT_LIMIT = 10
DEFAULT_COUNTRY = 'US'

# to separate the result chunks
CHUNK_SEPARATOR = '\r\n'

logger = logging.getLogger('restscopes')


class SimpleApp(object):
    """A simple wsgi app that serves the two mandatory endpoints.

    Subclass at will, feel free to not use it at all if have more
    complex needs.

    Then overwrite the 'search' and 'preview' methods. These will be
    automatically called, on requests. Both methods will receive the request
    parameters; 'search' needs to 'yield' the categories/results/etc,
    and 'preview' needs to 'yield' the different previews. The
    specification of what these methods will receive and what they
    must yield/return is in FIXME.
    """

    def __init__(self, config):
        self.config = config
        self.default_limit = DEFAULT_LIMIT
        self.scope_module_name = self.__class__.__module__.split('.')[-1]

    def error(self, status, start_response, error_msg=None):
        """Answer an error to the client."""
        data = {"error": status}
        if error_msg is not None:
            data['error_msg'] = error_msg
        message = json.dumps(data, indent=4)

        response_headers = [
            ('Content-type', "application/json"),
            ('Content-Length', str(len(message))),
        ]
        start_response(status, response_headers)
        return [message]

    def _get_search_args(self, environ):
        """Get the parameters for the 'search', checking all is ok."""
        # get the query string, decode, build the query dictionary
        qs = environ.get("QUERY_STRING", "")
        qd = dict(urlparse.parse_qsl(qs, keep_blank_values=True))

        # latitude and longitude must be paired
        lat = float(qd['latitude']) if 'latitude' in qd else None
        lng = float(qd['longitude']) if 'longitude' in qd else None
        if lat is None and lng is not None or lng is None and lat is not None:
            msg = "latitude and longitude must be paired, both present or none"
            raise ValueError(msg)

        # get a valid locale
        locale = utils.sanitize_locale(qd.get('locale'))

        # dissasemble the filters, if present
        filters = qd.get('filters')
        if filters is not None:
            filters = json.loads(urlparse.unquote(filters))

        try:
            x_fwd_for = environ['HTTP_X_FORWARDED_FOR'].split(",")
            client_ip = x_fwd_for[0].strip()
        except Exception:
            # handle all errors, we don't want to fail if there is no IP
            client_ip = None
        # get all the rest of parameters, checking also type
        args = {
            'query': qd['q'].decode('utf8'),
            'session_id': qd.get('session_id'),
            'query_id': int(qd['query_id']) if 'query_id' in qd else None,
            'locale': locale,
            'platform': qd['platform'],
            'country': qd.get('country', DEFAULT_COUNTRY),
            'latitude': lat,
            'longitude': lng,
            'limit': int(qd.get('limit', self.default_limit)),
            'client_ip': client_ip,
            'department': qd.get('department'),
            'filters': filters,
        }
        return args

    def _get_preview_args(self, environ):
        """Get the parameters for the 'preview', checking all is ok."""
        # get the query string, decode, build the query dictionary
        qs = environ.get("QUERY_STRING", "")
        qd = dict(urlparse.parse_qsl(qs))

        # latitude and longitude must be paired
        lat = float(qd['latitude']) if 'latitude' in qd else None
        lng = float(qd['longitude']) if 'longitude' in qd else None
        if lat is None and lng is not None or lng is None and lat is not None:
            msg = "latitude and longitude must be paired, both present or none"
            raise ValueError(msg)

        # get a valid locale
        locale = utils.sanitize_locale(qd.get('locale'))

        try:
            x_fwd_for = environ['HTTP_X_FORWARDED_FOR'].split(",")
            client_ip = x_fwd_for[0].strip()
        except Exception:
            # handle all errors, we don't want to fail if there is no IP
            client_ip = None
        # get all the rest of parameters, checking also type
        args = {
            'session_id': qd.get('session_id'),
            'locale': locale,
            'platform': qd['platform'],
            'country': qd.get('country', DEFAULT_COUNTRY),
            'latitude': lat,
            'longitude': lng,
            'widgets_api_version': int(qd['widgets_api_version']),
            'result': json.loads(qd['result']),
            'client_ip': client_ip,
        }
        return args

    def search(self, **kwargs):
        """To be overwritten by subclass.

        Will receive the request parameters, needs to 'yield'
        the categories/results/etc.
        """
        raise NotImplementedError("Need to overwrite 'search' method.")

    def preview(self, **kwargs):
        """To be overwritten by subclass.

        Will receive the request parameters; needs to 'yield' the
        columns info and widgets.
        """
        raise NotImplementedError("Need to overwrite 'preview' method.")

    def __call__(self, environ, start_response):
        """Serve the request."""
        path = environ["PATH_INFO"]

        # select the methods to serve the request
        if path == '/search':
            args_checker = self._get_search_args
            serving_method = self.search
            log_msg = ("Start search [%(scope)s]: session_id=%(session_id)r "
                       "locale=%(locale)r country=%(country)s query=%(query)r")
        elif path == '/preview':
            args_checker = self._get_preview_args
            serving_method = self.preview
            log_msg = ("Start preview [%(scope)s]: session_id=%(session_id)r "
                       "locale=%(locale)r country=%(country)s")
        else:
            return self.error("404 NOT FOUND", start_response)

        # get parameters
        try:
            args = args_checker(environ)
        except ValueError as err:
            msg = "Problem in parameters: " + str(err)
            return self.error("400 BAD REQUEST", start_response, msg)
        except KeyError as err:
            msg = "Missing parameter " + str(err)
            return self.error("400 BAD REQUEST", start_response, msg)

        log_args = dict(scope=self.scope_module_name)
        log_args.update(args)
        logger.info(log_msg % log_args)

        def chunk_generator():
            """Send encoded chunks to the user."""
            for result in serving_method(**args):
                msg = json.dumps(result)
                yield msg + CHUNK_SEPARATOR

        # start the response and send the chunks to the user
        response_headers = [
            ('Content-type', 'application/json'),
        ]
        start_response("200 OK", response_headers)
        return chunk_generator()
