# -*- coding: utf-8 -*-

"""The Amazon scope."""

import bottlenose
import collections
import json
import logging
import operator
import os
import random
import urllib
import yaml

from itertools import groupby
from lxml import etree
from operator import itemgetter

from ..scopes import SimpleApp
from ..utils import markup_cleaner, localized_category
from ..translator import translator

logger = logging.getLogger("restscopes.amazon")

AMAZON_FILTERS = ['MaximumPrice', 'MinimumPrice', 'Publisher', 'Orchestra',
                  'Manufacturer', 'Director', 'Conductor', 'Condition',
                  'Composer', 'Brand', 'Availability', 'Author',
                  'AudienceRating', 'Actor', 'Artist']

# NOTE that we still say that "subtitle" show the price, because the 'price'
# field is not still shown in the client (see LP:1282460)
_renderer_generic = {
    "schema-version": 1,
    'template': {'category-layout': 'grid',
                 'card-size': 'large',
                 'card-layout': 'horizontal',
                 'collapsed-rows': 0},
    'components': {'title': 'title',
                   'art': 'art',
                   'subtitle': 'price'},
}
GENERIC_CATEGORY = {
    'id': 'generic',
    'title': translator("Results"),
    'render_template': json.dumps(_renderer_generic),
    'icon': "FIXME",
}

_renderer_landpage = {
    "schema-version": 1,
    'template': {'category-layout': 'grid',
                 'card-size': 'small',
                 'collapsed-rows': 0},
    'components': {'title': 'title',
                   'art': 'art',
                   'subtitle': 'price'},
}
LANDPAGE_CATEGORY = {
    'id': 'landpage',
    'title': '',  # this title will be fulfilled later
    'render_template': json.dumps(_renderer_landpage),
    'icon': "FIXME",
}

SORTING_FILTER_ID = 'sorting_primary_filter'
SORTING_INFO = {
    '+pmrank': translator("Featured items"),
    '-age-min': translator("Age: high to low"),
    '-albumrank': translator("Album rank: low to high"),
    '-artistalbumrank': translator("Artist album rank: low to high"),
    '-date': translator("Release date: old to new"),
    '-daterank': translator("Publication date: old to new"),
    '-edition-sales-velocity': translator("Quickest to slowest "
                                          "selling products"),
    '-launch-date': translator("Newest arrivals"),
    '-launchdate': translator("Launch date: latest to earliest"),
    '-mfg-age-min': translator("Age: high to low"),
    '-orig-rel-date': translator("Release date: older to newer"),
    '-price': translator("Price: high to low"),
    '-price-new-bin': translator("Price: high to low"),
    '-pricerank': translator("Price: high to low"),
    '-pubdate': translator("Publish date: oldest to recent"),
    '-publication_date': translator("Publication date: oldest to most recent"),
    '-publicationdate': translator("Publication date: oldest to most recent"),
    '-releank': translator("Review rank: low to high"),
    '-release-date': translator("Release date: newer to older"),
    '-releasedate': translator("Release date: newer to older"),
    '-runtime': translator("Track length: low to high"),
    '-titlerank': translator("Alphabetical: Z to A"),
    '-video-release-date': translator("Release date: newer to older"),
    'albumrank': translator("Album rank: high to low"),
    'amzrank': translator("Most to least available"),
    'artistalbumrank': translator("Artist album rank: high to low"),
    'artistrank': translator("Artist name: A to Z"),
    'availability': translator("Most to least available"),
    'date': translator("Release date: new to old"),
    'daterank': translator("Publication date: newer to older"),
    'inverse-pricerank': translator("Price: high to low"),
    'inverseprice': translator("Price: high to low"),
    'launch-date': translator("Newest arrivals"),
    'launchdate': translator("Launch date: newer to older"),
    'mfg-age-min': translator("Age: low to high"),
    'orig-rel-date': translator("Original release date: newer to older"),
    'pct-off': translator("Discount:  high to low"),
    'pmrank': translator("Featured items"),
    'popularityrank': translator("Items ranked by popularity"),
    'price': translator("Price: low to high"),
    'price-new-bin': translator("Price: low to high"),
    'pricerank': translator("Price: low to high"),
    # Bestseller ranking taking into consideration projected sales. The lower
    # the value, the better the sales.
    'psrank': translator("Bestseller (projected, lower is better)"),
    'pubdate': translator("Publication date: older to newer"),
    'publication_date': translator("Publication date: most recent to oldest"),
    'publicationdate': translator("Publication date: most recent to oldest"),
    'release-date': translator("Latest release date: newer to older"),
    'releasedate': translator("Release date: most recent to oldest"),
    # Items ranked according to the following criteria: how often the
    # keyword appears in the description, where the keyword appears (for
    # example, the ranking is higher when keywords are found in titles),
    # how closely they occur in descriptions (if there are multiple
    # keywords), and how often customers purchased the products they found
    # using the keyword.
    'relevancerank': translator("Relevance ranking"),
    'reviewrank': translator("Average customer review: high to low"),
    'reviewrank_authority': translator("Review rank: high to low"),
    'runtime': translator("Track length: high to low"),
    'sale-flag': translator("On sale"),
    'salesrank': translator("Bestselling"),
    'songtitlerank': translator("Most popular"),
    'subslot-salesrank': translator("Bestselling"),
    'titlerank': translator("Alphabetical: A to Z"),
    'uploaddaterank': translator("Date added"),
}

SHOPS = {
    "CA": "u1treatyrslf-ca-20",
    "CN": "u1treatyrslf-23",
    "DE": "u1treatyrslf-21",
    "ES": "u1treatyrslf-es-21",
    "FR": "u1treatyrslf-fr-21",
    "IT": "u1treatyrslf-it-21",
    "JP": "u1treatyrslf-22",
    "UK": "u1treatyrslf-uk-21",
    "US": "u1treatyrslf-20",
}

SHOP_MAP = {
    "GB": "UK",
}

AMZ_VERSION = "2011-08-01"
AMZ_NS = {'x': "http://webservices.amazon.com/AWSECommerceService/2011-08-01"}

CATEGORIES_BLACKLIST = {
    u"Mobile Application",
    u"Digital Music Album",
    u"Digital Music Track",
    u"Digital Music Artist",
}


def _normalize_shop(country):
    """Return a useful Amazon's shop."""
    shop = country.upper()
    if shop in SHOP_MAP:
        shop = SHOP_MAP[shop]
    if shop not in SHOPS:
        shop = "US"
    return shop


def _get_amz(shop, key, secret):
    """Call the Amazon API."""
    return bottlenose.Amazon(key, secret, Version=AMZ_VERSION,
                             AssociateTag=SHOPS[shop], Region=shop)


def _parse_item(item, shop, subtitle_attribs):
    """Parse a product result."""
    ns = AMZ_NS

    # filter out some items
    if item.xpath('x:ItemAttributes/x:IsAdultProduct[text()="1"]',
                  namespaces=ns):
        return None
    pgroup = item.findtext("x:ItemAttributes/x:ProductGroup", namespaces=ns)
    if pgroup in CATEGORIES_BLACKLIST:
        return None

    # the first usable attribute we find, we use it (or them, if several)
    itemattrs = item.xpath("x:ItemAttributes", namespaces=ns)
    subtitle = ''
    if itemattrs:
        itemattr = itemattrs[0]
        for attr in subtitle_attribs:
            internal = itemattr.xpath(attr, namespaces=ns)
            if internal:
                subtitle = ", ".join(x.text for x in internal[:3])
                break

    # get the useful info
    images = sorted([(int(i.findtext('x:Width', namespaces=ns)),
                      int(i.findtext('x:Height', namespaces=ns)),
                      i.findtext('x:URL', namespaces=ns))
                     for i in item.xpath('x:ImageSets/*/*', namespaces=ns)])
    images = dict(("%dx%d" % i[0], [j[2] for j in i[1]])
                  for i in groupby(images, itemgetter(0, 1)))
    price_type = item.xpath("x:Offers/x:Offer/x:OfferListing/x:Price",
                            namespaces=ns)
    if price_type:
        price = price_type[0].findtext("x:FormattedPrice", namespaces=ns)
    else:
        price = ''
    asin = item.findtext("x:ASIN", namespaces=ns)
    result = dict(
        id=asin,
        title=item.findtext("x:ItemAttributes/x:Title", namespaces=ns),
        url=item.findtext("x:DetailPageURL", namespaces=ns),
        images=images,
        formatted_price=price,
        subtitle=subtitle,
    )
    return result


def _pick_images(images, min_size):
    """Pick images larger than min_size.

    Results are ordered from largest to smaller. If have one image, just use
    it. If no images found larger than min_size, return largests.
    """
    if not images:
        return []

    # sort by the numeric value of the sizes, largest first
    sizes = sorted((tuple(map(int, size.split('x'))) for size in images),
                   reverse=True)

    results = []
    results.extend(images["%dx%d" % sizes[0]])  # largests will always be there

    for cand in sizes[1:]:
        if cand[0] < min_size[0] and cand[1] < min_size[1]:
            break
        results.extend(images["%dx%d" % cand])

    return results


def details(shop, item_id, key, secret, subtitle_attribs):
    """Fetch details for current item."""
    amz = _get_amz(shop, key, secret)
    rg = "Images,ItemAttributes,OfferSummary,Offers,EditorialReview"
    raw = amz.ItemLookup(ItemId=item_id, ResponseGroup=rg)
    xml = etree.XML(raw)

    try:
        item = xml.xpath("//x:Item", namespaces=AMZ_NS)[0]
    except IndexError:
        return None

    result = _parse_item(item, shop, subtitle_attribs)
    if result is None:
        return None

    description = item.findtext(
        "x:EditorialReviews/x:EditorialReview[1]/x:Content", namespaces=AMZ_NS)
    if description:
        result['description'] = markup_cleaner(description)

    features = item.xpath("x:ItemAttributes/x:Feature", namespaces=AMZ_NS)
    if features:
        texts = [u" * " + feat.text for feat in features]
        result['features'] = "\n".join(texts)
    return result


def parse_filters(filters):
    """Parse and validate filters."""
    filters_args = {}
    for f in filters:
        if f in AMAZON_FILTERS:
            filters_args[f] = filters[f]
    return filters_args


def _get_department_info(amz, department):
    """Get surrounding info from an Amazon department."""
    raw = amz.BrowseNodeLookup(BrowseNodeId=department,
                               ResponseGroup='BrowseNodeInfo')
    xml = etree.XML(raw)

    # get parent info
    ancestors = xml.xpath("//x:Ancestors", namespaces=AMZ_NS)
    if ancestors:
        data = {e.tag.split('}')[1]: e.text for e in ancestors[0][0]}
        parent = {'name': data['Name'], 'id': data['BrowseNodeId']}
    else:
        parent = None

    # get parent and root info
    node = xml.xpath("//x:BrowseNodes/x:BrowseNode", namespaces=AMZ_NS)[0]
    data = {e.tag.split('}')[1]: e.text for e in node}
    current = {'name': data['Name'], 'id': data['BrowseNodeId']}

    # get children list
    nodes = xml.xpath("//x:BrowseNodes/x:BrowseNode/x:Children",
                      namespaces=AMZ_NS)
    children = []
    if nodes:
        for child in nodes[0]:
            data = {e.tag.split('}')[1]: e.text for e in child}
            child = {'name': data['Name'], 'id': data['BrowseNodeId']}
            children.append(child)

            # check if it has grandchildren
            raw = amz.BrowseNodeLookup(BrowseNodeId=child['id'],
                                       ResponseGroup='BrowseNodeInfo')
            with open("/tmp/test_%s.xml" % (child['id'],), "wb") as fh:
                fh.write(raw)
            xml = etree.XML(raw)
            nodes = xml.xpath("//x:BrowseNodes/x:BrowseNode/x:Children",
                              namespaces=AMZ_NS)
            child['has_children'] = bool(nodes)

    return parent, current, children


def _build_canned(dep_code=None, dep_id=None, filters=None):
    """Build a canned query for the department tree."""
    params = []
    if dep_code and dep_id:
        params.append('dep=%s:%s' % (dep_code, dep_id))
    if filters:
        params.append('filters=%s' % (filters,))

    if params:
        cq = 'scope://amazon?' + '&'.join(params)
    else:
        cq = 'scope://amazon'
    return cq


def _get_department_tree(amz, root_departments, department, locale, filters):
    """Build the tree to navigate in departments."""
    encfilt = urllib.quote(json.dumps(filters)) if filters else ''
    if department is None:
        # just send them all
        subdepartments = []
        for dep_id, (dep_code, dep_name) in root_departments.items():
            subdepartments.append({
                'label': dep_name,
                'canned_query': _build_canned(dep_code, dep_id, encfilt),
                'has_subdepartments': True,
            })
        subdepartments.sort(key=operator.itemgetter('label'))
        tree = {
            'label': translator("All departments", locale),
            'canned_query': _build_canned(filters=encfilt),
            'subdepartments': subdepartments,
        }
    else:
        # need to get the info for this node
        root_code, department_id = department.split(':')
        parent, current, children = _get_department_info(amz, department_id)
        if parent is None:
            parent = {'name': translator("All departments", locale),
                      'id': None}
        subdepartments = []
        for child in children:
            subdepartments.append({
                'label': child['name'],
                'canned_query': _build_canned(root_code, child['id'], encfilt),
                'has_subdepartments': child['has_children'],
            })
        subdepartments.sort(key=operator.itemgetter('label'))
        tree = {
            # this is the parent node
            'label': parent['name'],
            'canned_query': _build_canned(root_code, parent['id'], encfilt),
            'subdepartments': [{
                # this is 'current' node, just one in the list
                'label': current['name'],
                'canned_query': _build_canned(root_code,
                                              current['id'], encfilt),
                'subdepartments': subdepartments,  # this is the children list
            }],
        }
    return tree


def _get_sorting_filters(allowed_sorting, locale, current_filter_state):
    """Get the sorting filters."""
    # build the proper structure
    options = [{'id': s, 'label': translator(SORTING_INFO[s], locale)}
               for s in allowed_sorting]
    options.sort(key=operator.itemgetter('label'))
    new_sorting_list = {
        'filter_type': 'option_selector',
        'display_hints': 'primary',
        'id': SORTING_FILTER_ID,
        'label': translator("Featured"),
        'multi_select': False,
        'options': options,
    }

    # if the filter state is NOT using any of the current filters, fix it
    sorting_state = current_filter_state.get(SORTING_FILTER_ID)
    if sorting_state is not None:
        current_selected = sorting_state[0]  # only one, it's not multiselect
        if current_selected not in allowed_sorting:
            # make the sorting to have no state (untouching other filters)
            del current_filter_state[SORTING_FILTER_ID]

    return new_sorting_list, current_filter_state


class App(SimpleApp):
    """The Amazon app."""

    def __init__(self, *args, **kwargs):
        super(App, self).__init__(*args, **kwargs)
        self.api_utils = {'key': self.config['secrets'].get('key'),
                          'secret': self.config['secrets'].get('secret')}

        _rootdir = os.environ['ROOTDIR']
        _configdir = self.config['scopes_config_dir']
        _path = os.path.join(_rootdir, _configdir, 'amazon_departments.yaml')
        with open(_path, 'r') as f:
            self.departments_map = yaml.load(f)
        _path = os.path.join(_rootdir, _configdir, 'amazon_sorting.yaml')
        with open(_path, 'r') as f:
            self.sorting_info = yaml.load(f)
        _path = os.path.join(_rootdir, _configdir, 'amazon_subtitles.yaml')
        with open(_path, 'r') as f:
            self.subtitle_attribs = yaml.load(f)

        # build the attribs for the non-department case
        c = collections.Counter(a[0] for a in self.subtitle_attribs.values())
        self.subtitle_attribs[None] = [attr for attr, _ in c.most_common()]

    def search(self, **kwargs):
        """Do the search."""
        query = kwargs['query'].strip()
        limit = kwargs['limit']
        locale = kwargs['locale']
        session_id = kwargs['session_id']
        shop = _normalize_shop(kwargs['country'])
        shop_sorting = self.sorting_info[shop]
        shop_departments = self.departments_map[shop]

        department = kwargs['department']
        filter_state = kwargs['filters']
        if filter_state is None:
            filter_state = {}
        amz = _get_amz(shop, self.api_utils['key'], self.api_utils['secret'])

        # for surfacing we need to use a single space and
        # keep returning navigation (departments and sorting stuff)
        if query:
            # a normal search... put a title to the category
            category = localized_category(GENERIC_CATEGORY, locale)
        else:
            # no query, surfacing!
            category = GENERIC_CATEGORY.copy()
            category['title'] = ''
            query = ' '
            tree = _get_department_tree(amz, shop_departments,
                                        department, locale, filter_state)
            yield dict(departments=tree)

            if department is None:
                # no query and no department: this is the land page: choose
                # a random department and a different category for the results
                category = LANDPAGE_CATEGORY.copy()
                possible = list(shop_departments)
                while True:
                    dep_id = random.choice(possible)
                    root_code, dep_name = shop_departments[dep_id]
                    if 'salesrank' in shop_sorting.get(root_code, ()):
                        break
                category['title'] = dep_name + translator(', best sellers',
                                                          locale)
                # fake a department and a filter to be used in the query
                department = '%s:%s' % (root_code, dep_id)
                filter_state = {SORTING_FILTER_ID: ['salesrank']}
            else:
                # normal surfacing (no query, department indicated)
                root_code, dep_id = department.split(':')
                new_sorting_list, new_sorting_state = _get_sorting_filters(
                    shop_sorting[root_code], locale, filter_state)
                yield dict(filters=[new_sorting_list])
                if new_sorting_state:
                    yield dict(filter_state=new_sorting_state)

        # hit Amazon and get all the info
        kwargs = dict(
            SearchIndex='All',
            Keywords=query,
            ResponseGroup="Images,ItemAttributes,OfferSummary,Offers",
        )
        root_code = None
        if department is not None:
            # search in a specific department
            root_code, dep_id = department.split(':')
            kwargs['SearchIndex'] = root_code
            kwargs['BrowseNode'] = dep_id

            # sorting is only useful with a department selected
            sorting = filter_state.get(SORTING_FILTER_ID)
            if sorting is not None:
                kwargs['Sort'] = sorting[0]

        logger.debug("ItemSearch [%s]: %s", session_id, kwargs)
        raw = amz.ItemSearch(**kwargs)
        xml = etree.XML(raw)
        items = xml.xpath("//x:Item", namespaces=AMZ_NS)

        # send all responses
        subtitle_attribs = self.subtitle_attribs[root_code]
        yield dict(category=category)
        for item in items[0:limit]:
            item = _parse_item(item, shop, subtitle_attribs)
            if item is not None:
                images = _pick_images(item['images'], (200, 200))
                yield dict(result={
                    'cat_id': category['id'],
                    'uri': item['url'],
                    'dnd_uri': item['url'],
                    'art': images[-1] if images else None,
                    'title': item['title'],
                    'subtitle': item['subtitle'],
                    'price': item.get('formatted_price'),
                    'preview_info': {
                        'item_id': item['id'],
                        'root_code': root_code,
                    }
                })

    def preview(self, **kwargs):
        """Do the preview."""
        result = kwargs['result']
        preview_info = result['preview_info']
        locale = kwargs['locale']
        shop = _normalize_shop(kwargs['country'])

        # Fetch item details
        subtitle_attribs = self.subtitle_attribs[preview_info['root_code']]
        bits = details(shop=shop,
                       key=self.api_utils['key'],
                       secret=self.api_utils['secret'],
                       item_id=preview_info['item_id'],
                       subtitle_attribs=subtitle_attribs)
        images = _pick_images(bits['images'], (400, 400))
        uri = result['uri']

        # yield an 'image gallery' widget
        if images:
            yield dict(widget={
                'id': 'imagegallery',
                'type': 'gallery',
                'sources': images,
            })

        # yield a 'header' widget
        yield dict(widget={
            'id': 'header1',
            'type': 'header',
            'uri': uri,
            'title': result['title'],
            'subtitle': result['price'],
            'attribute-1': None,
            'attribute-2': None,
            'attribute-3': None,
            'emblem': None,
        })

        yield dict(widget={
            'id': 'buttons',
            'type': 'actions',
            'actions': [{'label': translator('View on Amazon', locale),
                         'uri': uri,
                         'id': 'action1'}]
        })

        # if there is some description, send it as a 'text' widget
        description = bits.get('description')
        if description:
            yield dict(widget={
                'id': 'text1',
                'type': 'text',
                'uri': uri,
                'title': translator("Description", locale),
                'text': description,
            })

        # if there is some features, send them as a 'text' widget
        features = bits.get('features')
        if features:
            yield dict(widget={
                'id': 'text2',
                'type': 'text',
                'uri': uri,
                'title': translator("Features and Details", locale),
                'text': features,
            })
