# Copyright 2014 Canonical
# All Rights Reserved

"""The Grooveshark scope."""

import urllib
import json
import hmac

from urllib2 import urlopen, Request

from ..scopes import SimpleApp
from ..translator import translator
from ..utils import localized_category

SEARCH_URI = 'https://api.grooveshark.com/ws3.php?sig='

DEFAULT_COUNTRY = {'CC1': 0,
                   'CC2': 0,
                   'CC3': 0,
                   'CC4': 0,
                   'DMA': 0,
                   'ID': 0,
                   'IPR': 0}

CLIENT_IP = '173.194.42.19'


class APIError(Exception):
    """ API errors catcher """


def api_call(method, api_utils, client_ip=CLIENT_IP, **parameters):
    """ Call the Grooveshark API """
    data = {}
    data['method'] = method
    data['parameters'] = parameters
    data['header'] = {'wsKey': api_utils['key']}
    data_str = json.dumps(data).encode("utf-8")
    sig = hmac.new(api_utils['secret'], data_str).hexdigest()
    headers = {'X-Client-IP': client_ip}
    req = Request(SEARCH_URI+sig, data_str,
                  headers=headers)
    response = urlopen(req)
    response_data = response.read()
    result = json.loads(response_data.decode("utf-8"))
    if 'errors' in result:
        error = result.get('errors')[0]
        raise APIError("Error: %r" % (error,))
    return result

# Override category renderer
RENDERER_MUSIC = {
    "schema-version": 1,
    'template': {'category-layout': 'grid', 'card-size': 'medium'},
    'components': {'title': 'title',
                   'art': 'art',
                   'subtitle': 'artist'},
}

RENDERER_MUSIC_SMALL = {
    "schema-version": 1,
    'template': {'category-layout': 'grid', 'card-size': 'small'},
    'components': {'title': 'title',
                   'art': 'art',
                   'subtitle': 'artist'},
}


# Declare categories
SONGS_CATEGORY = {
    'id': 'cat_0',
    'title': translator('Songs'),
    'render_template': json.dumps(RENDERER_MUSIC),
    'icon': "FIXME",
}

SONGS_CATEGORY_SMALL = {
    'id': 'cat_0',
    'title': translator('Songs'),
    'render_template': json.dumps(RENDERER_MUSIC_SMALL),
    'icon': "FIXME",
}

ALBUMS_CATEGORY = {
    'id': 'cat_1',
    'title': translator('Albums'),
    'render_template': json.dumps(RENDERER_MUSIC),
    'icon': "FIXME",
}

ALBUMS_CATEGORY_SMALL = {
    'id': 'cat_1',
    'title': translator('Albums'),
    'render_template': json.dumps(RENDERER_MUSIC_SMALL),
    'icon': "FIXME",
}


class App(SimpleApp):
    """The Grooveshark app."""

    def __init__(self, *args, **kwargs):
        super(App, self).__init__(*args, **kwargs)
        self.api_utils = {'key': self.config['secrets'].get('api_key'),
                          'secret': self.config['secrets'].get('api_secret')}

    def search(self, **kwargs):
        """Do the search."""
        query = kwargs['query']
        limit = kwargs['limit']
        locale = kwargs['locale']
        form_factor = 'phone'  # FIXME http://pad.lv/1281720
        client_ip = kwargs.get('client_ip') or CLIENT_IP
        if query == '':
            yield dict(category=localized_category(SONGS_CATEGORY_SMALL,
                                                   locale))
            yield dict(category=localized_category(ALBUMS_CATEGORY_SMALL,
                                                   locale))
        else:
            yield dict(category=localized_category(SONGS_CATEGORY, locale))
            yield dict(category=localized_category(ALBUMS_CATEGORY, locale))
        if form_factor == "phone":
            base_uri = 'http://html5.grooveshark.com'
        else:
            base_uri = 'http://grooveshark.com'
        albums = {'result': {'albums': []}}
        songs = {'result': {'songs': []}}

        if not query.strip():
            songs = api_call("getPopularSongsMonth",
                             self.api_utils,
                             client_ip=client_ip, limit=limit)
        else:
            albums = api_call("getAlbumSearchResults",
                              self.api_utils,
                              client_ip=client_ip,
                              query=query, limit=limit)
            songs = api_call("getSongSearchResults",
                             self.api_utils,
                             client_ip=client_ip,
                             query=query, limit=10,
                             country=DEFAULT_COUNTRY)

        for i in albums['result']['albums']:
            if i['AlbumName'].strip().lower() != query.strip().lower():
                continue
            album_name_parsed = urllib.quote(i['AlbumName'].encode('utf-8'))
            album_uri = '%(uri)s/#/album/%(aname)s/%(aid)s' \
                % dict(uri=base_uri,
                       aname=album_name_parsed,
                       aid=str(i['AlbumID']))
            if i.get('CoverArtFilename') and \
                    len(i.get('CoverArtFilename')) > 0:
                icon = 'http://beta.grooveshark.com/static/' \
                    'amazonart/m%(cover)s' \
                    % dict(cover=str(i['CoverArtFilename']))
            else:
                icon = 'http://beta.grooveshark.com/static/' \
                    'amazonart/m%(aid)s.jpg' \
                    % dict(aid=str(i['AlbumID']))
            yield dict(result={'uri': album_uri,
                               'title': i['AlbumName'],
                               'album': i['AlbumName'],
                               'artist': i['ArtistName'],
                               'album_id': str(i['AlbumID']),
                               'album_uri': album_uri,
                               'art': icon,
                               'form_factor': form_factor,
                               'cat_id': 'cat_1'})

        for i in songs['result']['songs']:
            album_name_parsed = urllib.quote(i['AlbumName'].encode('utf-8'))
            album_uri = '%(uri)s/#/album/%(aname)s/%(aid)s' \
                % dict(uri=base_uri,
                       aname=album_name_parsed,
                       aid=str(i['AlbumID']))
            if i.get('CoverArtFilename') and \
                    len(i.get('CoverArtFilename')) > 0:
                icon = 'http://beta.grooveshark.com/static/' \
                    'amazonart/m%(aid)s' \
                    % dict(aid=str(i['CoverArtFilename']))
            else:
                icon = 'http://beta.grooveshark.com/static/' \
                    'amazonart/m%(aid)s.jpg' \
                    % dict(aid=str(i['AlbumID']))
            yield dict(result={'uri': album_uri,
                               'title': i['SongName'],
                               'album': i['AlbumName'],
                               'artist': i['ArtistName'],
                               'album_id': str(i['AlbumID']),
                               'song_id': str(i['SongID']),
                               'art': icon,
                               'form_factor': form_factor,
                               'cat_id': 'cat_0'})

    def preview(self, **kwargs):
        """Do the preview."""
        result = kwargs['result']
        locale = kwargs['locale']
        client_ip = kwargs.get('client_ip') or CLIENT_IP
        action_uri = result['uri']
        try:
            song = api_call("getSongURLFromSongID",
                            self.api_utils,
                            client_ip=client_ip,
                            songID=result['song_id'])
            if result['form_factor'] == "phone":
                action_uri = song['url'].replace('/grooveshark.com/s',
                                                 '/html5.grooveshark.com/s')
        except:
            pass
        # yield an 'image' widget
        yield dict(widget={
            'id': 'art',
            'type': 'image',
            'source': result['art'],
        })
        # yield a 'header' widget
        yield dict(widget={
            'id': 'header',
            'type': 'header',
            'title': result['title'],
            'art': result['art'],
            'subtitle': result['artist'],
        })
        # add an 'actions' widget
        yield dict(widget={
            'id': 'buttons',
            'type': 'actions',
            'actions': [{'label': translator('Play in Grooveshark', locale),
                         'uri': action_uri,
                         'id': 'action1'}]
        })
