# -*- coding: utf-8 -*-

"""The Onlinemusic scope."""

import json
import uuid
import urllib
import urllib2

from ..scopes import SimpleApp
from ..translator import translator
from ..utils import localized_category

HEADERS = {
    'User-Agent': "Ubuntu Rest Server - Onlinemusic scope",
    'Accept': '*/*',
}
SEARCH_URL = '/search?'
PREVIEW_URL = '/preview?'

SCOPES_LIST = {'music-grooveshark.scope', 'music-soundcloud.scope'}


def _get_search(baseurl, query, locale, session_id, scopes):
    """Hit productsearch to do a search."""
    query = urllib.quote(query.encode("utf8"))
    url = baseurl + SEARCH_URL + urllib.urlencode({
        'q': query,
        'locale': locale,
        'session_id': session_id,
        'scopes': ','.join(scopes)})
    req = urllib2.Request(url, headers=HEADERS)
    return urllib2.urlopen(req).read()


def _get_preview(baseurl, locale, session_id, server_sid, prev_result,
                 result_id):
    """Hit productsearch with a preview request."""
    url = baseurl + PREVIEW_URL + urllib.urlencode(
        {
            'session_id': session_id,
            'locale': locale,
            'result_id': result_id
        })

    headers = {
        'X-SERVER-SID': server_sid.encode('utf-8'),
        'X-PREVIOUS-RESULT': json.dumps(prev_result).encode('utf-8')
    }
    req = urllib2.Request(url, None, headers)
    return urllib2.urlopen(req).read()


class App(SimpleApp):
    """ The Onlinemusic app. """

    RENDERER_DEFAULT = {
        "schema-version": 1,
        "template": {
            "category-layout": "grid",
            "card-size": "small"
        },
        "components": {
            "title": "title",
            "subtitle": "artist",
            "art": "art"
        }
    }

    CATEGORY_ONLINE = 'online'
    CATEGORY_POPULARONLINE = 'popular'
    music_categories = {
        CATEGORY_ONLINE: {
            'id': CATEGORY_ONLINE,
            'title': translator('Online'),
            'render_template': json.dumps(RENDERER_DEFAULT),
            'icon': "FIXME",
        },
        CATEGORY_POPULARONLINE: {
            'id': CATEGORY_POPULARONLINE,
            'title': translator('Popular online'),
            'render_template': json.dumps(RENDERER_DEFAULT),
            'icon': "FIXME",
        }
    }

    def __init__(self, *args, **kwargs):
        super(App, self).__init__(*args, **kwargs)
        self._baseurl = self.config['onlinemusic_server']

    def search(self, **kwargs):
        """ Do the search """

        query = kwargs['query']
        limit = kwargs['limit']
        locale = kwargs['locale']
        country = kwargs['country']
        session_id = uuid.uuid4()
        blacklist = self.config['blacklist'].get(country, [])
        scopes = list(SCOPES_LIST - set(blacklist))
        if query == '':
            scopes.append("more_suggestions-populartracks.scope")
            cat_id = self.CATEGORY_POPULARONLINE
        else:
            scopes.append("more_suggestions-u1ms.scope")
            cat_id = self.CATEGORY_ONLINE

        category = self.music_categories[cat_id]
        yield dict(category=localized_category(category, locale))

        r = _get_search(self._baseurl, query, locale, session_id, scopes)
        server_sid = None
        for line in r.decode('utf-8').split('\r\n'):
            if line != '':
                data = json.loads(line)
                # collect all results in a scope_id -> resultset dict
                results = {}
                if data['type'] == 'recommendations':
                    server_sid = self._process_recommendations(data)
                # we expect recommendation 1st, so that we get server_sid
                if server_sid is not None:
                    if 'type' in data and 'info' in data:
                        for (scope_id, resultset) in \
                            self._process_search_results(
                                data, cat_id, server_sid, session_id):
                            results[scope_id] = resultset

                # shuffle results (scopeA, scopeB, scopeA, scopeB ...)
                has_data = True
                count = 0
                while has_data:
                    has_data = False
                    for res in results.keys():
                        if len(results[res]):
                            yield dict(result=results[res].pop(0))
                            has_data = True
                            count += 1
                            if count >= limit:
                                return

    def _process_recommendations(self, data):
        """ Process recommendations to get server_sid """

        if 'server_sid' in data:
            return data['server_sid']
        return None

    def _process_search_results(self, data, category, server_sid, session_id):
        """ Process search results """

        if data['type'] == 'results':
            for scope_id in data['info'].keys():
                results = []
                for res in data['info'][scope_id]:
                    uri = res['uri']
                    title = res['title']
                    icon = res['icon_hint']
                    comment = res['comment']
                    results.append({
                        'cat_id': category,
                        'uri': uri,
                        'dnd_uri': uri,
                        'title': title,
                        'artist': comment,
                        'art': icon,
                        'ssid': server_sid,
                        'session': str(session_id),
                        'real_result': res
                    })
                yield (scope_id, results)

    def preview(self, **kwargs):
        """ Do the preview """

        result = kwargs['result']
        locale = kwargs['locale']

        session_id = result['session']
        server_sid = result['ssid']
        prev_result = result['real_result']
        result_id = prev_result['metadata']['id']

        r = _get_preview(self._baseurl, locale, session_id,
                         server_sid, prev_result, result_id)
        data = json.loads(r.decode('utf-8'))

        title = data['title']
        subtitle = data['subtitle']
        image = ''
        if 'image_hint' in data:
            image = data['image_hint']

        actions = []
        if 'actions' in data:
            for act in data['actions']:
                if 'activation_uri' in act and 'display_name' in act:
                    uri = act['activation_uri']
                    action = {'id': uri, 'label': act['display_name']}
                    if 'icon_hint' in act:
                        action['icon'] = act['icon_hint']
                    actions.append(action)

        # handle special attributes of music preview
        tracks = []
        if 'renderer_name' in data and \
                data['renderer_name'] == 'preview-music':
            if 'tracks' in data:
                for track in data['tracks']:
                    if all(attr in track for attr in
                            ['uri', 'title', 'track_no', 'length']):
                        tracks.append({
                            'title': track['title'],
                            'source': track['uri'],
                            'length': track['length']
                        })

        # column layouts
        cols = [[[]], [[], []]]

        if image != '':
            cols[0][0].append('art')
            cols[1][0].append('art')
            cols[1][1].append('header')
        else:
            cols[1][0].append('header')
        cols[0][0].append('header')

        if len(actions):
            cols[0][0].append('buttons')
            cols[1][1].append('buttons')
        if len(tracks):
            cols[0][0].append('tracks')
            cols[1][1].append('tracks')

        yield(dict(columns=cols))

        if image != '':
            yield dict(widget={
                'id': 'art',
                'type': 'image',
                'source': image
            })

        yield dict(widget={
            'id': 'header',
            'type': 'header',
            'title': title,
            'subtitle': subtitle
        })

        if len(actions):
            yield dict(widget={
                'id': 'buttons',
                'type': 'actions',
                'actions': actions
            })

        if len(tracks):
            yield dict(widget={
                'id': 'tracks',
                'type': 'audio',
                'tracks': tracks
            })
