# Copyright 2014 Canonical
# All Rights Reserved

"""The Open Library scope."""

import urllib
import json

from urllib2 import urlopen
from random import choice

from ..scopes import SimpleApp
from ..translator import translator
from ..utils import localized_category

SEARCH_URI = 'http://openlibrary.org'


SURFACING_SUBJECTS = [
    translator('Love'),
    translator('Space'),
    translator('Murder'),
    translator('Voyages and travels'),
    translator('Mystery'),
    translator('Cats'),
    translator('Science fiction'),
    translator('Romance'),
    translator('Fantasy'),
    translator('Drama'),
    translator('Classic Literature'),
    translator('Historical Fiction'),
    translator('Fiction'),
    translator('Pirates'),
    translator('Adventure and adventurers'),
    translator('Sea stories'),
    translator('Dragons'),
    translator('Dinosaurs'),
    translator('Space travel'),
    translator('Hugo Award Winner'),
    translator('Greek Mythology'),
    translator('Imaginary places'),
    translator('Wizards'),
    translator('Revenge'),
    translator('American fiction'),
    translator('Fairy tales'),
    translator('Short stories'),
]


RENDERER_BOOK_VERTICAL = {
    "schema-version": 1,
    'template': {'category-layout': 'grid',
                 'card-size': 'small',
                 'collapsed-rows': 1,
                 'card-layout': 'vertical'},
    'components': {'title': 'title',
                   'art': {'field': 'art', 'aspect-ratio': 0.75},
                   'subtitle': 'author', },
}


RENDERER_BOOK_BY = {
    "schema-version": 1,
    'template': {'category-layout': 'grid',
                 'card-size': 'small',
                 'collapsed-rows': 1, },
    'components': {'title': 'title',
                   'art': {'field': 'art', 'aspect-ratio': 0.75},
                   'subtitle': 'author'},
}


# the first two categories will have their 'title' field set later
# when building the search response
BOOKS_CATEGORY = {
    'id': 'books',
    'render_template': json.dumps(RENDERER_BOOK_VERTICAL),
    'icon': None,
}
BOOKS_BY_CATEGORY = {
    'id': 'books-by',
    'render_template': json.dumps(RENDERER_BOOK_BY),
    'icon': None,
}
BOOKS_SEARCH_CATEGORY = {
    'id': 'books',
    'title': translator('Books'),
    'render_template': json.dumps(RENDERER_BOOK_VERTICAL),
    'icon': None,
}


def _get_books(url):
    """Call the Open Library API."""
    response = urlopen(url).read()
    results = json.loads(response.decode('utf-8'))
    return results


def _pick_random_author(data):
    """Get the most common author in results."""
    author_list = [item.get('author_name', [''])[0] for item in data]
    author = choice(author_list)
    return author


def _get_book_cover(d):
    """Get book cover."""
    if d.get('cover_i'):
        cover = 'http://covers.openlibrary.org/b/id/' \
            + str(d['cover_i'])+'-M.jpg'
    elif d.get('cover_id'):
        cover = 'http://covers.openlibrary.org/b/id/' \
            + str(d['cover_id'])+'-M.jpg'
    else:
        cover = None
    return cover


def _get_book_author(d):
    """Get book main author."""
    if d.get('author_name'):
        author = d['author_name'][0]
    elif d.get('authors'):
        author = d['authors'][0]['name']
    else:
        author = ''
    return author


def _get_readable_formats(editions_data):
    """Pick uris from editions with readable content."""
    all_formats = {}
    for eds in editions_data:
        if editions_data[eds].get('ebooks'):
            if editions_data[eds]['ebooks'][0]['availability'] == 'full':
                if 'read_url' in editions_data[eds]['ebooks'][0]:
                    all_formats['online'] = editions_data[
                        eds]['ebooks'][0]['read_url']
                formats = editions_data[eds]['ebooks'][0]['formats']
                if 'pdf' in formats:
                    all_formats['pdf'] = formats['pdf']['url']
                if 'epub' in formats:
                    all_formats['epub'] = formats['epub']['url']
    return all_formats


def _get_description(work_data):
    """Get book description from data."""
    try:
        desc = work_data['description']['value'].strip()
    except (TypeError, KeyError):
        try:
            desc = work_data['description'].strip()
        except (TypeError, KeyError):
            desc = None
    return desc


class App(SimpleApp):
    """The Open Library app."""

    def __init__(self, *args, **kwargs):
        super(App, self).__init__(*args, **kwargs)
        self.default_limit = 30

    def search(self, **kwargs):
        """Do the search."""
        limit = kwargs['limit']
        locale = kwargs['locale']
        query = kwargs['query'].encode('utf-8')

        if not query.strip():
            subject = choice(SURFACING_SUBJECTS)
            uri = '%(base_uri)s/search.json?subject=%(subj)s&limit=%(l)i' % \
                dict(base_uri=SEARCH_URI,
                     subj=urllib.quote(subject.lower().encode('utf-8')),
                     l=limit)
            docs = _get_books(uri)['docs']
            cat_books = BOOKS_CATEGORY.copy()
            cat_books['title'] = translator(subject, locale)
            yield dict(category=cat_books)

            featured_author = _pick_random_author(docs)
            uri = '%(base_uri)s/search.json?author=%(q)s&limit=%(l)i' % \
                dict(base_uri=SEARCH_URI,
                     q=urllib.quote(featured_author.encode('utf-8')),
                     l=limit)
            docs += _get_books(uri)['docs']
            cat_books_by = BOOKS_BY_CATEGORY.copy()
            cat_books_by['title'] = featured_author
            yield dict(category=cat_books_by)

        else:
            featured_author = None
            q = urllib.quote(query)
            uri = '%(base_uri)s/search.json?q=%(q)s&limit=%(l)i' % \
                dict(base_uri=SEARCH_URI, q=q, l=limit)
            docs = _get_books(uri)['docs']
            yield dict(category=localized_category(BOOKS_SEARCH_CATEGORY,
                                                   locale))
        parsed_books = []
        for d in docs:
            cover = _get_book_cover(d)
            author = _get_book_author(d)
            if author+d['title'] in parsed_books or not cover:
                continue
            first_sentence = d.get('first_sentence', [None])[0]
            first_published = d.get('first_publish_year')
            if d.get('edition_key'):
                eds_list = d['edition_key']
                if len(eds_list) > 40:
                    # get some of the head (recent editions) and some of the
                    # tail (potentially public domain)
                    eds_list = eds_list[:20] + eds_list[-20:]
                editions = 'OLID:' + ',OLID:'.join(eds_list)
            else:
                editions = ''
            key = str(d['key']).split('/')[-1]
            uri = "https://openlibrary.org/works/" + key
            # Pick categories
            if author == featured_author:
                cat_id = 'books-by'
            else:
                cat_id = 'books'
            yield dict(result={
                'uri': uri,
                'dnd_uri': uri,
                'art': cover,
                'cat_id': cat_id,
                'title': d['title'],
                'author': author,
                'first_sentence': first_sentence,
                'first_published': first_published,
                'OLID': key,
                'editions': editions,
                'has_fulltext': d['has_fulltext'],
            })
            parsed_books.append(author+d['title'])

    def preview(self, **kwargs):
        """Do the preview."""
        result = kwargs['result']
        locale = kwargs['locale']
        if result['has_fulltext'] and result['editions'].strip():
            editions_uri = '%(base_uri)s/api/books/?bibkeys=%(eds)s&' \
                           'format=json&jscmd=data' % \
                           dict(base_uri=SEARCH_URI, eds=result['editions'])
            editions_data = _get_books(editions_uri)
            formats = _get_readable_formats(editions_data)
        else:
            formats = {}
        data_uri = '%(base_uri)s/works/%(key)s.json' % \
                   dict(base_uri=SEARCH_URI, key=result['OLID'])
        data = _get_books(data_uri)

        try:
            by = data['by_statement']
        except:
            by = result['author']

        desc = _get_description(data)
        if not desc:
            if result['first_sentence']:
                desc = '\"'+result['first_sentence']+'\"'
            else:
                desc = None

        # yield an 'image' widget
        yield dict(widget={
            'id': 'art',
            'type': 'image',
            'source': result['art'].replace('-M.jpg', '-L.jpg'),
        })

        # yield a 'header' widget
        yield dict(widget={
            'id': 'header',
            'type': 'header',
            'title': result['title'],
            'subtitle': by,
        })

        actions = []
        actions.append({'uri': result['uri'],
                        'label': translator('View', locale),
                        'id': 'view'})
        if formats.get('online'):
            actions.append({'uri': formats['online'],
                            'label': translator('Read Online', locale),
                            'id': 'online'})
        if formats.get('pdf'):
            actions.append({'uri': formats['pdf'],
                            'label': translator('PDF', locale),
                            'id': 'pdf'})
        if formats.get('epub'):
            actions.append({'uri': formats['epub'],
                            'label': translator('epub', locale),
                            'id': 'epub'})
        # yield an 'actions' widget
        yield dict(widget={
            'id': 'buttons',
            'type': 'actions',
            'actions': actions
        })

        if desc:
            # yield a 'text' widget
            yield dict(widget={
                'id': 'text',
                'type': 'text',
                'text': desc,
            })
