# Copyright 2014 Canonical
# All Rights Reserved

"""The Reddit scope."""

import urllib
import json

from urllib2 import urlopen
from urllib2 import Request

from ..scopes import SimpleApp
from ..translator import translator
from ..utils import localized_category

SEARCH_URI = 'http://reddit.com'

RENDERER_THREAD = {
    "schema-version": 1,
    'template': {'category-layout': 'grid',
                 'card-size': 'large',
                 'card-layout': 'horizontal'},
    'components': {'title': 'title',
                   'mascot': 'mascot',
                   'subtitle': 'domain'},
}
THREAD_CATEGORY = {
    'id': 'stories',
    'title': translator('Stories'),
    'render_template': json.dumps(RENDERER_THREAD),
    'icon': "FIXME",
}


def _get_threads(url):
    """Call the Reddit API."""
    hdr = {'User-Agent': 'unity-scope-reddit'}
    req = Request(url=url, headers=hdr)
    response = urlopen(req).read()
    results = json.loads(response.decode('utf-8'))
    return results


class App(SimpleApp):
    """The Reddit app."""

    def __init__(self, *args, **kwargs):
        super(App, self).__init__(*args, **kwargs)
        self.default_limit = 25

    def search(self, **kwargs):
        """Do the search."""
        limit = kwargs['limit']
        locale = kwargs['locale']
        query = kwargs['query'].encode('utf-8')
        if not query.strip() or len(query) < 3:
            uri = '%(base_uri)s/.json?limit=%(limit)i' % \
                dict(base_uri=SEARCH_URI, limit=limit)
        else:
            query_cleaned = urllib.quote(query.split('#')[0])
            uri = '%(base_uri)s/r/%(q)s/hot.json?limit=%(limit)i' % \
                dict(base_uri=SEARCH_URI, q=query_cleaned, limit=limit)
        yield dict(category=localized_category(THREAD_CATEGORY, locale))

        for d in _get_threads(uri)['data']['children']:

            # We need both or neither to have proper previews.
            try:
                media_type = d['data']['media']['oembed']['type']
                media_preview = d['data']['media']['oembed']['thumbnail_url']
            except (TypeError, KeyError):
                media_type, media_preview = None, None

            if d['data']['thumbnail'] != 'self':
                art = d['data']['thumbnail']
            else:
                art = ""
            if not art.strip() and media_preview:
                art = media_preview
            if d['data']['domain'] == 'i.imgur.com':
                if not d['data']['url'].endswith('.gif'):
                    media_preview = d['data']['url']
                    media_type = "rich"
            if d['data']['over_18'] is True and '#nsfw' not in query:
                art = None
                media_preview = None
            uri = 'http://reddit.com' + d['data']['permalink']
            yield dict(result={
                'uri': uri,
                'dnd_uri': uri,
                'mascot': art,
                'cat_id': 'stories',
                'title': d['data']['title'],
                'subreddit': d['data']['subreddit'],
                'author': d['data']['author'],
                'comments': str(d['data']['num_comments']),
                'domain': d['data']['domain'],
                'summary': d['data']['selftext'],
                'target': d['data']['url'],
                'media_type': media_type,
                'media_preview': media_preview,
            })

    def preview(self, **kwargs):
        """Do the preview."""
        result = kwargs['result']
        locale = kwargs['locale']
        mascot = None

        if result['media_type'] == "rich" \
                and result['media_preview'] \
                and not result['media_preview'].endswith('.gif'):
            yield dict(widget={
                'id': 'image',
                'type': 'image',
                'source': result['media_preview'],
            })
        elif result['media_type'] == "video":
            yield dict(widget={
                'id': 'video',
                'type': 'video',
                'source': result['target'],
                'screenshot': result['media_preview'],
            })
        else:
            mascot = result['mascot']
        # yield a 'header' widget
        yield dict(widget={
            'id': 'header',
            'type': 'header',
            'title': result['title'],
            'subtitle': result['domain'],
            'mascot': mascot,
        })
        # add an 'actions' widget
        if int(result['comments']) == 1:
            com_text = ' comment'
        else:
            com_text = ' comments'
        actions = [{'label': result['comments'] + com_text,
                    'uri': result['uri'],
                    'id': 'action1'}]
        if not result['target'].startswith("http://www.reddit.com/r/"):
            actions.append({'label': translator('View', locale),
                            'uri': result['target'],
                            'id': 'action2'})
        yield dict(widget={
            'id': 'buttons',
            'type': 'actions',
            'actions': actions,
        })
        # yield a 'header' widget
        yield dict(widget={
            'id': 'text',
            'type': 'text',
            'text': result['summary'],
        })
