"""The Remote videos scope."""

import json
import urllib
import urllib2

from HTMLParser import HTMLParser

from ..scopes import SimpleApp
from ..translator import translator
from ..utils import localized_category

SEARCH_URL = "/search?"
HEADERS = {
    'User-Agent': "Ubuntu Rest Server - Remotevideos scope",
    'Accept': '*/*',
}


def _get_search(baseurl, query, form_factor, geo_store):
    """Hit videosearch to do a search."""
    query = urllib.quote(query.encode("utf8"))
    url = baseurl + SEARCH_URL + urllib.urlencode({
        'q': query,
        'form_factor': form_factor,
        'split': 'true',
        'geo_store': geo_store})
    req = urllib2.Request(url, headers=HEADERS)
    return urllib2.urlopen(req).read()


def _get_preview(details_url):
    """Hit videosearch to do a preview."""
    req = urllib2.Request(details_url, headers=HEADERS)
    return urllib2.urlopen(req).read()


class MarkupCleaner (HTMLParser):
    """HTML Parser that removed all HTML tags."""
    def __init__(self):
        self.reset()
        self._data = []

    def handle_data(self, data):
        """Handles data."""
        self._data.append(data)

    def get_stripped_data(self):
        """Returns collected data."""
        return ''.join(self._data)


class App(SimpleApp):
    """The Remote videos app."""

    RENDERER_ONLINE = {
        "schema-version": 1,
        "template": {
            "category-layout": "grid",
            "card-size": "medium"
        },
        "components": {
            "title": "title",
            "art": {
                "field": "art",
                "aspect-ratio": 1.4
            }
        }
    }
    RENDERER_FREE = {
        "schema-version": 1,
        "template": {
            "category-layout": "grid",
            "card-size": "small"
        },
        "components": {
            "title": "title",
            "art": {
                "field": "art",
                "aspect-ratio": 0.66
            }
        }
    }

    CATEGORY_ONLINE = 'online'
    CATEGORY_MORESUGGESTIONS = 'treat'
    video_categories = {
        CATEGORY_ONLINE: {
            'id': CATEGORY_ONLINE,
            'title': translator('Online'),
            'render_template': json.dumps(RENDERER_ONLINE),
            'icon': "FIXME",
        },
        CATEGORY_MORESUGGESTIONS: {
            'id': CATEGORY_MORESUGGESTIONS,
            'title': translator('More suggestions'),
            'render_template': json.dumps(RENDERER_FREE),
            'icon': "FIXME",
        }
    }

    def __init__(self, *args, **kwargs):
        super(App, self).__init__(*args, **kwargs)
        self._baseurl = self.config['remotevideos_server']

    def search(self, **kwargs):
        """Do the search."""
        query = kwargs['query']
        limit = kwargs['limit']
        geo_store = kwargs['country']
        platform = kwargs['platform']
        locale = kwargs['locale']

        formfactor = 'phone'
        if platform:
            formfactor = platform.split('-')[0]

        data = None
        r = _get_search(self._baseurl, query, formfactor, geo_store)
        data = json.loads(r.decode('utf-8'))

        if data is not None:
            treats = data['treats']
            other = data['other']
            yield dict(category=localized_category(
                self.video_categories[self.CATEGORY_ONLINE], locale))
            yield dict(category=localized_category(
                self.video_categories[self.CATEGORY_MORESUGGESTIONS], locale))
            count = 0
            for res in self._process_search_results(
                    other, self.CATEGORY_ONLINE, locale):
                count += 1
                yield dict(result=res)
                if limit > 0 and count >= limit:
                    break
            for res in self._process_search_results(
                    treats, self.CATEGORY_MORESUGGESTIONS, locale):
                count += 1
                yield dict(result=res)
                if limit > 0 and count >= limit:
                    break

    def _process_search_results(self, data, category, locale):
        """Process search results."""
        results = []
        for res in data:
            url = res['url']
            title = res['title']
            icon = res['img']
            source = res['source']
            details_uri = ''
            if 'details' in res:
                details_uri = res['details']
            price = None
            if 'formatted_price' in res:
                price = res['formatted_price']
                if price == "free":
                    price = translator("Free", locale)
            results.append({
                'cat_id': category,
                'uri': url,
                'dnd_uri': url,
                'title': title,
                'art': icon,
                'details_uri': details_uri,
                'source': source
            })
        return results

    def preview(self, **kwargs):
        """Do the preview."""
        result = kwargs['result']
        locale = kwargs['locale']

        uri = result['uri']
        title = result['title']
        subtitle = result['source']
        image = result['art']
        description = ''

        details_uri = result['details_uri']
        if details_uri != '':
            r = _get_preview(details_uri)
            data = json.loads(r.decode('utf-8'))
            image = ''
            if 'image' in data:
                image = data['image']
            elif 'img' in data:
                image = data['img']
            description = ''
            if 'description' in data:
                description = data['description']
            elif 'description_html' in data:
                cleaner = MarkupCleaner()
                cleaner.feed(data['description_html'])
                description = cleaner.get_stripped_data()

            # source = data['source']
            # release date, genres and price not used at the moment
            # price = ''
            # if 'formatted_price' in data:
            #    price = data['formatted_price']
            directors = []
            if 'directors' in data:
                directors = data['directors']
            starring = []
            if 'starring' in data:
                starring = data['starring']
            # genres = []
            # if 'genres' in data:
            #    genres = data['genres']
            uploaded_by = ''
            if 'uploaded_by' in data:
                uploaded_by = data['uploaded_by']
            # if 'date_uploaded' in data:
            #    date_uploaded = data['date_uploaded']

            if uploaded_by != '':
                subtitle = uploaded_by
            elif directors != '':
                subtitle = ','.join(directors)
            elif starring != '':
                subtitle = ','.join(starring)
            if subtitle == '':
                subtitle = data['source']

        # column layouts
        cols = [[['video', 'header', 'buttons', 'description']],
                [['video', 'header'], ['buttons', 'description']]]
        if description == '':
            cols[1][1].remove('description')
        yield(dict(columns=cols))

        yield dict(widget={
            'id': 'video',
            'type': 'video',
            'source': result['uri'],
            'screenshot': image
        })

        yield dict(widget={
            'id': 'header',
            'type': 'header',
            'title': title,
            'subtitle': subtitle
        })

        yield dict(widget={
            'id': 'buttons',
            'type': 'actions',
            'actions': [{'id': 'play',
                         'label': translator('Play', locale),
                         'uri': uri}]
        })

        if description != '':
            yield dict(widget={
                'id': 'description',
                'type': 'text',
                'text': description
            })
