# -*- coding: utf8 -*-

# Copyright 2014 Canonical
# All Rights Reserved

"""Tests for the Simple App base stuff."""

import json
import logging.handlers
import types
import urllib

from unittest import TestCase

from mock import Mock

from src import scopes, utils


class SetupLogChecker(logging.handlers.MemoryHandler):
    """A fake handler to store the records."""

    def __init__(self, test_instance, log_path):
        # init memory handler to never flush
        super(SetupLogChecker, self).__init__(capacity=100000, flushLevel=1000)
        self.test_instance = test_instance
        test_instance.assertLogged = self._check

        # hook in the logger
        logger = logging.getLogger(log_path)
        logger.addHandler(self)
        logger.setLevel(logging.DEBUG)
        self.setLevel(logging.DEBUG)

    def _check(self, *tokens):
        """Check if the the different tokens were logged in one record."""
        for record in self.buffer:
            msg = record.msg
            if all(token in msg for token in tokens):
                return

        # didn't exit, all tokens are not present in the same record
        self.test_instance.fail("Tokens %s were not logged: %s" % (
            tokens, [r.msg for r in self.buffer]))


class CallTestCase(TestCase):
    """Test when calling the class."""

    def setUp(self):
        self.app = scopes.SimpleApp(config={})
        self.start_response = Mock()
        SetupLogChecker(self, 'restscopes')

    def test_endpoints(self):
        self.app.error = mock = Mock()
        mock.return_value = 'error response'

        environ = {
            'PATH_INFO': '/bad_endpoint',  # 'search' or 'preview' only
        }
        resp = self.app(environ, 'start_response')
        self.assertEqual(resp, 'error response')
        mock.assert_called_with('404 NOT FOUND', 'start_response')

    def test_endpoints_search(self):
        self.app = scopes.SimpleApp(config={})
        self.app.scope_module_name = 'test scope name'
        self.app._get_search_args = check_mock = Mock()
        validated_args = dict(some='args', foo='bar', query='testquery',
                              session_id='test_session', locale='testlocale',
                              country='testcountry')
        check_mock.return_value = validated_args
        self.app.search = action_mock = Mock()
        action_mock.return_value = iter([{'resp': 'data'}, [1, 2]])

        environ = {
            'PATH_INFO': '/search',
            'FOO': 'bar',
        }
        chunked = self.app(environ, self.start_response)
        self.assertTrue(isinstance(chunked, types.GeneratorType))
        response = list(chunked)
        check_mock.assert_called_with(environ)
        action_mock.assert_called_with(**validated_args)
        self.assertEqual(response, ['{"resp": "data"}\r\n', '[1, 2]\r\n'])
        self.start_response.assert_called_with("200 OK", [
            ('Content-type', 'application/json')])
        self.assertLogged("Start search", "testquery", "session", "testlocale",
                          "testcountry", 'test scope name')

    def test_endpoints_preview(self):
        self.app = scopes.SimpleApp(config={})
        self.app.scope_module_name = 'test scope name'
        self.app._get_preview_args = check_mock = Mock()
        validated_args = dict(some='args', foo='bar', country='testcountry',
                              session_id='test_session', locale='testlocale')
        check_mock.return_value = validated_args
        self.app.preview = action_mock = Mock()
        action_mock.return_value = iter([{'resp': 'data'}, [1, 2]])

        environ = {
            'PATH_INFO': '/preview',
            'FOO': 'bar',
        }
        chunked = self.app(environ, self.start_response)
        self.assertTrue(isinstance(chunked, types.GeneratorType))
        response = list(chunked)
        check_mock.assert_called_with(environ)
        action_mock.assert_called_with(**validated_args)
        self.assertEqual(response, ['{"resp": "data"}\r\n', '[1, 2]\r\n'])
        self.start_response.assert_called_with("200 OK", [
            ('Content-type', 'application/json')])
        self.assertLogged("Start preview", "session", "testlocale",
                          "testcountry", 'test scope name')


class SearchParamsValidationTestCase(TestCase):
    """Test the validation of parameters needed in the search."""

    def setUp(self):
        self.app = scopes.SimpleApp(config={})

    def test_simple_ok(self):
        q = 'q=foo&platform=desktop&country=ar&limit=4&locale=es_AR'
        resp = self.app._get_search_args({'QUERY_STRING': q})
        should = {
            'query': u'foo',
            'session_id': None,
            'query_id': None,
            'locale': 'es_AR',
            'platform': 'desktop',
            'country': 'ar',
            'latitude': None,
            'longitude': None,
            'limit': 4,
            'client_ip': None,
            'department': None,
            'filters': None,
        }
        self.assertEqual(resp, should)

    def test_query_unicode(self):
        q = 'q=mo\xc3\xb1o&platform=desktop&country=ar&limit=4&locale=es_AR'
        resp = self.app._get_search_args({'QUERY_STRING': q})
        should = {
            'query': u'moño',
            'session_id': None,
            'query_id': None,
            'locale': 'es_AR',
            'platform': 'desktop',
            'country': 'ar',
            'latitude': None,
            'longitude': None,
            'limit': 4,
            'client_ip': None,
            'department': None,
            'filters': None,
        }
        self.assertEqual(resp, should)

    def test_latitude_longitude(self):
        q = (
            'q=mo\xc3\xb1o&platform=desktop&country=ar&limit=4&locale=es_AR'
            '&latitude=1.2312&longitude=3.13123'
        )
        resp = self.app._get_search_args({'QUERY_STRING': q})
        should = {
            'query': u'moño',
            'session_id': None,
            'query_id': None,
            'locale': 'es_AR',
            'platform': 'desktop',
            'country': 'ar',
            'latitude': 1.2312,
            'longitude': 3.13123,
            'limit': 4,
            'client_ip': None,
            'department': None,
            'filters': None,
        }
        self.assertEqual(resp, should)

    def test_latitude_only(self):
        q = (
            'q=mo\xc3\xb1o&platform=desktop&country=ar&limit=4&locale=es_AR'
            '&latitude=1.2312'
        )
        msg = "latitude and longitude must be paired, both present or none"
        with self.assertRaises(ValueError) as cm:
            self.app._get_search_args({'QUERY_STRING': q})
            self.assertEqual(str(cm.exception), msg)

    def test_longitude_only(self):
        q = (
            'q=mo\xc3\xb1o&platform=desktop&country=ar&limit=4&locale=es_AR'
            '&longitude=1.2312'
        )
        msg = "latitude and longitude must be paired, both present or none"
        with self.assertRaises(ValueError) as cm:
            self.app._get_search_args({'QUERY_STRING': q})
            self.assertEqual(str(cm.exception), msg)

    def test_no_locale_no_country_no_limit(self):
        q = 'q=mo\xc3\xb1o&platform=desktop'
        resp = self.app._get_search_args({'QUERY_STRING': q})
        should = {
            'query': u'moño',
            'session_id': None,
            'query_id': None,
            'locale': utils.DEFAULT_LOCALE,
            'platform': 'desktop',
            'country': scopes.DEFAULT_COUNTRY,
            'latitude': None,
            'longitude': None,
            'limit': scopes.DEFAULT_LIMIT,
            'client_ip': None,
            'department': None,
            'filters': None,
        }
        self.assertEqual(resp, should)

    def test_bad_locale(self):
        q = 'q=mo\xc3\xb1o&platform=desktop&country=ar&limit=4&locale=es|AR'
        with self.assertRaises(ValueError) as cm:
            self.app._get_search_args({'QUERY_STRING': q})
            self.assertEqual(str(cm.exception), 'Bad locale')

    def test_locale_C(self):
        q = 'q=mo\xc3\xb1o&platform=desktop&country=ar&limit=4&locale=C'
        resp = self.app._get_search_args({'QUERY_STRING': q})
        should = {
            'query': u'moño',
            'session_id': None,
            'query_id': None,
            'locale': 'en_US',
            'platform': 'desktop',
            'country': 'ar',
            'latitude': None,
            'longitude': None,
            'limit': 4,
            'client_ip': None,
            'department': None,
            'filters': None,
        }
        self.assertEqual(resp, should)

    def test_empty_query(self):
        q = 'q=&platform=desktop&country=ar&limit=4&locale=es_AR'
        resp = self.app._get_search_args({'QUERY_STRING': q})
        should = {
            'query': u'',
            'session_id': None,
            'query_id': None,
            'locale': 'es_AR',
            'platform': 'desktop',
            'country': 'ar',
            'latitude': None,
            'longitude': None,
            'limit': 4,
            'client_ip': None,
            'department': None,
            'filters': None,
        }
        self.assertEqual(resp, should)

    def test_query_decode(self):
        q = 'q=cami%C3%B3n&platform=desktop&country=ar&limit=4&locale=es_AR'
        resp = self.app._get_search_args({'QUERY_STRING': q})
        should = {
            'query': u'camión',
            'session_id': None,
            'query_id': None,
            'locale': 'es_AR',
            'platform': 'desktop',
            'country': 'ar',
            'latitude': None,
            'longitude': None,
            'limit': 4,
            'client_ip': None,
            'department': None,
            'filters': None,
        }
        self.assertEqual(resp, should)

    def test_with_client_ip(self):
        q = 'q=foo&platform=desktop&country=ar&limit=4&locale=es_AR'
        resp = self.app._get_search_args(
            {'QUERY_STRING': q,
             'HTTP_X_FORWARDED_FOR': '192.168.1.1'})
        should = {
            'query': u'foo',
            'session_id': None,
            'query_id': None,
            'locale': 'es_AR',
            'platform': 'desktop',
            'country': 'ar',
            'latitude': None,
            'longitude': None,
            'limit': 4,
            'client_ip': '192.168.1.1',
            'department': None,
            'filters': None,
        }
        self.assertEqual(resp, should)

    def test_default_limit_override(self):
        q = 'q=foo&platform=desktop'
        self.app.default_limit = 25
        resp = self.app._get_search_args({'QUERY_STRING': q})
        should = {
            'query': u'foo',
            'session_id': None,
            'query_id': None,
            'platform': 'desktop',
            'locale': 'en',
            'country': 'US',
            'latitude': None,
            'longitude': None,
            'limit': 25,
            'client_ip': None,
            'department': None,
            'filters': None,
        }
        self.assertEqual(resp, should)

    def test_department(self):
        q = 'q=foo&platform=desktop&department=shoes'
        resp = self.app._get_search_args({'QUERY_STRING': q})
        should = {
            'query': u'foo',
            'session_id': None,
            'query_id': None,
            'platform': 'desktop',
            'locale': 'en',
            'country': 'US',
            'latitude': None,
            'longitude': None,
            'limit': 10,
            'client_ip': None,
            'department': 'shoes',
            'filters': None,
        }
        self.assertEqual(resp, should)

    def test_filters(self):
        q = 'q=foo&platform=desktop&filters=%7B%22f1%22%3A%5B%22o1%22%5D%7D%0A'
        resp = self.app._get_search_args({'QUERY_STRING': q})
        should = {
            'query': u'foo',
            'session_id': None,
            'query_id': None,
            'platform': 'desktop',
            'locale': 'en',
            'country': 'US',
            'latitude': None,
            'longitude': None,
            'limit': 10,
            'client_ip': None,
            'department': None,
            'filters': {"f1": ["o1"]},
        }
        self.assertEqual(resp, should)


class PreviewParamsValidationTestCase(TestCase):
    """Test the validation of parameters needed in the preview."""

    def setUp(self):
        self.app = scopes.SimpleApp(config={})
        self.prev_res_dict = dict(example='foo', stuff=23)
        self.prev_res = urllib.quote(json.dumps(self.prev_res_dict))

    def test_simple_ok(self):
        q = (
            'platform=desktop&locale=es_AR&country=ar&widgets_api_version=1'
            '&result=' + self.prev_res
        )
        resp = self.app._get_preview_args({'QUERY_STRING': q})
        should = {
            'client_ip': None,
            'session_id': None,
            'locale': 'es_AR',
            'platform': 'desktop',
            'widgets_api_version': 1,
            'country': 'ar',
            'latitude': None,
            'longitude': None,
            'result': self.prev_res_dict,
        }
        self.assertEqual(resp, should)

    def test_latitude_longitude(self):
        q = (
            'platform=desktop&locale=es_AR&country=ar&widgets_api_version=1'
            '&latitude=1.2312&longitude=3.13123&result=' + self.prev_res
        )
        resp = self.app._get_preview_args({'QUERY_STRING': q})
        should = {
            'client_ip': None,
            'session_id': None,
            'locale': 'es_AR',
            'platform': 'desktop',
            'widgets_api_version': 1,
            'country': 'ar',
            'latitude': 1.2312,
            'longitude': 3.13123,
            'result': self.prev_res_dict,
        }
        self.assertEqual(resp, should)

    def test_latitude_only(self):
        q = (
            'platform=desktop&locale=es_AR&country=ar&widgets_api_version=1'
            '&latitude=1.2312&result=' + self.prev_res
        )
        msg = "latitude and longitude must be paired, both present or none"
        with self.assertRaises(ValueError) as cm:
            self.app._get_preview_args({'QUERY_STRING': q})
            self.assertEqual(str(cm.exception), msg)

    def test_longitude_only(self):
        q = (
            'platform=desktop&locale=es_AR&country=ar&widgets_api_version=1'
            '&longitude=1.2312&result=' + self.prev_res
        )
        msg = "latitude and longitude must be paired, both present or none"
        with self.assertRaises(ValueError) as cm:
            self.app._get_preview_args({'QUERY_STRING': q})
            self.assertEqual(str(cm.exception), msg)

    def test_no_locale_no_country(self):
        q = 'platform=desktop&widgets_api_version=1&result=' + self.prev_res
        resp = self.app._get_preview_args({'QUERY_STRING': q})
        should = {
            'client_ip': None,
            'session_id': None,
            'locale': utils.DEFAULT_LOCALE,
            'platform': 'desktop',
            'widgets_api_version': 1,
            'country': scopes.DEFAULT_COUNTRY,
            'latitude': None,
            'longitude': None,
            'result': self.prev_res_dict,
        }
        self.assertEqual(resp, should)

    def test_bad_locale(self):
        q = (
            'platform=desktop&locale=es|AR&country=ar&widgets_api_version=1'
            '&result=' + self.prev_res
        )
        with self.assertRaises(ValueError) as cm:
            self.app._get_preview_args({'QUERY_STRING': q})
            self.assertEqual(str(cm.exception), 'Bad locale')

    def test_locale_C(self):
        q = (
            'platform=desktop&locale=C&country=ar&widgets_api_version=1'
            '&result=' + self.prev_res
        )
        resp = self.app._get_preview_args({'QUERY_STRING': q})
        should = {
            'client_ip': None,
            'session_id': None,
            'locale': 'en_US',
            'platform': 'desktop',
            'widgets_api_version': 1,
            'country': 'ar',
            'latitude': None,
            'longitude': None,
            'result': self.prev_res_dict,
        }
        self.assertEqual(resp, should)

    def test_broken_prev_result(self):
        q = (
            'platform=desktop&locale=es_AR&country=ar&widgets_api_version=1'
            '&result=|no|json|'
        )
        with self.assertRaises(ValueError) as cm:
            self.app._get_preview_args({'QUERY_STRING': q})
            self.assertEqual(str(cm.exception), 'Broken previous result')

    def test_decode_prev_result(self):
        prev_res_dict = dict(example=u'moño', stuff=23)
        prev_res = urllib.quote(json.dumps(prev_res_dict))
        q = (
            'platform=desktop&locale=es_AR&country=ar&widgets_api_version=1'
            '&result=' + prev_res
        )
        resp = self.app._get_preview_args({'QUERY_STRING': q})
        should = {
            'session_id': None,
            'locale': 'es_AR',
            'platform': 'desktop',
            'widgets_api_version': 1,
            'country': 'ar',
            'latitude': None,
            'longitude': None,
            'result': prev_res_dict,
            'client_ip': None,
        }
        self.assertEqual(resp, should)

    def test_with_client_ip(self):
        q = (
            'platform=desktop&locale=es_AR&country=ar&widgets_api_version=1'
            '&result=' + self.prev_res
        )
        resp = self.app._get_preview_args(
            {'QUERY_STRING': q,
             'HTTP_X_FORWARDED_FOR': '192.168.1.1'})
        should = {
            'client_ip': '192.168.1.1',
            'session_id': None,
            'locale': 'es_AR',
            'platform': 'desktop',
            'widgets_api_version': 1,
            'country': 'ar',
            'latitude': None,
            'longitude': None,
            'result': self.prev_res_dict,
        }
        self.assertEqual(resp, should)


class HelpersTestCase(TestCase):
    """Test some helper methods."""

    def setUp(self):
        self.app = scopes.SimpleApp(config={})
        self.start_response = Mock()

    def test_error_sanity(self):
        resp = self.app.error("400 test", self.start_response,
                              "test message")
        should_error_message = json.dumps(
            {'error': '400 test', 'error_msg': 'test message'}, indent=4)

        self.assertEqual(resp, [should_error_message])
        should_status = "400 test"
        should_headers = [
            ('Content-type', "application/json"),
            ('Content-Length', str(len(should_error_message))),
        ]
        self.start_response.assert_called_with(should_status, should_headers)
