# -*- coding: utf8 -*-

# Copyright 2014 Canonical
# All Rights Reserved

"""Tests for the Simple App base stuff."""

import urlparse

from unittest import TestCase
from mock import patch

from src.scopes import ebay
from src.scopes.tests import get_fixture

CONFIG = {'secrets': {}}

SEARCH_URI = ('https://svcs.ebay.com/services/search/FindingService/v1'
              '?SECURITY-APPNAME=None&OPERATION-NAME=findItemsByKeywords'
              '&SERVICE-VERSION=1.0.0&RESPONSE-DATA-FORMAT=JSON'
              '&GLOBAL-ID=EBAY-US&REST-PAYLOAD'
              '&keywords=paris&paginationInput.entriesPerPage=10'
              '&outputSelector(0)=PictureURLSuperSize'
              '&outputSelector(1)=PictureURLLarge')

SURFACING_URI = ('http://open.api.ebay.com/shopping?callname=FindPopularItems'
                 '&responseencoding=JSON&appid=None&siteid=0&version=531&'
                 'CategoryID=550&MaxEntries=10')


class SearchTestCase(TestCase):
    """Tests for the search function."""

    def test_product(self):
        app = ebay.App(CONFIG)
        with patch.object(ebay, '_get_products') as mock:
            with patch.object(ebay, '_get_available_departments') as mdep:
                mdep.return_value = {}
                mock.return_value = get_fixture('ebay-results.json')
                response = list(app.search(query='paris', limit=10,
                                           platform='desktop', filters=None,
                                           locale='C', country='us',
                                           department=''))

        # got the info as it should
        mock.assert_called_with(SEARCH_URI)

        # category
        category_mock = ebay.PRODUCTS_CATEGORY.copy()
        category_mock['title'] = '271775 results'
        should_cat = {'category': category_mock}
        self.assertEqual(response[0], should_cat)

        # the result
        should_res = {'result': {
            'condition': u'Pre-owned',
            'currency_code': u'USD',
            'art': u'http://thumbs3.ebaystatic.com/m/' +
            'mA8ILqmlSWokT6vyEN1KFyw/140.jpg',
            'cat_id': 'cat_0',
            'dnd_uri': u'http://www.ebay.com/itm/Paris-Eiffel-' +
            'Tower-Hook-Black-Pink-EUC-/321313863006?pt=Decals' +
            '_Stickers_Vinyl_Art',
            'image_hint': u'http://i.ebayimg.com/00/s/MTAwMFg3NTA=' +
            '/z/a4cAAOxyVLNS4Ts7/$_3.JPG',
            'payment_methods': u'PayPal',
            'price': u'USD 4.0',
            'shipping_to': u'US',
            'title': u'Paris Eiffel Tower Hook Black/Pink EUC',
            'uri': u'http://www.ebay.com/itm/Paris-Eiffel-Tower-Hook-' +
            'Black-Pink-EUC-/321313863006?pt=Decals_Stickers_Vinyl_Art',
            'item_id': u'321313863006',
            'attributes': [{'display_hints': 'default',
                            'style': 'default',
                            'value': '1h 30m'}],
        }}
        self.maxDiff = None
        self.assertEqual(response[1], should_res)

    def test_surfacing(self):
        app = ebay.App(CONFIG)
        with patch.object(ebay, '_get_products') as mock:
            with patch.object(ebay, '_get_surfacing_department') as dep_mock:
                with patch.object(ebay, '_get_available_departments') as mdep:
                    mdep.return_value = {}
                    mock.return_value = get_fixture('ebay-surfacing.json')
                    dep_mock.return_value = ('550', 'Foo')
                    response = list(app.search(query='', limit=10,
                                               platform='desktop',
                                               locale='C', country='us',
                                               department='', filters=None))

        # got the info as it should
        mock.assert_called_with(SURFACING_URI)

        # category
        self.assertEqual(response[1]['category']['title'], "Featured in Foo")

        # the result
        should_res = {'result': {
            'condition': None,
            'currency_code': u'USD',
            'art': u'http://thumbs2.ebaystatic.com/d/l150/pict/' +
            '350682346349808000000001_11.jpg',
            'cat_id': 'cat_1',
            'dnd_uri': u'http://www.ebay.com/itm/Every-Day-Carry-Tactical-' +
            'Assault-Bag-EDC-Day-Pack-Backpack-Molle-Webbing-/' +
            '350682346349?var=620083903946',
            'image_hint': u'http://thumbs2.ebaystatic.com/d/l300/pict/' +
            '350682346349808000000001_11.jpg',
            'payment_methods': None,
            'price': u'USD 36.92',
            'shipping_to': None,
            'title': u'Every Day Carry Tactical Assault Bag EDC Day ' +
            'Pack Backpack with Molle Webbing',
            'uri': u'http://www.ebay.com/itm/Every-Day-Carry-Tactical-' +
            'Assault-Bag-EDC-Day-Pack-Backpack-Molle-Webbing-/' +
            '350682346349?var=620083903946',
            'item_id': u'350682346349',
            'attributes': [{'display_hints': 'default',
                            'style': 'default',
                            'value': '4d 2h'}],

        }}
        self.maxDiff = None
        self.assertEqual(response[2], should_res)

    def test_surfacing_no_img(self):
        app = ebay.App(CONFIG)
        with patch.object(ebay, '_get_products') as mock:
            with patch.object(ebay, '_get_surfacing_department') as dep_mock:
                with patch.object(ebay, '_get_available_departments') as mdep:
                    mdep.return_value = {}
                    mock.return_value = get_fixture(
                        'ebay-surfacing-no-img.json')
                    dep_mock.return_value = ('550', 'Bar')
                    response = list(app.search(query='', limit=10,
                                               platform='desktop',
                                               locale='C', country='us',
                                               department='', filters=None))

        # got the info as it should
        mock.assert_called_with(SURFACING_URI)

        # category
        self.assertEqual(response[1]['category']['title'], "Featured in Bar")

        # Ensure items without images are not surfaced
        self.assertEqual(len(response), 2)

    def test_surfacing_with_department(self):
        app = ebay.App(CONFIG)
        with patch.object(ebay, '_get_products') as mock:
            with patch.object(ebay, '_get_available_departments') as mdep:
                mdep.return_value = {}
                mock.return_value = get_fixture('ebay-surfacing.json')
                list(app.search(query='', limit=10, filters=None,
                                platform='desktop', department='550',
                                locale='C', country='us'))

        # Fake a surfacing by department uri
        uri = ('http://open.api.ebay.com/shopping?callname=FindPopularItems'
               '&responseencoding=JSON&appid=None&siteid=0&version=531&'
               'CategoryID=550&MaxEntries=10')
        mock.assert_called_with(uri)

    def test_filters(self):
        filters = {"MaxPrice": 5000,
                   "MinPrice": [10, 30],
                   "foo": "bar"}
        should_filters = {"itemFilter(0).name": ["MinPrice"],
                          "itemFilter(1).value(0)": ['5000'],
                          "itemFilter(1).name": ["MaxPrice"],
                          "itemFilter(0).value(0)": ['10'],
                          "itemFilter(0).value(1)": ['30'], }
        validate = ebay._parse_filters(filters)

        # Ensure only valid Ebay filters are kept and correctly parsed
        self.assertEqual(should_filters, urlparse.parse_qs(validate))

    def test_valid_country(self):
        app = ebay.App(CONFIG)
        with patch.object(ebay, '_get_products') as mock:
            with patch.object(ebay, '_get_available_departments') as mdep:
                mdep.return_value = {}
                mock.return_value = get_fixture('ebay-surfacing.json')
                list(app.search(query='', limit=10, filters=None,
                                platform='desktop', department='550',
                                locale='C', country='fr'))

        # Fake a surfacing by department uri
        uri = ('http://open.api.ebay.com/shopping?callname=FindPopularItems'
               '&responseencoding=JSON&appid=None&siteid=71&version=531&'
               'CategoryID=550&MaxEntries=10')
        mock.assert_called_with(uri)

    def test_invalid_country(self):
        app = ebay.App(CONFIG)
        with patch.object(ebay, '_get_products') as mock:
            with patch.object(ebay, '_get_available_departments') as mdep:
                mdep.return_value = {}
                mock.return_value = get_fixture('ebay-surfacing.json')
                list(app.search(query='', limit=10, filters=None,
                                platform='desktop', department='550',
                                locale='C', country='foo'))

        # Fake a surfacing by department uri
        uri = ('http://open.api.ebay.com/shopping?callname=FindPopularItems'
               '&responseencoding=JSON&appid=None&siteid=0&version=531&'
               'CategoryID=550&MaxEntries=10')
        mock.assert_called_with(uri)

    def test_valid_country_with_multiple_locales(self):
        app = ebay.App(CONFIG)
        with patch.object(ebay, '_get_products') as mock:
            with patch.object(ebay, '_get_available_departments') as mdep:
                mdep.return_value = {}
                mock.return_value = get_fixture('ebay-surfacing.json')
                list(app.search(query='', limit=10, filters=None,
                                platform='desktop', department='550',
                                locale='fr_BE', country='be'))

        # Fake a surfacing by department uri
        uri = ('http://open.api.ebay.com/shopping?callname=FindPopularItems'
               '&responseencoding=JSON&appid=None&siteid=23&version=531&'
               'CategoryID=550&MaxEntries=10')
        mock.assert_called_with(uri)


class PreviewTestCase(TestCase):
    """Tests for the preview function."""

    def test_preview(self):
        # fake a previous result
        result = {
            'condition': u'Pre-owned',
            'currency_code': u'USD',
            'art': u'http://thumbs3.ebaystatic.com/m/' +
            'mA8ILqmlSWokT6vyEN1KFyw/140.jpg',
            'cat_id': 'cat_0',
            'dnd_uri': u'http://www.ebay.com/itm/Paris-Eiffel-' +
            'Tower-Hook-Black-Pink-EUC-/321313863006?pt=Decals' +
            '_Stickers_Vinyl_Art',
            'image_hint': u'http://i.ebayimg.com/00/s/MTAwMFg3NTA=' +
            '/z/a4cAAOxyVLNS4Ts7/$_3.JPG',
            'payment_methods': u'PayPal',
            'price': u'4.0',
            'shipping_to': u'US',
            'title': u'Paris Eiffel Tower Hook Black/Pink EUC',
            'uri': u'http://www.ebay.com/itm/Paris-Eiffel-Tower-Hook-' +
            'Black-Pink-EUC-/321313863006?pt=Decals_Stickers_Vinyl_Art',
            'item_id': 'foo',
        }
        app = ebay.App(CONFIG)
        with patch.object(ebay, '_get_products') as mock:
            mock.return_value = get_fixture('ebay-iteminfo.json')
            response = list(app.preview(result=result, locale='C'))

        # check the 'image'
        should = dict(widget={
            'id': 'imagegallery',
            'type': 'gallery',
            'sources': [(u'http://i.ebayimg.com/00/s/NTMxWDUzMQ==/'
                         'z/hlwAAOxydlFSuFyz/$_1.JPG?set_id=880000500F')],
        })
        self.assertEqual(response[0], should)

        # check the 'header'
        should = dict(widget={
            'id': 'header',
            'type': 'header',
            'title': u'Paris Eiffel Tower Hook Black/Pink EUC',
        })
        self.assertEqual(response[1], should)

        # check the 'top_info' widget
        should = dict(widget={
            'id': 'top_info',
            'type': 'text',
            'text': u'Current bid: 4.0\nShipping: USD 0.99',
            'title': '',
        })
        self.assertEqual(response[2], should)

        # check the 'actions'
        should = dict(widget={
            'id': 'buttons',
            'type': 'actions',
            'actions': [{'id': 'action1',
                         'uri': 'http://www.ebay.com/itm/Paris-Eiffel-' +
                         'Tower-Hook-Black-Pink-EUC-/321313863006?pt=' +
                         'Decals_Stickers_Vinyl_Art',
                         'label': 'View on eBay'}],
        })
        self.assertEqual(response[3], should)

        # check the 'info'
        should = dict(widget={
            'id': 'info',
            'type': 'text',
            'title': 'Info',
            'text': ('Time left: 26d 22h\nItem location: '
                     'West Palm Beach, Florida\nCondition: Pre-owned'),
        })
        self.assertEqual(response[4], should)

        # check the 'description'
        should_desc = 'Brand new set of all 5 books'
        self.assertTrue(response[5]['widget']['text'].startswith(should_desc))

        # check the 'seller'
        should = dict(widget={
            'id': 'seller',
            'type': 'text',
            'title': '',
            'text': ('Sold by treebeardbooks (117300)\n'
                     'Feedback: 99.9% positive'),
        })
        self.assertEqual(response[6], should)

        # check the 'shipping and postage'
        should = dict(widget={
            'id': 'shipping_and_postage',
            'type': 'text',
            'title': 'Shipping and postage',
            'text': ('Postage\nItem location: West Palm Beach, Florida\n'
                     'Ships to: US, CA, GB, DK, RO, SK, BG, CZ, '
                     'FI, HU, LV, LT, MT, EE, AU, GR, PT, CY, SI, '
                     'JP, CN, SE, KR, TW, ZA, TH, BE, FR, HK, IE, NL, '
                     'PL, ES, IT, DE, AT, RU, IL, MX, NZ, PH, SG, NO, '
                     'SA, UA, AE, QA, KW, BH, HR, MY, TR\n\nPackaging\n'
                     'Dispatch time: 1 business day\n\nPayment details\n'
                     'Payment method: PayPal\n\nReturns policy\n'
                     'Policy: Returns accepted'
                     '\nRefund: Money back\nReturns within: 14 days\n\n'
                     'There is a restocking fee of $5 which covers our '
                     'cost of shipping the item to you.  The set must not '
                     'be opened and still sealed.'),
        })
        self.maxDiff = None
        self.assertEqual(response[7], should)


class DepartmentsTestCase(TestCase):
    """Tests for the departments tree."""

    def test_parent_in_tree(self):
        with patch.object(ebay, '_get_products') as mock:
            mock.return_value = get_fixture('ebay-depinfo.json')
            deps = ebay._get_available_departments('77933', 16, '', '')

        self.assertEqual(deps['canned_query'], 'scope://ebay?dep=267')
        self.assertEqual(deps['label'], 'Livres, BD, revues')
        self.assertEqual(len(deps['subdepartments']), 1)

    def test_current_in_tree(self):
        with patch.object(ebay, '_get_products') as mock:
            mock.return_value = get_fixture('ebay-depinfo.json')
            deps = ebay._get_available_departments('77933', 16, '', '')

        self.assertEqual(deps['subdepartments'][0]['canned_query'],
                         'scope://ebay?dep=77933')
        self.assertEqual(deps['subdepartments'][0]['label'],
                         u'Romans, littérature')
        self.assertEqual(len(deps['subdepartments'][0]['subdepartments']), 21)

    def test_root_canned_query(self):
        with patch.object(ebay, '_get_products') as mock:
            mock.return_value = get_fixture('ebay-root-depinfo.json')
            deps = ebay._get_available_departments('', 16, '', '')

        self.assertEqual(deps['canned_query'],
                         'scope://ebay')


class SortingTestCase(TestCase):
    """Tests for sorting filters."""

    def test_filters_response(self):
        app = ebay.App(CONFIG)
        filter_state = {"sorting_filter": ["PricePlusShippingLowest"]}
        with patch.object(ebay, '_get_products') as mock:
            with patch.object(ebay, '_get_available_departments') as mdep:
                mdep.return_value = {}
                mock.return_value = get_fixture('ebay-results-sorting.json')
                response = list(app.search(query='', limit=1,
                                           platform='desktop',
                                           locale='C', country='us',
                                           department='267',
                                           filters=filter_state))

        self.assertEqual(len(response), 9)
        self.assertEqual(response[3]['filter_state'], filter_state)
        self.assertEqual(len(response[1]['filters'][0]['options']), 8)

    def test_filters_canned_query(self):
        filter_state = {"sorting_filter": ["PricePlusShippingLowest"]}
        with patch.object(ebay, '_get_products') as mock:
            mock.return_value = get_fixture('ebay-root-depinfo.json')
            deps = ebay._get_available_departments('', 16, '', '',
                                                   filters=filter_state)

        expected_cq = ('scope://ebay?filters=%7B%22sorting_filter'
                       '%22%3A%20%5B%22PricePlusShippingLowest%22%5D%7D')
        self.assertEqual(deps['canned_query'], expected_cq)


class TimeTestCase(TestCase):
    """Tests for time parsing."""

    def test_time_left(self):
        # Invalid time
        time = 'foo'
        t_should = None
        self.assertEqual(ebay._parse_time_left(time, None), t_should)

        # d, h
        time = 'P2DT23H32M51S'
        t_should = '2d 23h'
        self.assertEqual(ebay._parse_time_left(time, None), t_should)

        # h, m
        time = 'PT23H32M51S'
        t_should = '23h 32m'
        self.assertEqual(ebay._parse_time_left(time, None), t_should)

        # m
        time = 'P0DT0H32M'
        t_should = '32m'
        self.assertEqual(ebay._parse_time_left(time, None), t_should)

        # d
        time = 'P1DT0H32M'
        t_should = '1d'
        self.assertEqual(ebay._parse_time_left(time, None), t_should)

        # d, h
        time = 'P5DT5H0M'
        t_should = '5d 5h'
        self.assertEqual(ebay._parse_time_left(time, None), t_should)


class ShippingPostagePreviewTestCase(TestCase):
    """Tests for Postage and shipping preview text building."""

    def test_postage_only(self):
        details = {'location': 'A',
                   'ships_to': 'B',
                   'handling_time': None,
                   'payment_methods': None,
                   'return_policy': None}
        text = ebay._build_shipping_and_postage_text(details, '')
        should_text = 'Postage\nItem location: A\nShips to: B'
        self.assertEqual(text, should_text)

    def test_packaging_only(self):
        details = {'location': None,
                   'ships_to': None,
                   'handling_time': 1,
                   'payment_methods': None,
                   'return_policy': None}
        text = ebay._build_shipping_and_postage_text(details, '')
        should_text = 'Packaging\nDispatch time: 1 business day'
        self.assertEqual(text, should_text)

    def test_payment_only(self):
        details = {'location': None,
                   'ships_to': None,
                   'handling_time': None,
                   'payment_methods': 'Foo, Bar',
                   'return_policy': None}
        text = ebay._build_shipping_and_postage_text(details, '')

        should_text = 'Payment details\nPayment method: Foo, Bar'
        self.assertEqual(text, should_text)

    def test_returns_only(self):
        details = {'location': None,
                   'ships_to': None,
                   'handling_time': None,
                   'payment_methods': None,
                   'return_policy': {'ReturnsAccepted': 'Maybe',
                                     'Refund': 'Certainly Not'}}
        text = ebay._build_shipping_and_postage_text(details, '')
        should_text = 'Returns policy\nPolicy: Maybe\nRefund: Certainly not'
        self.assertEqual(text, should_text)

    def test_postage_n_returns_only(self):
        details = {'location': 'A',
                   'ships_to': None,
                   'handling_time': None,
                   'payment_methods': None,
                   'return_policy': {'ReturnsAccepted': 'Returns accepted',
                                     'Description': None}}
        text = ebay._build_shipping_and_postage_text(details, '')
        should_text = ('Postage\nItem location: A\n\nReturns policy\n'
                       'Policy: Returns accepted')
        self.assertEqual(text, should_text)


class CategoriesTestCase(TestCase):
    """Tests for building categories."""

    def test_counter_in_title(self):
        category = ebay._build_counted_cat(ebay.PRODUCTS_CATEGORY, '', None)
        self.assertEqual(category['title'], 'Results')

        category = ebay._build_counted_cat(ebay.PRODUCTS_CATEGORY, '', 1)
        self.assertEqual(category['title'], '1 result')

        category = ebay._build_counted_cat(ebay.PRODUCTS_CATEGORY, '', 2)
        self.assertEqual(category['title'], '2 results')
