# -*- coding: utf8 -*-

# Copyright 2014 Canonical
# All Rights Reserved

"""Tests for the Simple App base stuff."""

import json

from unittest import TestCase

from mock import patch

from src.scopes import grooveshark
from src.scopes.tests import get_fixture

CONFIG = {'secrets': {}}


class SearchTestCase(TestCase):
    """Tests for the search function."""

    def test_song_with_query(self):
        app = grooveshark.App(CONFIG)
        with patch.object(grooveshark, 'api_call') as mock:
            mock.return_value = json.loads(
                get_fixture('grooveshark-song-album.json'))
            response = list(app.search(query='q', limit=10,
                                       platform='desktop', locale='C'))

        # got the info as it should
        mock.assert_called_with('getSongSearchResults',
                                {'secret': None, 'key': None},
                                country=grooveshark.DEFAULT_COUNTRY,
                                client_ip='173.194.42.19',
                                limit=10, query='q')

        # category
        should_cat = {'category': grooveshark.SONGS_CATEGORY}
        self.assertEqual(response[0], should_cat)
        should_cat = {'category': grooveshark.ALBUMS_CATEGORY}
        self.assertEqual(response[1], should_cat)

        # the result
        should_res = {'result': {
            'album': u'Fräns Wild Years',
            'song_id': '37510211',
            'art': 'http://beta.grooveshark.com/static/amazonart/m186211.jpg',
            'title': u'Yesterday Is Here',
            'form_factor': 'phone',
            'cat_id': 'cat_0',
            'artist': u'Tom Waits',
            'album_id': '186211',
            'uri': 'http://html5.grooveshark.com/#/album/' +
            'Fr%C3%A4ns%20Wild%20Years/186211',
        }}
        self.assertEqual(response[3], should_res)

    def test_surfacing(self):
        app = grooveshark.App(CONFIG)
        with patch.object(grooveshark, 'api_call') as mock:
            mock.return_value = json.loads(
                get_fixture('grooveshark-song-album.json'))
            response = list(app.search(query='', limit=10,
                                       platform='desktop', locale='C'))

        # got the info as it should
        mock.assert_called_with('getPopularSongsMonth',
                                {'secret': None, 'key': None},
                                client_ip='173.194.42.19',
                                limit=10)

        # category
        should_cat = {'category': grooveshark.SONGS_CATEGORY_SMALL}
        self.assertEqual(response[0], should_cat)
        should_cat = {'category': grooveshark.ALBUMS_CATEGORY_SMALL}
        self.assertEqual(response[1], should_cat)

        # the result
        should_res = {'result': {
            'album': u'Fräns Wild Years',
            'song_id': '37510211',
            'art': 'http://beta.grooveshark.com/static/amazonart/m186211.jpg',
            'title': u'Yesterday Is Here',
            'form_factor': 'phone',
            'cat_id': 'cat_0',
            'artist': u'Tom Waits',
            'album_id': '186211',
            'uri': 'http://html5.grooveshark.com/#/album/' +
            'Fr%C3%A4ns%20Wild%20Years/186211',
        }}
        self.assertEqual(response[3], should_res)

    def test_album(self):
        app = grooveshark.App(CONFIG)
        with patch.object(grooveshark, 'api_call') as mock:
            mock.return_value = json.loads(
                get_fixture('grooveshark-song-album.json'))
            response = list(app.search(query='Franks Wild Years', limit=10,
                                       platform='desktop', locale='C'))

        # got the info as it should
        mock.assert_called_with('getSongSearchResults',
                                {'secret': None, 'key': None},
                                country={'DMA': 0,
                                         'CC4': 0,
                                         'CC1': 0,
                                         'CC3': 0,
                                         'CC2': 0,
                                         'IPR': 0,
                                         'ID': 0
                                         },
                                client_ip='173.194.42.19',
                                limit=10, query='Franks Wild Years')

        # category
        should_cat = {'category': grooveshark.SONGS_CATEGORY}
        self.assertEqual(response[0], should_cat)
        should_cat = {'category': grooveshark.ALBUMS_CATEGORY}
        self.assertEqual(response[1], should_cat)

        # the result
        should_res = {'result': {
            'album': u'Franks Wild Years',
            'art': 'http://beta.grooveshark.com/static/amazonart/m186211.jpg',
            'title': u'Franks Wild Years',
            'form_factor': 'phone',
            'cat_id': 'cat_1',
            'artist': u'Tom Waits',
            'album_id': '186211',
            'uri': 'http://html5.grooveshark.com/#/album/' +
            'Franks%20Wild%20Years/186211',
            'album_uri': 'http://html5.grooveshark.com/#/album/' +
            'Franks%20Wild%20Years/186211',
        }}
        self.assertEqual(response[2], should_res)

    def test_get_client_ip(self):
        app = grooveshark.App(CONFIG)
        with patch.object(grooveshark, 'api_call') as mock:
            list(app.search(query='q', limit=10, platform='desktop',
                            client_ip='192.168.1.1', locale='C'))

        # got the info as it should
        mock.assert_called_with('getSongSearchResults',
                                {'secret': None, 'key': None},
                                country={'DMA': 0,
                                         'CC4': 0,
                                         'CC1': 0,
                                         'CC3': 0,
                                         'CC2': 0,
                                         'IPR': 0,
                                         'ID': 0
                                         },
                                client_ip='192.168.1.1',
                                limit=10, query='q')


class PreviewTestCase(TestCase):
    """Tests for the preview function."""

    def test_preview(self):
        # fake a previous result
        result = {
            'album': 'Franks Wild Years',
            'art': 'http://beta.grooveshark.com/static/amazonart/m186211.jpg',
            'title': 'Franks Wild Years',
            'form_factor': 'desktop',
            'cat_id': 'cat_1',
            'artist': 'Tom Waits',
            'album_id': '186211',
            'uri': 'http://grooveshark.com/#/album/' +
            'Franks%20Wild%20Years/186211',
            'album_uri': 'http://grooveshark.com/#/album/' +
            'Franks%20Wild%20Years/186211',
        }
        app = grooveshark.App(CONFIG)
        response = list(app.preview(result=result, locale='C'))

        # check the 'image'
        should = dict(widget={
            'id': 'art',
            'type': 'image',
            'source': 'http://beta.grooveshark.com/' +
            'static/amazonart/m186211.jpg',
        })
        self.assertEqual(response[0], should)

        # check the 'header'
        should = dict(widget={
            'id': 'header',
            'type': 'header',
            'title': 'Franks Wild Years',
            'art': 'http://beta.grooveshark.com/static/amazonart/m186211.jpg',
            'subtitle': 'Tom Waits',
        })
        self.assertEqual(response[1], should)

        # check the 'actions'
        should = dict(widget={
            'id': 'buttons',
            'type': 'actions',
            'actions': [{'label': 'Play in Grooveshark',
                         'id': 'action1',
                         'uri': 'http://grooveshark.com/#/album/' +
                         'Franks%20Wild%20Years/186211'}],
        })
        self.assertEqual(response[2], should)

    def test_preview_song_phone(self):
        # fake a previous result
        result = {
            'album': u'Fräns Wild Years',
            'song_id': '37510211',
            'art': 'http://beta.grooveshark.com/static/amazonart/m186211.jpg',
            'title': u'Yesterday Is Here',
            'form_factor': 'desktop',
            'cat_id': 'cat_0',
            'artist': u'Tom Waits',
            'album_id': '186211',
            'uri': 'http://grooveshark.com/#/album/' +
            'Franks%20Wild%20Years/186211',
            'form_factor': 'phone',
        }
        app = grooveshark.App(CONFIG)
        with patch.object(grooveshark, 'api_call') as mock:
            mock.return_value = {"url": "http://grooveshark.com/s/foo"}
            response = list(app.preview(result=result,
                                        client_ip='192.168.1.1', locale='C'))

        # check the 'image'
        should = dict(widget={
            'id': 'art',
            'type': 'image',
            'source': 'http://beta.grooveshark.com/' +
            'static/amazonart/m186211.jpg',
        })
        self.assertEqual(response[0], should)

        # check the 'header'
        should = dict(widget={
            'id': 'header',
            'type': 'header',
            'title': u'Yesterday Is Here',
            'art': 'http://beta.grooveshark.com/static/amazonart/m186211.jpg',
            'subtitle': 'Tom Waits',
        })
        self.assertEqual(response[1], should)

        # check the 'actions'
        should = dict(widget={
            'id': 'buttons',
            'type': 'actions',
            'actions': [{'label': 'Play in Grooveshark',
                         'id': 'action1',
                         'uri': 'http://html5.grooveshark.com/s/foo'}],
        })
        self.assertEqual(response[2], should)

    def test_get_client_ip(self):
        # fake a previous result
        result = {
            'album': u'Fräns Wild Years',
            'song_id': '37510211',
            'art': 'http://beta.grooveshark.com/static/amazonart/m186211.jpg',
            'title': u'Yesterday Is Here',
            'form_factor': 'desktop',
            'cat_id': 'cat_0',
            'artist': u'Tom Waits',
            'album_id': '186211',
            'uri': 'http://grooveshark.com/#/album/' +
            'Fr%C3%A4ns%20Wild%20Years/186211',
        }
        app = grooveshark.App(CONFIG)
        with patch.object(grooveshark, 'api_call') as mock:
            list(app.preview(result=result,
                             client_ip='192.168.1.1', locale='C'))

        # got the info as it should
        mock.assert_called_with('getSongURLFromSongID',
                                {'secret': None, 'key': None},
                                songID='37510211',
                                client_ip='192.168.1.1')
