# -*- coding: utf8 -*-

# Copyright 2014 Canonical
# All Rights Reserved

"""Tests for the Simple App base stuff."""

import json

from unittest import TestCase
from mock import patch

from src.scopes import openlibrary
from src.scopes.tests import get_fixture

CONFIG = {'secrets': {}}

SURFACING_SUBJECTS = ['Love']


class SearchTestCase(TestCase):
    """Tests for the search function."""

    def test_surfacing(self):
        app = openlibrary.App(CONFIG)
        with patch.object(openlibrary, '_get_books') as mock:
            openlibrary.SURFACING_SUBJECTS = SURFACING_SUBJECTS
            mock.return_value = json.loads(get_fixture(
                'openlibrary-subject.json'))
            response = list(app.search(query='', platform='desktop',
                                       limit=app.default_limit, locale='C'))

        # got the info as it should
        mock.assert_called_with('http://openlibrary.org/search.json?'
                                'author=William%20Shakespeare&limit=30')

        # subject category
        should_cat = {'category':
                      {'id': 'books',
                       'title': 'Love',
                       'render_template': json.dumps(
                           openlibrary.RENDERER_BOOK_VERTICAL),
                       'icon': None, }
                      }
        self.assertEqual(response[0], should_cat)

        # author category
        should_cat = {'category':
                      {'id': 'books-by',
                       'title': u'William Shakespeare',
                       'render_template': json.dumps(
                           openlibrary.RENDERER_BOOK_BY),
                       'icon': None, }
                      }
        self.maxDiff = None
        self.assertEqual(response[1], should_cat)

        # the result
        should_res = {'result': {
            'first_published': 1603,
            'cat_id': 'books-by',
            'first_sentence': u'[Enter two Sentinels first, Francisco, who' +
            ' paces up and down at his post; then Bernardo, ' +
            'who approaches him.]',
            'has_fulltext': True,
            'dnd_uri': 'https://openlibrary.org/works/OL15203981W',
            'title': u'The Tragedie of Hamlet, Prince of Denmarke',
            'author': u'William Shakespeare',
            'uri': 'https://openlibrary.org/works/OL15203981W',
            'art': 'http://covers.openlibrary.org/b/id/6743733-M.jpg',
            'editions': u'OLID:OL12203284M,OLID:OL24032549M',
            'OLID': 'OL15203981W',
        }}
        self.maxDiff = None
        self.assertEqual(response[2], should_res)

    def test_pick_author_from_list(self):
        surfacing_data = json.loads(get_fixture(
            'openlibrary-subject.json'))
        author = openlibrary._pick_random_author(surfacing_data['docs'])
        self.assertEqual(author, u'William Shakespeare')

    def test_search(self):
        app = openlibrary.App(CONFIG)
        with patch.object(openlibrary, '_get_books') as mock:
            mock.return_value = json.loads(get_fixture(
                'openlibrary-search.json'))
            response = list(app.search(query='alice', platform='desktop',
                                       limit=app.default_limit, locale='C'))

        # got the info as it should
        mock.assert_called_with('http://openlibrary.org/search.json?' +
                                'q=alice&limit=30')

        # subject category
        cat = openlibrary.BOOKS_CATEGORY
        cat['title'] = 'Books'
        self.assertEqual(response[0], {'category': cat})

        # the result
        should_res = {'result': {
            'first_published': 1865,
            'cat_id': 'books',
            'first_sentence': u'ALICE was beginning to get very tired ' +
            'of sitting by her sister on the bank, and of having nothing ' +
            'to do: once or twice she had peeped into the book her sister ' +
            'was reading, but it had no pictures or conversations in it, ' +
            '"and what is the use of a book," thought Alice, "without ' +
            'pictures or conversations?"',
            'has_fulltext': True,
            'uri': 'https://openlibrary.org/works/OL13101191W',
            'title': u"Alice's Adventures in Wonderland",
            'author': u'Lewis Carroll',
            'uri': 'https://openlibrary.org/works/OL13101191W',
            'dnd_uri': 'https://openlibrary.org/works/OL13101191W',
            'art': 'http://covers.openlibrary.org/b/id/2319784-M.jpg',
            'editions': u'OLID:OL21398313M,OLID:OL25182882M',
            'OLID': 'OL13101191W',
        }}
        self.maxDiff = None
        self.assertEqual(response[2], should_res)


class PreviewTestCase(TestCase):
    """Tests for the preview function."""

    def test_preview_with_editions(self):
        result = {
            'first_published': 1865,
            'cat_id': 'books',
            'first_sentence': u'ALICE was beginning to get very tired ' +
            'of sitting by her sister on the bank, and of having nothing ' +
            'to do: once or twice she had peeped into the book her sister ' +
            'was reading, but it had no pictures or conversations in it, ' +
            '"and what is the use of a book," thought Alice, "without ' +
            'pictures or conversations?"',
            'has_fulltext': True,
            'uri': 'https://openlibrary.org/works/OL13101191W',
            'title': u"Alice's Adventures in Wonderland",
            'author': u'Lewis Carroll',
            'uri': 'https://openlibrary.org/works/OL13101191W',
            'dnd_uri': 'https://openlibrary.org/works/OL13101191W',
            'art': 'http://covers.openlibrary.org/b/id/2319784-M.jpg',
            'editions': u'OLID:OL21398313M,OLID:OL25182882M,' +
            'OLID:OL21398313M,OLID:OL25182882M',
            'OLID': 'OL13101191W',
        }
        app = openlibrary.App(CONFIG)
        with patch.object(openlibrary, '_get_books') as mock:
            mock.return_value = json.loads(get_fixture(
                'openlibrary-edition.json'))
            response = list(app.preview(result=result, locale='C'))

        # check available actions
        should_actions = [{'id': 'view',
                           'label': 'View',
                           'uri': 'https://openlibrary.org/works/OL13101191W'},
                          {'id': 'online',
                           'label': 'Read Online',
                           'uri': u'https://archive.org/stream/' +
                           'alicesadventur1960carr'},
                          {'id': 'pdf',
                           'label': 'PDF',
                           'uri': u'https://archive.org/download/alicesa' +
                           'dventur1960carr/alicesadventur1960carr.pdf'},
                          {'id': 'epub',
                           'label': 'epub',
                           'uri': u'https://archive.org/download/alices' +
                           'adventur1960carr/alicesadventur1960carr.epub'}]
        self.maxDiff = None
        self.assertEqual(response[2]['widget']['actions'], should_actions)

        # check description
        self.assertTrue(response[3]['widget']['text'].startswith(
            "\"ALICE was beginning to get very tired"))

        # check art
        should_art = 'http://covers.openlibrary.org/b/id/2319784-L.jpg'
        self.assertEqual(response[0]['widget']['source'], should_art)

    def test_preview_without_editions(self):
        result = {
            'first_published': 1865,
            'cat_id': 'books',
            'first_sentence': u'ALICE was beginning to get very tired ' +
            'of sitting by her sister on the bank, and of having nothing ' +
            'to do: once or twice she had peeped into the book her sister ' +
            'was reading, but it had no pictures or conversations in it, ' +
            '"and what is the use of a book," thought Alice, "without ' +
            'pictures or conversations?"',
            'has_fulltext': True,
            'uri': 'https://openlibrary.org/works/OL13101191W',
            'title': u"Alice's Adventures in Wonderland",
            'author': u'Lewis Carroll',
            'uri': 'https://openlibrary.org/works/OL13101191W',
            'dnd_uri': 'https://openlibrary.org/works/OL13101191W',
            'art': 'http://covers.openlibrary.org/b/id/2319784-M.jpg',
            'editions': '',
            'OLID': 'OL13101191W',
        }
        app = openlibrary.App(CONFIG)
        with patch.object(openlibrary, '_get_books') as mock:
            mock.return_value = json.loads(get_fixture(
                'openlibrary-work.json'))
            response = list(app.preview(result=result, locale='C'))

        # check available actions
        should_actions = [{'id': 'view',
                           'label': 'View',
                           'uri': 'https://openlibrary.org/works/OL13101191W'}]
        self.maxDiff = None
        self.assertEqual(response[2]['widget']['actions'], should_actions)

        # check description
        self.assertTrue(response[3]['widget']['text'].startswith(
            "As popular now as it was 150 years ago"))
