# -*- coding: utf8 -*-

# Copyright 2014 Canonical
# All Rights Reserved

"""Tests for the Simple App base stuff."""

import json

from unittest import TestCase
from mock import patch

from src.scopes import reddit
from src.scopes.tests import get_fixture

CONFIG = {'secrets': {}}


class SearchTestCase(TestCase):
    """Tests for the search function."""

    def test_surfacing(self):
        app = reddit.App(CONFIG)
        with patch.object(reddit, '_get_threads') as mock:
            mock.return_value = json.loads(get_fixture('reddit-front.json'))
            response = list(app.search(query='', platform='desktop',
                                       limit=app.default_limit, locale='C'))

        # got the info as it should
        mock.assert_called_with('http://reddit.com/.json?limit=25')

        # category
        should_cat = {'category': reddit.THREAD_CATEGORY}
        self.assertEqual(response[0], should_cat)

        # the result
        should_res = {'result': {
            'cat_id': 'stories',
            'dnd_uri': u'http://reddit.com/r/funny/comments/1z2w0l/' +
            'woman_in_scotland_fills_out_the_wrong_form_on/',
            'uri': u'http://reddit.com/r/funny/comments/1z2w0l/' +
            'woman_in_scotland_fills_out_the_wrong_form_on/',
            'title': u'Woman in Scotland fills out the wrong form on ' +
            'TripAdvisor. Is now ranked 87 out of 168 attraction in Glasgow.',
            'author': u'chrisTMMurphy',
            'comments': '358',
            'mascot': u'http://c.thumbs.redditmedia.com/U-52P34Hz9WDtid2.jpg',
            'media_type': None,
            'media_preview': None,
            'target': u'http://imgur.com/RtCUqDy',
            'summary': u'',
            'subreddit': u'funny',
            'domain': u'imgur.com',
        }}
        self.maxDiff = None
        self.assertEqual(response[1], should_res)

    def test_search(self):
        app = reddit.App(CONFIG)
        with patch.object(reddit, '_get_threads') as mock:
            mock.return_value = json.loads(
                get_fixture('reddit-subreddit.json'))
            response = list(app.search(query='ubuntu', platform='desktop',
                                       limit=app.default_limit, locale='C'))

        # got the info as it should
        mock.assert_called_with('http://reddit.com/r/ubuntu/hot.json?limit=25')

        # category
        should_cat = {'category': reddit.THREAD_CATEGORY}
        self.assertEqual(response[0], should_cat)

        # the result
        should_res = {'result': {
            'cat_id': 'stories',
            'dnd_uri': u'http://reddit.com/r/Ubuntu/comments/1z342m/' +
            'i_successfully_took_an_old_netbook_from_1104_to/',
            'uri': u'http://reddit.com/r/Ubuntu/comments/1z342m/' +
            'i_successfully_took_an_old_netbook_from_1104_to/',
            'title': u'I successfully took an old netbook from 11.04 ' +
            'to 13.10 yesterday without reinstalling. But this error ' +
            'made me laugh.',
            'author': u'CMahaff',
            'comments': '8',
            'mascot': u'',
            'media_type': None,
            'media_preview': None,
            'target': u'http://imgur.com/eS19PVG',
            'summary': u'',
            'subreddit': u'Ubuntu',
            'domain': u'imgur.com',
        }}
        self.maxDiff = None
        self.assertEqual(response[1], should_res)


class PreviewTestCase(TestCase):
    """Tests for the preview function."""

    def test_video_preview(self):
        # fake a previous video result
        result = {
            'cat_id': 'stories',
            'dnd_uri': u'The uri',
            'uri': u'The uri',
            'title': u'The title',
            'author': u'The author',
            'comments': '2',
            'mascot': u'The mascot',
            'media_type': u'video',
            'media_preview': u'some image',
            'target': u'The target',
            'summary': u'The summary',
            'subreddit': u'The subreddit',
            'domain': u'The domain',
        }
        app = reddit.App(CONFIG)
        response = list(app.preview(result=result, locale='C'))

        # check the 'video'
        should = dict(widget={
            'id': 'video',
            'type': 'video',
            'source': 'The target',
            'screenshot': 'some image',
        })
        self.assertEqual(response[0], should)

    def test_image_preview(self):
        # fake a previous image result
        result = {
            'cat_id': 'stories',
            'dnd_uri': u'The uri',
            'uri': u'The uri',
            'title': u'The title',
            'author': u'The author',
            'comments': '2',
            'mascot': u'The mascot',
            'media_type': u'rich',
            'media_preview': u'some image',
            'target': u'The target',
            'summary': u'The summary',
            'subreddit': u'The subreddit',
            'domain': u'The domain',
        }
        app = reddit.App(CONFIG)
        response = list(app.preview(result=result, locale='C'))

        # check the 'image'
        should = dict(widget={
            'id': 'image',
            'type': 'image',
            'source': 'some image',
        })
        self.assertEqual(response[0], should)

    def test_generic_preview(self):
        # fake a previous generic result
        result = {
            'cat_id': 'stories',
            'dnd_uri': u'The uri',
            'uri': u'The uri',
            'title': u'The title',
            'author': u'The author',
            'comments': '2',
            'mascot': u'The mascot',
            'media_type': None,
            'media_preview': None,
            'target': u'The target',
            'summary': u'The summary',
            'subreddit': u'The subreddit',
            'domain': u'The domain',
        }
        app = reddit.App(CONFIG)
        response = list(app.preview(result=result, locale='C'))

        # check the 'header'
        should = dict(widget={
            'id': 'header',
            'type': 'header',
            'mascot': 'The mascot',
            'subtitle': 'The domain',
            'title': 'The title',
        })
        self.assertEqual(response[0], should)

        # check the 'actions'
        should = dict(widget={
            'id': 'buttons',
            'type': 'actions',
            'actions': [{'id': 'action1',
                         'label': '2 comments',
                         'uri': u'The uri'},
                        {'id': 'action2',
                         'label': 'View',
                         'uri': u'The target'}],
        })
        self.assertEqual(response[1], should)

        # check the 'text'
        should = dict(widget={
            'id': 'text',
            'type': 'text',
            'text': 'The summary',
        })
        self.assertEqual(response[2], should)
