# -*- coding: utf8 -*-

# Copyright 2014 Canonical
# All Rights Reserved

"""Tests for the weatherchannel scope."""

import json

from StringIO import StringIO

from unittest import TestCase

from mock import patch

from src.scopes import weatherchannel
from src.scopes.tests import get_fixture


class AppTestCase(TestCase):
    """Internal tests."""

    def test__get_location_key_with_query(self):
        app = weatherchannel.App({'base_image_url': 'foo',
                                  'secrets': {'api_key': 'bar'}})
        with patch.object(weatherchannel, 'urlopen') as mock:
            response = StringIO(get_fixture('weatherchannel-loc_key'
                                            '-lookup.json'))
            mock.side_effect = [response]
            # with query and country
            app._get_location_key("foo", "US", "en_GB", 1.0, 1.0)
            mock.assert_called_with("http://wxdata.weather.com/wxdata/ta/"
                                    "foo.js?key=bar&countrySort=US")

    def test__get_location_key_without_query(self):
        app = weatherchannel.App({'base_image_url': 'foo',
                                  'secrets': {'api_key': 'bar'}})
        with patch.object(weatherchannel, 'urlopen') as mock:
            response = StringIO(get_fixture('weatherchannel-loc_key.json'))
            mock.side_effect = [response]
            # without query
            app._get_location_key("", "US", "en_GB", 1.0, 1.0)
            loc_uri = ("http://wxdata.weather.com/wxdata/loc/get.js?"
                       "lat=1.0&lng=1.0&key=bar&locale=en_GB")
            mock.assert_called_with(loc_uri)

    def test__get_location_key_empty(self):
        app = weatherchannel.App({'base_image_url': 'foo',
                                  'secrets': {'api_key': 'bar'}})
        with patch.object(weatherchannel, 'urlopen') as mock:
            response = StringIO("")
            mock.side_effect = [response]
            # without query
            self.assertRaises(ValueError, app._get_location_key,
                              "", "US", "en_GB", 1.0, 1.0)

    def test__get_location_key_empty_json(self):
        app = weatherchannel.App({'base_image_url': 'foo',
                                  'secrets': {'api_key': 'bar'}})
        with patch.object(weatherchannel, 'urlopen') as mock:
            response = StringIO('{}')
            response2 = StringIO('{"results":[]}')
            mock.side_effect = [response, response2]
            # without query
            self.assertRaises(KeyError, app._get_location_key,
                              "", "US", "en_GB", 1.0, 1.0)
            # with query
            self.assertRaises(ValueError, app._get_location_key,
                              "foo", "US", "en_GB", 1.0, 1.0)

    def test__get_location_key_no_data(self):
        app = weatherchannel.App({'base_image_url': 'foo',
                                  'secrets': {'api_key': 'bar'}})
        resp_data = {"class": "error", "type": 1000, "cat": "data",
                     "text": "The requested data is not available"}
        with patch.object(weatherchannel, 'urlopen') as mock:
            response = StringIO(json.dumps(resp_data))
            mock.side_effect = [response]
            # without query
            self.assertRaises(weatherchannel.LocationNotFound,
                              app._get_location_key,
                              "", "US", "en_GB", 1.0, 1.0)

    def test__get_location_(self):
        app = weatherchannel.App({'base_image_url': 'foo',
                                  'secrets': {'api_key': 'bar'}})
        with patch.object(weatherchannel, 'urlopen') as mock:
            response = StringIO("")
            mock.side_effect = [response]
            # without query
            self.assertRaises(ValueError, app._get_location_key,
                              "", "US", "en_GB", 1.0, 1.0)


class SearchTestCase(TestCase):
    """Search tests"""

    maxDiff = None

    def test_small_query(self):
        app = weatherchannel.App({'base_image_url': 'foo',
                                  'secrets': {}})
        response = list(app.search(query=u'f', locale='en_GB'))
        self.assertEqual(response, [])

    def test_simple(self):
        app = weatherchannel.App({'base_image_url': 'foo',
                                  'secrets': {}})

        with patch.object(app, '_search') as mock, \
                patch.object(app, '_get_location_key') as mock1:
            mock1.return_value = 'UKXX0085'
            mock.return_value = json.loads(
                get_fixture('weatherchannel-simple.json'))
            response = list(app.search(query=u'foo', locale='en_GB'))

        # got the info as it should
        mock.assert_called_once_with(u'foo', 'UKXX0085', 'en_GB')
        mock1.assert_called_once()

        # category
        should_cat = {'category': weatherchannel.CATEGORY_DEFAULT}
        self.assertEqual(response[0], should_cat)

        # the result
        result = response[1]
        should_res = {'result': {
            'art': 'foo/weatherchannel/suru/logo.png',
            'cat_id': weatherchannel.CATEGORY_DEFAULT['id'],
            'comment': u'Mainly cloudy and rainy. Low 7C. Winds SSW at 10 to '
            '20 mph. Rainfall around 12mm. Localized flooding is possible.',
            'conditions': u'Showers',
            'current_temp': u'10\xb0',
            'date': '06 Feb',
            'day': 'Today',
            'dnd_uri': 'http://www.weather.com/weather/today/UKXX0085#0'
            '?par=ubuntu',
            'formfactor': 'desktop',
            'humidity': '83%',
            'loc_key': 'UKXX0085',
            'locale': 'en_GB',
            'location': u'London, GLA, UK',
            'max_temp': u'9\xb0',
            'min_temp': u'4\xb0',
            'pop': '40%',
            'result_index': 0,
            'title': '',
            'uri': 'http://www.weather.com/weather/today/UKXX0085#0'
            '?par=ubuntu#logo',
            'uv': '0',
            'weather_type_code': 12,
            'wind_sentence': 'S at 16 mph'}
        }
        self.assertDictEqual(result, should_res)
        self.assertEqual(response[2]['result']['title'], u'London, UK\nToday')
        self.assertEqual(response[3]['result']['title'], u'Tomorrow\n07 Feb')

    def test_simple_US(self):
        app = weatherchannel.App({'base_image_url': 'foo',
                                  'secrets': {}})

        with patch.object(app, '_search') as mock, \
                patch.object(app, '_get_location_key') as mock1:
            mock1.return_value = 'UKXX0085'
            mock.return_value = json.loads(
                get_fixture('weatherchannel-simple.json'))
            response = list(app.search(query=u'foo', locale='en_US'))

        # got the info as it should
        mock.assert_called_once_with(u'foo', 'UKXX0085', 'en_US')
        mock1.assert_called_once()

        # category
        should_cat = {'category': weatherchannel.CATEGORY_DEFAULT}
        self.assertEqual(response[0], should_cat)

        # the result
        result = response[1]
        should_res = {'result': {
            'art': 'foo/weatherchannel/suru/logo.png',
            'cat_id': weatherchannel.CATEGORY_DEFAULT['id'],
            'comment': u'Mainly cloudy and rainy. Low 7C. Winds SSW at 10 to '
            '20 mph. Rainfall around 12mm. Localized flooding is possible.',
            'conditions': u'Showers',
            'current_temp': u'50\xb0',
            'date': '06 Feb',
            'day': 'Today',
            'dnd_uri': 'http://www.weather.com/weather/today/UKXX0085#0'
            '?par=ubuntu',
            'formfactor': 'desktop',
            'humidity': '83%',
            'loc_key': 'UKXX0085',
            'locale': 'en_US',
            'location': u'London, GLA, UK',
            'max_temp': u'48\xb0',
            'min_temp': u'39\xb0',
            'pop': '40%',
            'result_index': 0,
            'title': '',
            'uri': 'http://www.weather.com/weather/today/UKXX0085#0'
            '?par=ubuntu#logo',
            'uv': '0',
            'weather_type_code': 12,
            'wind_sentence': 'S at 16 mph'}
        }
        self.assertDictEqual(result, should_res)
        self.assertEqual(response[2]['result']['title'], u'London, UK\nToday')
        self.assertEqual(response[3]['result']['title'], u'Tomorrow\n07 Feb')

    def test_chinese(self):
        app = weatherchannel.App({'base_image_url': 'foo',
                                  'secrets': {}})

        with patch.object(app, '_search') as mock_search, \
                patch.object(app, '_get_location_key') as mock_lockey:
            mock_lockey.return_value = 'UKXX0085'
            mock_search.return_value = json.loads(
                get_fixture('weatherchannel-chinese.json'))
            response = list(app.search(query=u'foo', locale='en_US'))

        # category
        should_cat = {'category': weatherchannel.CATEGORY_DEFAULT}
        self.assertEqual(response[0], should_cat)

        # the result
        result = response[1]
        should_res = {'result': {
            'art': 'foo/weatherchannel/suru/logo.png',
            'cat_id': weatherchannel.CATEGORY_DEFAULT['id'],
            'comment': u'夜间有阵雨。 最低温度 10。 東北風 级大风，风速 10 '
                       u'至 15。 降水概率为 60%。',
            'conditions': u'局部多云',
            'current_temp': u'53\xb0',
            'date': '24 Apr',
            'day': 'Today',
            'dnd_uri': 'http://www.weather.com/weather/today/UKXX0085#0'
            '?par=ubuntu',
            'formfactor': 'desktop',
            'humidity': '70%',
            'loc_key': 'UKXX0085',
            'locale': 'en_US',
            'location': u'伦敦, GLA, UK',
            'max_temp': u'59\xb0',
            'min_temp': u'42\xb0',
            'pop': '10%',
            'result_index': 0,
            'title': '',
            'uri': 'http://www.weather.com/weather/today/UKXX0085#0'
            '?par=ubuntu#logo',
            'uv': '0',
            'weather_type_code': 45,
            'wind_sentence': u'西風 at 9 mph'}
        }
        self.assertDictEqual(result, should_res)


class PreviewTestCase(TestCase):
    """Tests for the preview function."""

    def test_ok(self):
        # fake a previous result
        result = {
            'art': 'foo/weatherchannel/weather-showers.svg',
            'cat_id': weatherchannel.CATEGORY_DEFAULT['id'],
            'comment': u'Mainly cloudy and rainy. Low 7C. Winds SSW at 10 to '
            '20 mph. Rainfall around 12mm. Localized flooding is possible.',
            'conditions': u'Showers',
            'current_temp': u'50\xb0',
            'date': '06 Feb',
            'day': 'Today',
            'dnd_uri': 'http://www.weather.com/weather/today/UKXX0085#0'
            '?par=ubuntu',
            'formfactor': 'desktop',
            'humidity': '83%',
            'icon': 'foo/weatherchannel/logo.svg',
            'loc_key': 'UKXX0085',
            'locale': 'en_US',
            'location': u'London, GLA, UK',
            'max_temp': u'48\xb0',
            'min_temp': u'39\xb0',
            'pop': '40%',
            'result_index': 0,
            'title': '',
            'uri': 'http://www.weather.com/weather/today/UKXX0085#0'
            '?par=ubuntu#logo',
            'uv': '0',
            'weather_type_code': 12,
            'wind_sentence': 'S at 16 mph'}
        app = weatherchannel.App({'base_image_url': 'foo',
                                  'secrets': {}})
        response = list(app.preview(result=result, locale='en_US'))

        # check the 'image'
        should = dict(widget={
            'id': 'art',
            'type': 'image',
            'source': 'foo/weatherchannel/weather-showers.svg',
        })
        self.assertEqual(response[0], should)

        # check the 'header'
        should = dict(widget={
            'id': 'header1',
            'type': 'header',
            'uri': 'http://www.weather.com/weather/today/UKXX0085#0'
            '?par=ubuntu#logo',
            'subtitle': u'London, GLA, UK   06 Feb',
            'title': u'Today: Showers 50\xb0',
            'mascot': 'foo/weatherchannel/suru/logo.png',
        })
        self.maxDiff = None
        self.assertDictEqual(response[1], should)
        # check the text
        should = dict(widget={
                      'id': 'text1',
                      'text': u'Minimum temperature: 39\xb0\nMaximum '
                      u'temperature: 48\xb0\nChances of rain: 40%\nWind:'
                      u' S at 16 mph\nHumidity: 83%\nUV index: 0',
                      'title': None,
                      'type': 'text'})
        self.assertDictEqual(response[2], should)
        # check the actions
        should = dict(widget={'actions': [
            {'icon': None,
             'id': 'action1',
             'label': '10 days',
             'uri': 'http://www.weather.com/weather/tenday/UKXX0085'
             '?par=ubuntu'},
            {'icon': None,
             'id': 'action2',
             'label': 'Hourly',
             'uri': 'http://www.weather.com/weather/hourbyhour/graph/UKXX0085'
             '#0?par=ubuntu'}],
            'id': 'actions',
            'type': 'actions'
        })
        self.assertDictEqual(response[3], should)
