# -*- coding: utf8 -*-

# Copyright 2014 Canonical
# All Rights Reserved

"""Tests for the Simple App base stuff."""

import feedparser

from unittest import TestCase

from mock import patch

from src.scopes import wikipedia
from src.scopes.tests import get_fixture
from src.utils import localized_category

CONFIG = {'base_image_url': ''}


class SearchTestCase(TestCase):
    """The tests for the search part."""

    def test_simple_english(self):
        app = wikipedia.App(CONFIG)
        with patch.object(wikipedia, '_get_search') as mock:
            mock.return_value = get_fixture('wikipedia-simple.xml')
            response = list(app.search(query=u'q', locale='en_US', limit=10))

        # got the info as it should
        mock.assert_called_with(u'q', 10, 'en')

        # category
        should_cat = {'category': wikipedia.CATEGORY_DEFAULT}
        self.assertEqual(response[0], should_cat)

        # the result
        should_res = {'result': {
            'cat_id': wikipedia.CATEGORY_DEFAULT['id'],
            'uri': u'http://en.wikipedia.org/wiki/Ubuntu_(operating_system)',
            'dnd_uri':
            u'http://en.wikipedia.org/wiki/Ubuntu_(operating_system)',
            'title': u'Ubuntu (operating system)',
            'art': '/icons/wikipedia-scope.svg',
        }}
        self.assertEqual(response[1], should_res)

    def test_simple_otherlang(self):
        app = wikipedia.App(CONFIG)
        with patch.object(wikipedia, '_get_search') as mock:
            mock.return_value = get_fixture('wikipedia-otherlang.xml')
            response = list(app.search(query=u'q', locale='fr_FR', limit=10))

        # got the info as it should
        mock.assert_called_with(u'q', 10, 'fr')

        # category
        should_cat = {
            'category': localized_category(wikipedia.CATEGORY_DEFAULT, 'fr')
        }
        self.assertEqual(response[0], should_cat)

        # the result
        uri = u'http://fr.wikipedia.org/wiki/Ubuntu'
        should_res = {'result': {
            'cat_id': wikipedia.CATEGORY_DEFAULT['id'],
            'uri': uri,
            'dnd_uri': uri,
            'title': u'Ubuntu',
            'art': (
                'http://upload.wikimedia.org/wikipedia/commons/thumb/'
                '8/81/Ubuntu_12.04_LTS_%28Fran%C3%A7ais%29.png/'
                '96px-Ubuntu_12.04_LTS_%28Fran%C3%A7ais%29.png'),
        }}
        self.assertEqual(response[1], should_res)

    def test_surfacing(self):
        app = wikipedia.App(CONFIG)
        with patch.object(wikipedia, '_get_featured') as mock:
            mock.return_value = feedparser.parse(
                get_fixture('wikipedia-featured.rss'))
            response = list(app.search(query='', locale='fr_FR', limit=10))

        # got the info as it should
        mock.assert_called_with(10, 'fr')

        # category
        should_cat = {
            'category': localized_category(wikipedia.CATEGORY_FEATURED, 'fr')
        }
        self.assertEqual(response[0], should_cat)

        # the result
        should_res = {'result': {
            'cat_id': wikipedia.CATEGORY_FEATURED['id'],
            'uri': 'http://fr.wikipedia.org/wiki/Jean%20B%C3%A9liveau',
            'dnd_uri': 'http://fr.wikipedia.org/wiki/Jean%20B%C3%A9liveau',
            'title':  u'Jean Béliveau',
            'art': '/icons/wikipedia-scope.svg',
        }}
        self.maxDiff = None
        self.assertEqual(response[1], should_res)


class PreviewTestCase(TestCase):
    """Tests for the preview function."""

    def test_ok(self):
        # fake a previous result
        result = {
            'title': u'test title moño',
            'art': 'http://w.org/wikipedia/new.jpg/96px-new.jpg',
            'uri': 'test uri',
        }
        app = wikipedia.App(CONFIG)
        with patch.object(wikipedia, '_get_preview') as mock:
            with patch.object(wikipedia, '_get_status_code') as status_code:
                status_code.return_value = 200
                mock.return_value = get_fixture('wikipedia-extraction.json')
                response = list(app.preview(result=result, locale='en_US'))

        # got the info as it should
        mock.assert_called_with('test%20title%20mo%C3%B1o', 200, 'en')

        # the art here is bigger
        should_art = 'http://w.org/wikipedia/new.jpg/380px-new.jpg'

        # check the 'image'
        should = dict(widget={
            'id': 'art',
            'type': 'image',
            'source': should_art,
        })
        self.assertEqual(response[0], should)

        # check the 'header'
        should = dict(widget={
            'id': 'header1',
            'type': 'header',
            'uri': 'test uri',
            'title': u'test title moño',
            'mascot': None,
        })
        self.assertEqual(response[1], should)

        # check the 'text'
        extract = "<p><b>Argentina</b></p>..."  # from fixture file
        should = dict(widget={
            'id': 'text1',
            'type': 'text',
            'uri': 'test uri',
            'title': None,
            'text': extract,
        })
        self.assertEqual(response[2], should)

        # check the 'actions'
        should = dict(widget={
            'id': 'buttons',
            'type': 'actions',
            'actions': [{'id': 'action1',
                         'uri': 'test uri',
                         'label': 'View on Wikipedia'}],
        })
        self.assertEqual(response[3], should)

    def test_preview_image_missing(self):
        # fake a previous result
        result = {
            'title': u'test title moño',
            'art': 'http://w.org/wikipedia/new.jpg/96px-new.jpg',
            'uri': 'test uri',
        }
        app = wikipedia.App(CONFIG)
        with patch.object(wikipedia, '_get_preview') as mock:
            with patch.object(wikipedia, '_get_status_code') as status_code:
                status_code.return_value = 500
                mock.return_value = get_fixture('wikipedia-extraction.json')
                response = list(app.preview(result=result, locale='en_US'))

        # If the bigger art is missing, make sure the mascot is the art
        self.assertEqual(response[0]['widget']['mascot'], result['art'])

        # check we don't have an 'image' widget
        for widget in response:
            self.assertNotEqual(widget['widget']['type'], 'image')

    def test_preview_image(self):
        # fake a previous result
        result = {
            'title': u'test title moño',
            'art': 'http://w.org/wikipedia/new.jpg/96px-new.jpg',
            'uri': 'test uri',
        }
        app = wikipedia.App(CONFIG)
        with patch.object(wikipedia, '_get_preview') as mock:
            with patch.object(wikipedia, '_get_status_code') as status_code:
                status_code.return_value = 200
                mock.return_value = get_fixture('wikipedia-extraction.json')
                response = list(app.preview(result=result, locale='en_US'))

        # If the bigger art is present, make sure the mascot is cancelled
        self.assertEqual(response[1]['widget']['mascot'], None)

        # check the 'image' widget
        should = dict(widget={
            'id': 'art',
            'type': 'image',
            'source': 'http://w.org/wikipedia/new.jpg/380px-new.jpg',
        })
        self.assertEqual(response[0], should)
