# -*- coding: utf8 -*-

# Copyright 2014 Canonical
# All Rights Reserved

"""Tests for the Simple App base stuff."""

from unittest import TestCase
from mock import patch

from src.scopes import yahoofinance

CONFIG = {'secrets': {}}


class SearchTestCase(TestCase):
    """Tests for the search function."""

    def test_surfacing(self):
        app = yahoofinance.App(CONFIG)
        with patch.object(yahoofinance, '_get_trending_symbols') as trending:
            with patch.object(yahoofinance, '_get_quotes') as quotes:
                with patch.object(yahoofinance, '_get_news') as news:
                    trending.return_value = "UBNT"
                    quotes.return_value = [{"symbol": "UBNT",
                                            "change": "+1 +14.04%",
                                            "name": "Ubuntu Corp.",
                                            "value": "1337",
                                            "summary": ""}]
                    news.return_value = [{"uri": "news uri",
                                          "title": "news title",
                                          "date": "news date",
                                          "summary": "news summary"}]
                    response = list(app.search(query='', platform='desktop',
                                               limit=app.default_limit,
                                               locale='C'))

        # Ensure we ask for the right number of trends
        trending.assert_called_with(app.default_limit)
        # Ensure the trends are used for the quotes query
        quotes.assert_called_with("http://finance.yahoo.com/d/quotes.cs"
                                  "v?s=UBNT&f=snl1d1t1vc")
        # Ensure we look for global headlines
        news.assert_called_with(None)

        # categories
        should_cat = {'category': yahoofinance.HEADLINES_CATEGORY}
        self.assertEqual(response[0], should_cat)
        should_cat = {'category': yahoofinance.TRENDING_CATEGORY}
        self.assertEqual(response[2], should_cat)

        # Headline result
        should_res = {'result': {
            'cat_id': 'headlines',
            'dnd_uri': 'news uri',
            'uri': 'news uri',
            'title': 'news title',
            'summary': 'news summary',
            'date': 'news date',
        }}
        self.maxDiff = None
        self.assertEqual(response[1], should_res)

        # Trending result
        should_res = {'result': {
            'cat_id': 'trending',
            'dnd_uri': 'http://finance.yahoo.com/q?s=UBNT',
            'uri': 'http://finance.yahoo.com/q?s=UBNT',
            'title': 'Ubuntu Corp.',
            'symbol': 'UBNT',
            'change_p': '+14.04%',
            'change_v': '+1',
            'value': '1337',
            'graph': 'http://chart.finance.yahoo.com/z?s=UBNT&z=s&t=7d',
        }}
        self.maxDiff = None
        self.assertEqual(response[3], should_res)

    def test_news_with_various_feedparser_versions(self):
        expected_headlines = [{'date': 'date',
                               'summary': 'summary',
                               'uri': 'uri',
                               'title': 'title'}]

        class Mock_old_feedparser(object):
            """Mock an old feedparser version."""

            class Entry(object):
                """Mock an Entry object."""
                # updated instead of published
                updated = "date"
                title = "title"
                link = "uri"
                summary = "summary"

                def keys(self):
                    """Mock keys."""
                    return ["title", "link", "updated"]
            entry = Entry()
            entries = [entry]

        with patch.object(yahoofinance, 'feedparser') as feedp:
            feedp.parse.return_value = Mock_old_feedparser()
            headlines = yahoofinance._get_news(None)
            self.maxDiff = None
            self.assertEqual(headlines, expected_headlines)

        class Mock_new_feedparser(object):
            """Mock a more recent feedparser version."""

            class Entry(object):
                """Mock an Entry object."""
                # published instead of updated
                published = "date"
                title = "title"
                link = "uri"
                summary = "summary"

                def keys(self):
                    """Mock keys."""
                    return ["title", "link", "published"]
            entry = Entry()
            entries = [entry]

        with patch.object(yahoofinance, 'feedparser') as feedp:
            feedp.parse.return_value = Mock_new_feedparser()
            headlines = yahoofinance._get_news(None)
            self.maxDiff = None
            self.assertEqual(headlines, expected_headlines)

    def test_search(self):
        app = yahoofinance.App(CONFIG)
        with patch.object(yahoofinance, '_get_quotes') as quotes:
            with patch.object(yahoofinance, '_get_symbol_from_name') as name:
                name.return_value = "UBNT", {"UBNT": {
                                             "exchange": "TUX",
                                             "name": "Ubuntu Corp.",
                                             "value": "1337", }
                                             }
                quotes.return_value = [{"symbol": "UBNT",
                                        "change": "+1 +14.04%",
                                        "name": "Ubuntu Corp.",
                                        "value": "1337",
                                        "summary": ""}]
                response = list(app.search(query='ubu', platform='desktop',
                                           limit=app.default_limit,
                                           locale='C'))

        # Ensure the quotes are used for the query
        quotes.assert_called_with("http://finance.yahoo.com/d/quotes.csv"
                                  "?s=UBNT&f=snl1d1t1vc")

        # Custom quotes categories
        QUOTES_CATEGORY = {
            'id': 'TUX',
            'title': 'TUX',
            'render_template': yahoofinance.RENDERER_QUOTES,
        }
        should_cat = {'category': QUOTES_CATEGORY}
        self.assertEqual(response[0], should_cat)

        # Quotes result
        should_res = {'result': {
            'cat_id': 'TUX',
            'dnd_uri': 'http://finance.yahoo.com/q?s=UBNT',
            'uri': 'http://finance.yahoo.com/q?s=UBNT',
            'title': 'Ubuntu Corp.',
            'symbol': 'UBNT',
            'change_p': '+14.04%',
            'change_v': '+1',
            'value': '1337',
            'graph': 'http://chart.finance.yahoo.com/z?s=UBNT&z=s&t=7d',
        }}
        self.maxDiff = None
        self.assertEqual(response[2], should_res)


class PreviewTestCase(TestCase):
    """Tests for the preview function."""

    def test_quote_preview(self):
        # fake a previous quote result
        result = {
            'cat_id': 'TUX',
            'dnd_uri': 'http://finance.yahoo.com/q?s=UBNT',
            'uri': 'http://finance.yahoo.com/q?s=UBNT',
            'title': 'Ubuntu Corp.',
            'symbol': 'UBNT',
            'change_p': '+14.04%',
            'change_v': '+1',
            'value': '1337',
            'graph': 'http://chart.finance.yahoo.com/z?s=UBNT&z=s&t=7d',
        }
        app = yahoofinance.App(CONFIG)
        response = list(app.preview(result=result, locale='C'))

        # check the graph
        should = dict(widget={
            'id': 'art',
            'type': 'image',
            'source': 'http://chart.finance.yahoo.com/z?s=UBNT&z=s&t=7d',
        })
        self.assertEqual(response[0], should)

        # check the header
        should = dict(widget={
            'id': 'header',
            'type': 'header',
            'subtitle': 'UBNT',
            'title': 'Ubuntu Corp.'
        })
        self.assertEqual(response[1], should)

        # check the buttons
        should = dict(widget={
            'id': 'buttons',
            'type': 'actions',
            'actions': [{'id': 'yahoo',
                         'label': 'Yahoo! Finance',
                         'uri': 'http://finance.yahoo.com/q?s=UBNT'}],
        })
        self.assertEqual(response[2], should)

    def test_news_preview(self):
        # fake a previous news result
        result = {
            'cat_id': 'headlines',
            'dnd_uri': 'news uri',
            'uri': 'news uri',
            'title': 'news title',
            'summary': 'news summary',
            'date': 'news date',
        }
        app = yahoofinance.App(CONFIG)
        response = list(app.preview(result=result, locale='C'))

        # check the header
        should = dict(widget={
            'id': 'header',
            'type': 'header',
            'subtitle': 'news date',
            'title': 'news title'
        })
        self.assertEqual(response[0], should)

        # check the buttons
        should = dict(widget={
            'id': 'buttons',
            'type': 'actions',
            'actions': [{'id': 'yahoo',
                         'label': 'Yahoo! Finance',
                         'uri': 'news uri'}],
        })
        self.assertEqual(response[1], should)

        # check the summary
        should = dict(widget={
            'id': 'text',
            'type': 'text',
            'text': 'news summary',
        })
        self.assertEqual(response[2], should)
