
# Copyright 2014 Canonical
# All Rights Reserved

"""The Weather channel scope."""

import datetime
import json
import logging
import urllib

from urllib2 import urlopen

from ..scopes import SimpleApp
from ..translator import translator
from ..utils import localized_category

TA_SEARCH_URI = 'http://wxdata.weather.com/wxdata/ta/'
LOC_GET_URI = 'http://wxdata.weather.com/wxdata/loc/get.js?' + \
    'lat=%s&lng=%s&key=%s&locale=%s'
SEARCH_URI = 'http://wxdata.weather.com/wxdata/mobile/mobagg/'
# limit of the quantity of forecast days, business reasons
MAX_FORECAST_DAYS = 5
_icons_map = [
    ('weather-clear', [32, 36]),
    ('weather-clear-night', [31]),
    ('weather-clouds', [28, 30]),
    ('weather-clouds-night', [27, 29]),
    ('weather-few-clouds', [34]),
    ('weather-few-clouds-night', [33]),
    ('weather-fog', [19, 20, 21, 22, 23, 24]),
    ('weather-overcast', [26]),
    ('weather-showers', [6, 10, 11, 12, 18, 40]),
    ('weather-showers-scattered', [8, 9, 39, 45]),
    ('weather-snow', [5, 7, 13, 14, 15, 16, 41, 42, 43, 46]),
    ('weather-storm', [0, 1, 2, 3, 4, 17, 35, 37, 38, 47]),
]
WEATHER_ICONS_MAP = {}
for icon, wids in _icons_map:
    WEATHER_ICONS_MAP.update(dict.fromkeys(wids, icon))

LOCALURLS_MAP = {
    'fr': {'mobile': {'hourly': 'm.fr.weather.com/horaire/',
                      'today': 'm.weather.com/weather/today/',
                      'tendays': 'm.fr.weather.com/10jours/'},
           'desktop': {'hourly': 'fr.weather.com/horaire/',
                       'today': 'www.weather.com/weather/today/',
                       'tendays': 'fr.weather.com/10jours/'}},
    'zh': {'mobile': {'hourly': u'm.zh.weather.com/\\u6bcf\\u5c0f\\u65f6/',
                      'today': 'm.weather.com/weather/today/',
                      'tendays': u'm.zh.weather.com/10\\u5929/'},
           'desktop': {'hourly': u'zh.weather.com/\\u6bcf\\u5c0f\\u65f6/',
                       'today': 'www.weather.com/weather/today/',
                       'tendays': u'zh.weather.com/10\\u5929/'}},
    'pt_BR': {'mobile': {'hourly': 'm.br.weather.com/toda_hora/',
                         'today': 'm.weather.com/weather/today/',
                         'tendays': 'm.br.weather.com/10dias/'},
              'desktop': {'hourly': 'br.weather.com/toda_hora/',
                          'today': 'www.weather.com/weather/today/',
                          'tendays': 'br.weather.com/10dias/'}},
    'de': {'mobile': {'hourly': u'm.de.weather.com/st\\xfcndlich/',
                      'today': 'm.weather.com/weather/today/',
                      'tendays': 'm.de.weather.com/10tage/'},
           'desktop': {'hourly': u'de.weather.com/st\\xfcndlich/',
                       'today': 'www.weather.com/weather/today/',
                       'tendays': 'de.weather.com/10tage/'}},
    'other': {'mobile': {'hourly': 'm.weather.com/weather/hourbyhour/graph/',
                         'today': 'm.weather.com/weather/today/',
                         'tendays': 'm.weather.com/weather/tenday/'},
              'desktop': {
                  'hourly': 'www.weather.com/weather/hourbyhour/graph/',
                  'today': 'www.weather.com/weather/today/',
                  'tendays': 'www.weather.com/weather/tenday/'}},
    'es': {'mobile': {'hourly': 'm.es.weather.com/weather/por_hora/',
                      'today': 'm.weather.com/weather/today/',
                      'tendays': u'm.es.weather.com/weather/10d\\xedas/'},
           'desktop': {'hourly': 'espanol.weather.com/weather/hourByHour-',
                       'today': 'www.weather.com/weather/today/',
                       'tendays': 'espanol.weather.com/weather/10day-'}},
    'ja': {
        'mobile': {
            'hourly': u'm.ja.weather.com/1\\u6642\\u9593\\u3054\\u3068/',
            'today': 'm.weather.com/weather/today/',
            'tendays': u'm.ja.weather.com/10\\u65e5/'},
        'desktop': {'hourly': u'ja.weather.com/1\\u6642\\u9593\\u3054\\u3068/',
                    'today': 'www.weather.com/weather/today/',
                    'tendays': u'ja.weather.com/10\\u65e5/'}},
    'en_GB': {'mobile': {'hourly': 'm.weather.co.uk/hourly/',
                         'today': 'm.weather.com/weather/today/',
                         'tendays': 'm.weather.co.uk/10day/'},
              'desktop': {'hourly': 'weather.co.uk/weather/hourbyhour/graph/',
                          'today': 'www.weather.com/weather/today/',
                          'tendays': 'weather.co.uk/weather/10day-'}}
}

# the category to use, with a renderer
_renderer_default = {
    "schema-version": 1,
    "template": {
        "category-layout": "grid",
        "card-size": "medium",
    },
    "components": {
        "title": "title",
        "subtitle": "comment",
        "art": "art",
    },
}
CATEGORY_DEFAULT = {
    'id': 'default_category',
    'title': translator('Results'),
    'render_template': json.dumps(_renderer_default),
    'icon': "FIXME",
}

logger = logging.getLogger("restscopes.twc")


class LocationNotFound(Exception):
    """Exception for TWC not finding a location."""


class App(SimpleApp):
    """The Weather channel wsgi app."""

    def __init__(self, *args, **kwargs):
        super(App, self).__init__(*args, **kwargs)
        self.base_image_url = self.config['base_image_url']
        self.api_key = self.config['secrets'].get('api_key')

    def _get_location_key(self, query, country, locale, latitude, longitude):
        """Return the location key."""
        if query and country:
            q = urllib.quote(query)
            # we have something to work with, lookup the city.
            ta_uri = '%s%s.js?key=%s&countrySort=%s' % (
                TA_SEARCH_URI, q, self.api_key, country)
            logger.debug("Requesting place key for city: %r", ta_uri)
            r = urlopen(ta_uri).read()
            d = json.loads(r.decode('utf-8'))
            if len(d['results']) > 0:
                return d['results'][0]['key']
        else:
            # no search string, use geolocation
            q = None
            lat = latitude
            lng = longitude
            loc_uri = LOC_GET_URI % (lat, lng, self.api_key, locale)
            resp = urlopen(loc_uri).read()
            loc_result = json.loads(resp.decode('utf-8'))
            if loc_result['class'] == 'error' \
                    and loc_result['cat'] == 'data':
                raise LocationNotFound("lat: %s - lon: %s - response: %r" %
                                       (lat, lng, json.dumps(loc_result)))
            return loc_result['key']
        raise ValueError("No location key found for query")

    def _forc(self, locale):
        """Return degrees unit: F|C based on locale."""
        locales = ["en_US"]
        if locale in locales:
            unit = "F"
        else:
            unit = "C"
        return unit

    def _korm(self, locale):
        """Return unit K|M based on locale"""
        locales = ["en_US", "en_GB"]
        if locale in locales:
            unit = "M"
        else:
            unit = "K"
        return unit

    def _temp_converter(self, temp, locale):
        """Convert temp. from C/F based on locale."""
        if self._forc(locale) == "C":
            return temp
        else:
            return temp * (9.0 / 5.0) + 32

    def _speed_converter(self, speed, locale):
        """Return the speed as miles or km based on locale."""
        if self._korm(locale) == "M":
            return speed
        else:
            return round(speed * 0.62, 1)

    def get_localized_url(self, locale, formfactor, hint):
        """Retrun the localized URL."""
        if formfactor != 'desktop':
            formfactor = 'mobile'
        try:
            url = LOCALURLS_MAP[locale][formfactor][hint]
        except KeyError:
            lang = locale.split("_")[0]
            try:
                url = LOCALURLS_MAP[lang][formfactor][hint]
            except KeyError:
                url = LOCALURLS_MAP['other'][formfactor][hint]
        return 'http://' + url

    def _search(self, query, loc_key, locale):
        """The actual http request"""
        logger.debug("Search: %s (%s - %s)", query, locale, loc_key)
        # Request forecast
        uri = '%s%s.js?units=m&locale=%s&key=%s' % (SEARCH_URI, loc_key,
                                                    locale, self.api_key)
        try:
            logger.debug("Requesting forecast: '%s'", uri)
            r = urlopen(uri).read()
            data = json.loads(r.decode('utf-8'))
            logger.debug('Got forecast for %s', loc_key)
            return data
        except Exception:
            logger.error("Error while requesting 'forecast'")
            raise

    def search(self, query=None, locale=None, limit=None, country=None,
               latitude=None, longitude=None, platform=None, **kwargs):
        """Search in The Weather Channel backend."""
        if query and len(query) < 2:
            # if the query is really small, just ignore it
            return
        if locale is None:
            locale = "en_GB"
        formfactor = 'desktop'
        if platform:
            formfactor = platform.split('-')[0]
        if query and query.lower() == "weather":
            # use geolocation for this exact match.
            query = ""
        loc_key = self._get_location_key(query, country, locale,
                                         latitude, longitude)
        data = self._search(query, loc_key, locale)
        max_temp, min_temp, conditions = '', '', ''
        pop, wind_direction, wind_speed = 0, '', ''
        # Iterate through narrative forecasts
        # since they have the most interesting data available
        results = {}
        for f in data[0]['NarrativeForecasts']:
            if len(results) == 0:
                title = "%s, %s" % \
                        (data[0]['Location']['name'],
                         data[0]['Location']['countryCode'])
                uri_date = "today"
                day = translator("Today", locale)
                title = "%s\n%s" % (title, day)
            else:
                title = f['dayPart']
                uri_date = "tendays"
                day = title
            realdate = datetime.datetime.fromtimestamp(f['validDate'])
            if realdate in results:
                continue
            date = realdate.strftime('%d %b')

            for daily in data[0]['DailyForecasts']:
                # Match the narrative forecast timestamp with
                # a daily forecast entry based on timestamp
                if datetime.datetime.fromtimestamp(
                    daily['validDate']).strftime('%d %b') == date or \
                        len(results) == 0:
                    if 'maxTemp' in daily:
                        max_temp = '%i%s' % \
                            (self._temp_converter(daily['maxTemp'], locale),
                             u"\u00B0")
                    if 'minTemp' in daily:
                        min_temp = '%i%s' % \
                            (self._temp_converter(daily['minTemp'], locale),
                             u"\u00B0")
                    # Match the day or night part of the daily forecast
                    dayparts = ['day', 'night']
                    if "day" not in daily or \
                            daily['day']['weatherCode'] == "9999":
                        dayparts = ['night', 'night']
                    if datetime.datetime.fromtimestamp(
                        daily['validDate']).strftime(
                            '%H') == datetime.datetime.fromtimestamp(
                            f['validDate']).strftime('%H'):
                        conditions = daily[dayparts[1]]['phrase']
                        pop = daily[dayparts[1]]['pop']
                        wind_speed = self._speed_converter(
                            daily[dayparts[1]]['wSpeed'], locale)
                        wind_direction = daily[dayparts[1]]['wDirText']
                        uv = daily[dayparts[1]]['uv']
                        humidity = daily[dayparts[1]]['humid']
                    else:
                        conditions = daily[dayparts[0]]['phrase']
                        pop = daily[dayparts[0]]['pop']
                        wind_speed = self._speed_converter(
                            daily[dayparts[0]]['wSpeed'], locale)
                        wind_direction = daily[dayparts[0]]['wDirText']
                        uv = daily[dayparts[0]]['uv']
                        humidity = daily[dayparts[0]]['humid']
            if len(results) == 0:
                # some results doesn't have StandardObservation key/value
                if 'StandardObservation' not in data[0]:
                    current_temp = "N/A"
                else:
                    current_temp = "%i%s" % (
                        self._temp_converter(
                            data[0]['StandardObservation']['temp'], locale),
                        u"\u00B0")
            else:
                current_temp = max_temp
                title = '%s\n%s' % (title, date)
            weather_code = f['icon']
            if weather_code in WEATHER_ICONS_MAP:
                iname = WEATHER_ICONS_MAP[weather_code]
                icon = '%s/weatherchannel/suru/%s.svg' % (self.base_image_url,
                                                          iname)
            else:
                icon = "http://s.imwx.com/img/wxicon/130/%s.png" % (
                    weather_code,)
            location = "%s, %s, %s" % (data[0]['Location']['name'],
                                       data[0]['Location']['state'],
                                       data[0]['Location']['countryCode'])
            if self._korm(locale) == "M":
                wind_unit = "mph"
            else:
                wind_unit = "km/h"
            wind_sentence = u"%s at %s %s" % (
                u'-'.join(wind_direction.split()), wind_speed, wind_unit)
            uri = self.get_localized_url(locale, formfactor, uri_date) + \
                '%s#%s?par=ubuntu' % (loc_key, len(results))
            results[realdate] = {
                'cat_id': CATEGORY_DEFAULT['id'],
                'uri': uri,
                'dnd_uri': uri,
                'title': title,
                'art': icon,
                'day': day,
                'comment': f['phrase'],
                'date': date,
                'max_temp': max_temp,
                'min_temp': min_temp,
                'weather_type_code': weather_code,
                'conditions': conditions,
                'pop': '%i%%' % (pop),
                'wind_sentence': wind_sentence,
                'uv': str(uv),
                'current_temp': current_temp,
                'humidity': '%i%%' % (humidity),
                'result_index': len(results),
                'location': location,
                'loc_key': loc_key,
                'locale': locale,
                'formfactor': formfactor,
            }

        results = [v for k, v in sorted(results.items())]
        results = results[:MAX_FORECAST_DAYS]

        # fake a result, with same data than the first ('today') one, but
        # with no title and the TWC logo, and put it at the left end
        logo_result = results[0].copy()
        logo_result['art'] = '%s/weatherchannel/suru/logo.png' % \
            (self.base_image_url,)
        logo_result['title'] = ''
        logo_result['uri'] = logo_result['uri'] + "#logo"
        results.insert(0, logo_result)

        # send the category we'll use
        yield dict(category=localized_category(CATEGORY_DEFAULT, locale))

        for result in results:
            yield dict(result=result)

    def preview(self, **kwargs):
        """The preview"""
        result = kwargs['result']
        locale = kwargs['locale']
        title = "%s: %s %s" % \
                (result['day'], result['conditions'], result['current_temp'])

        subtitle = "%s   %s" % (result['location'], result['date'])

        # yield an 'image' widget
        yield dict(widget={
            'id': 'art',
            'type': 'image',
            'source': result['art'],
        })

        # yield a 'header' widget
        yield dict(widget={
            'id': 'header1',
            'type': 'header',
            'uri': result['uri'],
            'title': title,
            'subtitle': subtitle,
            'mascot': self.base_image_url + '/weatherchannel/suru/logo.png',
        })

        max_temp = (translator("Maximum temperature", locale),
                    result['max_temp'])
        min_temp = (translator("Minimum temperature", locale),
                    result['min_temp'])
        rain = (translator("Chances of rain", locale), result['pop'])
        wind = (translator("Wind", locale), result['wind_sentence'])
        humm = (translator("Humidity", locale), result['humidity'])
        uv = (translator("UV index", locale), result['uv'])
        # send it as a 'text' widget
        infos = [min_temp, max_temp, rain, wind, humm, uv]
        text = '\n'.join('%s: %s' % (desc, value) for (desc, value) in infos)
        yield dict(widget={
            'id': 'text1',
            'type': 'text',
            'title': None,
            'text': text
        })
        loc_key = result['loc_key']
        formfactor = result['formfactor']
        # always add an action for the next day
        uri_tendays = self.get_localized_url(locale, formfactor, "tendays") + \
            "%s?par=ubuntu" % (loc_key,)
        actions = [{
            "id": "action1",
            "icon": None,
            "label": translator("10 days", locale),
            "uri": uri_tendays,
        }]

        # only add an "hourly" action if the forecast result was for the day
        if "today" in result['uri']:
            uri = self.get_localized_url(locale, formfactor, "hourly") + \
                "%s#0?par=ubuntu" % (loc_key,)
            actions.append({
                "id": "action2",
                "icon": None,
                "label": translator("Hourly", locale),
                'uri': uri,
            })

        yield dict(widget={
            "id": "actions",
            "type": "actions",
            "actions": actions,
        })
