# Copyright 2014 Canonical
# All Rights Reserved

"""The Wikipedia scope."""

import json
import re
import urllib
import feedparser

from urllib2 import urlopen, Request
from xml.dom.minidom import parseString

from ..scopes import SimpleApp
from ..translator import translator
from ..utils import localized_category
from .. import rest_scopes

# We use the opensearch action as that seems to be the only action that
# searches only on page names -- the standard search function performs a
# full-text search and may return less relevant pages first.
SEARCH_URL = (
    'http://%(language)s.wikipedia.org'
    '/w/api.php?action=opensearch&limit=%(limit)s&format=xml&search=%(query)s'
)
PREVIEW_URL = (
    'http://%(language)s.wikipedia.org/w/api.php?action=query&'
    'prop=extracts&exchars=%(text_len)s&titles=%(title)s&format=json'
)
FEATURED_URL = (
    'http://%(language)s.wikipedia.org'
    '/w/api.php?action=featuredfeed&limit=%(limit)s&format=xml&feed=featured'
)

# the categories to use, with some renderers
_renderer_featured = {
    "schema-version": 1,
    'template': {'category-layout': 'grid', 'card-size': 'small'},
    'components': {'title': 'title',
                   'art': 'art'},
}
CATEGORY_FEATURED = {
    'id': 'featured',
    'title': translator('Featured articles'),
    'render_template': json.dumps(_renderer_featured),
    'icon': "FIXME",
}
_renderer_default = {
    "schema-version": 1,
    "template": {
        "category-layout": "grid",
        "card-size": "medium",
    },
    "components": {
        "title": "title",
        "subtitle": "comment",
        "art": "art",
    },
}
CATEGORY_DEFAULT = {
    'id': 'default_category',
    'title': translator('Results'),
    'render_template': json.dumps(_renderer_default),
    'icon': "FIXME",
}

# these are the default image sizes for the search and preview
IMAGE_DISPLAY_SIZE = '96'
IMAGE_PREVIEW_SIZE = '380'

# the regex to replace the image size in the returned URL
RE_IMAGE_SIZE = re.compile(r'(?<=\/)\d+(?=px-)')
RE_FEATURED_TITLE = re.compile(r'\stitle\=\"(.*?)\">[^<]')
RE_FEATURED_IMAGE = re.compile(r'.*(http:\/\/.*(\.jpg|\.png)).*')

HEADERS = {
    'User-Agent': "Ubuntu Rest Server - Wikipedia scope",
    'Accept': '*/*',
}


def _get_featured(limit, language):
    """Hit Wikipedia for featured articles."""
    url = FEATURED_URL % dict(language=language, limit=limit)
    feed = feedparser.parse(url)
    return feed


def _get_search(query, limit, language):
    """Hit Wikipedia to do a search."""
    query = urllib.quote(query.encode("utf8"))
    url = SEARCH_URL % dict(language=language, limit=limit, query=query)
    req = Request(url, headers=HEADERS)
    return urlopen(req).read()


def _get_preview(title, text_len, language):
    """Hit Wikipedia to do a preview."""
    url = PREVIEW_URL % dict(language=language, text_len=text_len, title=title)
    req = Request(url, headers=HEADERS)
    return urlopen(req).read()


def get_lang_from_locale(locale):
    """Get the language part from the locale."""
    if locale is None:
        language = 'en'
    else:
        language = locale.split('_', 1)[0]
    return language


def _parse_feed(feed, language):
    """Parse the featured RSS response."""
    feed['entries'].reverse()
    for item in feed['entries']:
        # Look for page image and title in the featured item
        try:
            img = RE_FEATURED_IMAGE.match(item['summary']).group(1)
        except AttributeError:
            art = None
        else:
            art = RE_IMAGE_SIZE.sub(IMAGE_DISPLAY_SIZE, img)
        title = RE_FEATURED_TITLE.findall(item['summary'])[0]
        # Build the page uri
        url = ('http://%(locale)s.wikipedia.org/'
               'wiki/%(title)s' %
               dict(title=urllib.quote(title.encode("utf8")),
                    locale=language))
        cat_id = CATEGORY_FEATURED['id']
        yield art, title, url, cat_id


def _parse_xml(raw):
    """Parse the search based XML response."""
    xml = parseString(raw)
    for item in xml.getElementsByTagName('Item'):
        img = item.getElementsByTagName('Image')
        if img:
            # the image for the search result
            img_source = img[0].getAttribute("source")
            art = RE_IMAGE_SIZE.sub(IMAGE_DISPLAY_SIZE, img_source)
        else:
            art = None
        title = item.getElementsByTagName('Text')[0].firstChild.data
        url = item.getElementsByTagName('Url')[0].firstChild.data
        cat_id = CATEGORY_DEFAULT['id']
        yield art, title, url, cat_id


def _get_status_code(uri):
    """Return HTTP status code of uri."""
    return urllib.urlopen(uri).getcode()


class App(SimpleApp):
    """The Wikipedia app."""

    def __init__(self, *args, **kwargs):
        super(App, self).__init__(*args, **kwargs)
        config = rest_scopes.get_scopes_config()['wikipedia']
        self.icon = self.config['base_image_url'] + '/' + config['icon']

    def search(self, **kwargs):
        """Do the search."""
        query = kwargs['query']
        locale = kwargs['locale']
        limit = kwargs['limit']
        language = get_lang_from_locale(locale)

        # do the search, and get the result as an xml
        if not query.strip():
            yield dict(category=localized_category(CATEGORY_FEATURED, locale))
            feed = _get_featured(limit, language)
            info = _parse_feed(feed, language)
        else:
            yield dict(category=localized_category(CATEGORY_DEFAULT, locale))
            raw = _get_search(query, limit, language)
            info = _parse_xml(raw)

        # Create the result
        for art, title, url, cat_id in info:
            if not art:
                art = self.icon
            yield dict(result={
                'cat_id': cat_id,
                'uri': url,
                'dnd_uri': url,
                'title': title,
                'art': art,
            })

    def preview(self, **kwargs):
        """Do the preview."""
        result = kwargs['result']
        locale = kwargs['locale']
        language = get_lang_from_locale(locale)

        # some info from the previous search result
        title = result['title']
        art = result['art']
        uri = result['uri']
        mascot = art

        # if Wikimedia returns ok for the bigger art,
        # send an image widget and cancel the header mascot
        bigger_art = RE_IMAGE_SIZE.sub(IMAGE_PREVIEW_SIZE, art)
        if _get_status_code(bigger_art) == 200:
            mascot = None
            # yield an 'image' widget
            yield dict(widget={
                'id': 'art',
                'type': 'image',
                'source': bigger_art,
            })

        # yield a 'header' widget
        yield dict(widget={
            'id': 'header1',
            'type': 'header',
            'uri': uri,
            'title': title,
            'mascot':  mascot,
        })

        # get a longer text
        title = urllib.quote(title.encode("utf8"))
        raw = _get_preview(title, 200, language)
        data = json.loads(raw)
        extract = data['query']['pages'].values()[0]['extract']

        # send it as a 'text' widget
        yield dict(widget={
            'id': 'text1',
            'type': 'text',
            'uri': uri,
            'title': None,
            'text': extract,
        })

        # add an 'actions' widget
        yield dict(widget={
            'id': 'buttons',
            'type': 'actions',
            'actions': [{'label': translator('View on Wikipedia', locale),
                         'uri': uri,
                         'id': 'action1'}]
        })
