# -*- coding: utf-8 -*-
# Copyright 2014 Canonical
# All Rights Reserved


"""The Yahoo Finance scope."""

import urllib
import json
import ast
import feedparser

from urllib2 import urlopen

from ..scopes import SimpleApp
from ..translator import translator
from ..utils import localized_category

BASE_URI = 'http://finance.yahoo.com'
STOCKTWITS_URI = 'https://api.stocktwits.com/api/2/trending/symbols.json'
SYMBOL_NEWS_URL = "http://finance.yahoo.com/rss/headline?s=%(symbol)s"
SYMBOL_LOOKUP_URL = "http://d.yimg.com/autoc.finance.yahoo.com/autoc?" \
    "query=%(query)s&callback=YAHOO.Finance.SymbolSuggest.ssCallback"
NEWS_URL = "http://finance.yahoo.com/rss/topfinstories"

RENDERER_QUOTES_SURFACING = {
    "schema-version": 1,
    'template': {'category-layout': 'grid',
                 'card-size': 'medium',
                 'card-layout': 'vertical',
                 'collapsed-rows': 2},
    'components': {'title': 'title',
                   'mascot': 'graph',
                   'subtitle': 'change_p', },
}

RENDERER_QUOTES = json.dumps({
    "schema-version": 1,
    'template': {'category-layout': 'grid',
                 'card-size': 'large',
                 'card-layout': 'horizontal'},
    'components': {'title': 'title',
                   'mascot': 'graph',
                   'subtitle': 'change_p', },
})

RENDERER_NEWS = {
    "schema-version": 1,
    'template': {'category-layout': 'grid',
                 'card-size': 'large',
                 'card-layout': 'horizontal'},
    'components': {'title': 'title',
                   'subtitle': 'date', },
}

TRENDING_CATEGORY = {
    'id': 'trending',
    'title': translator('Trending on Twitter'),
    'render_template': json.dumps(RENDERER_QUOTES_SURFACING),
}

HEADLINES_CATEGORY = {
    'id': 'headlines',
    'title': translator('Headlines'),
    'render_template': json.dumps(RENDERER_NEWS),
}


def _get_symbol_from_name(query):
    """Get symbols for a name search."""
    symbols = {}
    uri = SYMBOL_LOOKUP_URL % dict(query=query)
    response = urlopen(uri).read().encode('utf-8')
    # Get rid of the callback
    cleaned = response[response.index("(") + 1: response.rindex(")")]
    r = json.loads(cleaned)
    symbols_query = []
    for s in r['ResultSet']['Result']:
        symbols_query.append(s['symbol'])
        try:
            exchange = s['exchDisp']
        except KeyError:
            exchange = "other"
        symbols[s['symbol']] = {'exchange': exchange, 'name': s['name']}
    return " ".join(symbols_query), symbols


def _get_trending_symbols(limit):
    """Get symbols trending on Twitter with StockTwits."""
    uri = STOCKTWITS_URI + "?limit=" + str(limit)
    response = urlopen(uri).read()
    r = json.loads(response.encode('utf-8'))
    symbols = ' '.join([s['symbol'] for s in r['symbols']])
    return symbols


def _get_quotes(url):
    """Call the Yahoo Finance API."""
    results = []
    response = urlopen(url).read()
    for r in response.strip().split('\r\n'):
        try:
            quote = ast.literal_eval(r)
        except ValueError:
            continue
        result = {}
        result['symbol'] = quote[0]
        result['name'] = quote[1]
        result['value'] = quote[2]
        result['date'] = quote[3]
        result['time'] = quote[4]
        result['volume'] = quote[5]
        result['change'] = quote[6]
        results.append(result)
    return results


def _get_news(symbol):
    """Get news for symbol or top news from Yahoo! Finance rss feeds."""
    stories = []
    if symbol:
        url = SYMBOL_NEWS_URL % dict(symbol=symbol)
    else:
        url = NEWS_URL
    d = feedparser.parse(url)
    if not d:
        return stories
    for entry in d.entries:
        story = {}
        # Discard non-free articles
        if "title" not in entry.keys() or entry.title.startswith('[$$]'):
            continue
        if "link" not in entry.keys():
            continue
        if "published" in entry.keys():
            story["date"] = entry.published
        else:
            story["date"] = entry.updated
        story['title'] = entry.title
        story['uri'] = entry.link
        story['summary'] = entry.summary
        stories.append(story)
    return stories


class App(SimpleApp):
    """The Yahoo Finance app."""

    def search(self, **kwargs):
        """Do the search."""
        query = kwargs['query'].encode('utf-8')
        limit = kwargs['limit']
        locale = kwargs['locale']
        headlines = []
        surfacing = False
        if not query.strip():
            surfacing = True
            query = _get_trending_symbols(limit)
            symbol_map = {}
            d = dict(base_uri=BASE_URI, quotes=urllib.quote(query))
            uri = '%(base_uri)s/d/quotes.csv?s=%(quotes)s&f=snl1d1t1vc' % d
            quotes = _get_quotes(uri)
            headlines = _get_news(None)
        else:
            symbol_map = {}
            query_pieces = []
            for q in [query]:
                q_part, map_part = _get_symbol_from_name(urllib.quote(q))
                query_pieces.append(q_part)
                symbol_map = dict(symbol_map.items() + map_part.items())
            query_string = " ".join(query_pieces)
            d = dict(base_uri=BASE_URI, quotes=urllib.quote(query_string))
            uri = '%(base_uri)s/d/quotes.csv?s=%(quotes)s&f=snl1d1t1vc' % d
            quotes = _get_quotes(uri)
            sent_exchanges = []
            for s in symbol_map:
                if symbol_map[s]['exchange'] and \
                        not symbol_map[s]['exchange'] in sent_exchanges:
                    cat = {
                        'render_template': RENDERER_QUOTES,
                        'id': symbol_map[s]['exchange'],
                        'title': symbol_map[s]['exchange'],
                    }
                    yield dict(category=cat)
                    sent_exchanges.append(symbol_map[s]['exchange'])
            other_quotes = {
                'render_template': RENDERER_QUOTES,
                'id': "other",
                'title': translator("Other exchanges", locale),
            }
            yield dict(category=other_quotes)
        if headlines:
            yield dict(category=localized_category(HEADLINES_CATEGORY, locale))
            for h in headlines:
                    yield dict(result={
                        'cat_id': 'headlines',
                        'uri': h['uri'],
                        'dnd_uri': h['uri'],
                        'title': h['title'],
                        'date': h['date'],
                        'summary': h['summary'],
                    })
        if surfacing:
            yield dict(category=localized_category(TRENDING_CATEGORY, locale))
        for quote in quotes:
            graph = "http://chart.finance.yahoo.com/z?" \
                "s=" + quote['symbol'] + "&z=s&t=7d"
            value_change = quote['change'].split(' ')[0]
            percent_change = quote['change'].split(' ')[-1]
            uri = 'http://finance.yahoo.com/q?s=' + quote['symbol']
            if surfacing:
                cat_id = "trending"
            else:
                if quote['symbol'] in symbol_map:
                    cat_id = symbol_map[quote['symbol']]['exchange']
                else:
                    cat_id = "other"
            yield dict(result={
                'uri': uri,
                'dnd_uri': uri,
                'graph': graph,
                'cat_id': cat_id,
                'title': quote['name'],
                'value': str(quote['value']),
                'symbol': quote['symbol'],
                'change_v': value_change,
                'change_p': percent_change,
            })

    def preview(self, **kwargs):
        """Do the preview."""
        result = kwargs['result']
        locale = kwargs['locale']
        if result['cat_id'] != 'headlines':
            graph = "http://chart.finance.yahoo.com/z?" \
                    "s=" + result['symbol'] + "&z=s&t=7d"

            if result['cat_id'] != 'headlines':
                # yield an 'image' widget
                yield dict(widget={
                    'id': 'art',
                    'type': 'image',
                    'source': graph,
                })

            # yield a 'header' widget
            yield dict(widget={
                'id': 'header',
                'type': 'header',
                'title': result['title'],
                'subtitle': result['symbol'],
            })
        else:
            # yield a 'header' widget
            yield dict(widget={
                'id': 'header',
                'type': 'header',
                'title': result['title'],
                'subtitle': result['date'],
            })

        # yield an 'actions' widget
        yield dict(widget={
            'id': 'buttons',
            'type': 'actions',
            'actions': [{'uri': result['uri'],
                         'label': translator('Yahoo! Finance', locale),
                         'id': 'yahoo'}]
        })

        if result['cat_id'] == 'headlines':
            # yield an 'text' widget
            yield dict(widget={
                'id': 'text',
                'type': 'text',
                'text': result['summary'],
            })
