# Copyright 2014 Canonical
# All Rights Reserved

"""Tests for the Rest Scopes server."""

import json
import uuid
import os

from unittest import TestCase

from mock import Mock, patch

from src import rest_scopes


class InitTestCase(TestCase):
    """Test initialization."""

    def test_set_scope_config(self):
        secrets = {'wikipedia': {'foo': 'bar'}}
        rest_scopes._get_scopes_secrets = Mock(return_value=secrets)
        app = rest_scopes.App()
        wikipedia = app.scope_apps['wikipedia']

        should_config = rest_scopes.config.copy()
        should_config['secrets'] = {'foo': 'bar'}
        self.assertEqual(wikipedia.config, should_config)

    def test_extra_scope_config(self):
        extra = {'wikipedia': {'foo': 'bar'}}
        app = rest_scopes.App(extra)

        # wikipedia config is changed
        wikipedia = app.scope_apps['wikipedia']
        self.assertEqual(wikipedia.config['foo'], 'bar')

        # but the other scopes' isn't
        other_scope = app.scope_apps['ebay']
        self.assertNotIn('foo', other_scope.config)


class CallTestCase(TestCase):
    """Test when calling the class."""

    def test_list_remote_scopes(self):
        app = rest_scopes.App()
        app.list_remote_scopes = mock = Mock()
        mock.return_value = 'response'

        environ = {
            'PATH_INFO': '/remote-scopes',
        }
        resp = app(environ, 'start_response')
        self.assertEqual(resp, 'response')
        mock.assert_called_with(environ, 'start_response')

    def test_scope_not_found(self):
        app = rest_scopes.App()
        app._error = mock = Mock()
        mock.return_value = 'error response'

        environ = {
            'PATH_INFO': '/not_a_scope',
        }
        resp = app(environ, 'start_response')
        self.assertEqual(resp, 'error response')
        mock.assert_called_with('404 NOT FOUND', 'start_response')

    def test_scope_with_bad_path(self):
        app = rest_scopes.App()
        app._error = mock = Mock()
        mock.return_value = 'error response'

        assert 'wikipedia' in app.scope_apps
        environ = {
            'PATH_INFO': '/wikipedia',
        }
        resp = app(environ, 'start_response')
        self.assertEqual(resp, 'error response')
        msg = "Malformed URL: need an action after the scope name"
        mock.assert_called_with('400 BAD REQUEST', 'start_response', msg)

    def test_scope_ok(self):
        app = rest_scopes.App()

        app.scope_apps['somescope'] = app_mock = Mock()
        app_mock.return_value = "some response from the scope"
        environ = {
            'PATH_INFO': '/somescope/somemethod',
            'FOO': 'bar',
        }
        resp = app(environ, 'start_response')
        self.assertEqual(resp, "some response from the scope")
        new_environ = {
            'PATH_INFO': '/somemethod',
            'FOO': 'bar',
        }
        app_mock.assert_called_with(new_environ, 'start_response')


class ListRemoteScopesTestCase(TestCase):
    """Test the remote scopes listing."""

    def test_blacklist(self):
        rest_scopes.config['blacklist'] = {'FR': ['foo']}
        app = rest_scopes.App()
        app._generic_response = Mock()
        app._generic_response.return_value = 'generic response result'

        # set up the original data from the config
        app.scope_info[None] = [{
            'id': 'foo',
            'description': 'description',
            'name': 'name',
        }, {
            'id': 'bar',
            'description': 'description',
            'name': 'name',
        }]

        environ = {'QUERY_STRING': 'country=FR'}

        # Ensure blacklisted scopes are not returned
        final_scope_list = [{
            'id': 'bar',
            'description': 'description',
            'name': 'name',
        }]
        app.list_remote_scopes(environ, 'start_response')
        args, kwargs = app._generic_response.call_args
        self.assertEqual(args[0], "200 OK")
        self.assertEqual(args[1], 'start_response')
        self.assertEqual(json.loads(args[2]), final_scope_list)

    def test_call_simple(self):
        app = rest_scopes.App()
        app._generic_response = Mock()
        app._generic_response.return_value = 'generic response result'

        # set up the original data from the config
        app.scope_info[None] = [{
            'foo': 1,
            'description': 'description 1',
            'name': 'name 1',
            'bar': 1,
        }, {
            'foo': 2,
            'description': 'description 2',
            'name': 'name 2',
            'bar': 2,
        }]

        environ = {'QUERY_STRING': 'locale=es_AR'}

        def fake_translator(text, locale):
            """Translate in a funny way."""
            return "translated-%s %s" % (locale, text)

        with patch.object(rest_scopes, 'translator', fake_translator):
            resp = app.list_remote_scopes(environ, 'start_response')

        # check method returned proper info
        final_scope_list = [{
            'foo': 1,
            'description': 'translated-es_AR description 1',
            'name': 'translated-es_AR name 1',
            'bar': 1,
        }, {
            'foo': 2,
            'description': 'translated-es_AR description 2',
            'name': 'translated-es_AR name 2',
            'bar': 2,
        }]
        self.assertEqual(resp, "generic response result")
        app._generic_response.assert_called_with(
            "200 OK", 'start_response', json.dumps(final_scope_list))

        # verify we had stored the translation for future usage
        self.assertEqual(app.scope_info['es_AR'], final_scope_list)

    def test_call_dont_translate_already_translated(self):
        app = rest_scopes.App()
        app._generic_response = Mock()
        app._generic_response.return_value = 'generic response result'

        # set up the original data from the config, already translated
        scope_list = [{
            'foo': 1,
            'description': 'translated 1',
            'bar': 1,
        }, {
            'foo': 2,
            'description': 'translated 2',
            'bar': 2,
        }]
        app.scope_info['es_AR'] = scope_list

        environ = {'QUERY_STRING': 'locale=es_AR'}

        with patch.object(rest_scopes, 'translator') as trans_mock:
            resp = app.list_remote_scopes(environ, 'start_response')

        self.assertFalse(trans_mock.called)
        self.assertEqual(resp, "generic response result")
        app._generic_response.assert_called_with(
            "200 OK", 'start_response', json.dumps(scope_list))

    def test_empty(self):
        with patch.object(rest_scopes, 'get_scopes_config', lambda: {}):
            app = rest_scopes.App()
        self.assertEqual(app.scope_info[None], [])

    def test_internal(self):
        config = {
            'somescope': {
                'name': 'name',
                'description': 'desc',
                'author': 'John Doe',
                'base_url': '/somescope',
                'invisible': False,
            }
        }
        with patch.object(rest_scopes, 'get_scopes_config', lambda: config):
            with patch('importlib.import_module') as mock:
                app = rest_scopes.App()
                mock.assert_called_with("src.scopes.somescope")
        result = [{
            'id': 'somescope',
            'name': 'name',
            'description': 'desc',
            'author': 'John Doe',
            'base_url':  "http://localhost:8000/somescope",
            'invisible': False,
            'icon': None,
            'art': None,
            'appearance': {},
        }]
        self.assertEqual(app.scope_info[None], result)

    def test_external(self):
        config = {
            'somescope': {
                'name': 'name',
                'description': 'desc',
                'author': 'John Doe',
                'base_url': 'http://server.com/somescope',
                'invisible': False,
            }
        }
        with patch.object(rest_scopes, 'get_scopes_config', lambda: config):
            app = rest_scopes.App()
        result = [{
            'id': 'somescope',
            'name': 'name',
            'description': 'desc',
            'author': 'John Doe',
            'base_url': 'http://server.com/somescope',
            'invisible': False,
            'icon': None,
            'art': None,
            'appearance': {},
        }]
        self.assertEqual(app.scope_info[None], result)

    def test_fqdn(self):
        config = {
            'somescope': {
                'name': 'name',
                'description': 'desc',
                'author': 'John Doe',
                'base_url': 'http://server.com/somescope',
                'invisible': False,
                'fqdn': 'some.fqdn.name.somescope',
            }
        }
        with patch.object(rest_scopes, 'get_scopes_config', lambda: config):
            app = rest_scopes.App()
        result = [{
            'id': 'some.fqdn.name.somescope',
            'name': 'name',
            'description': 'desc',
            'author': 'John Doe',
            'base_url': 'http://server.com/somescope',
            'invisible': False,
            'icon': None,
            'art': None,
            'appearance': {},
        }]
        self.assertEqual(app.scope_info[None], result)

    def test_strip_desc_name(self):
        config = {
            'somescope': {
                'name': ' name',
                'description': 'desc\n',
                'author': 'John Doe',
                'base_url': 'http://server.com/somescope',
                'invisible': False,
                'fqdn': 'some.fqdn.name.somescope',
            }
        }
        with patch.object(rest_scopes, 'get_scopes_config', lambda: config):
            app = rest_scopes.App()
        returned = app.scope_info[None][0]
        self.assertEqual(returned['name'], 'name')
        self.assertEqual(returned['description'], 'desc')

    def test_internal_otherpath(self):
        config = {
            'somescope': {
                'name': 'name',
                'description': 'desc',
                'author': 'John Doe',
                'base_url': '/somescope',
                'invisible': False,
                'mod_package': 'src.other',
            }
        }
        with patch.object(rest_scopes, 'get_scopes_config', lambda: config):
            with patch('importlib.import_module') as mock:
                app = rest_scopes.App()
                mock.assert_called_with("src.other.somescope")
        result = [{
            'id': 'somescope',
            'name': 'name',
            'description': 'desc',
            'author': 'John Doe',
            'base_url':  "http://localhost:8000/somescope",
            'invisible': False,
            'icon': None,
            'art': None,
            'appearance': {},
        }]
        self.assertEqual(app.scope_info[None], result)

    def test_icon_art(self):
        config = {
            'somescope': {
                'name': 'name',
                'description': 'desc',
                'author': 'John Doe',
                'base_url': 'http://server.com/somescope',
                'invisible': False,
                'icon': 'somedir/foo-icon.png',
                'art': 'somedir/foo-art.png',
            }
        }
        with patch.object(rest_scopes, 'get_scopes_config', lambda: config):
            app = rest_scopes.App()
        result = [{
            'id': 'somescope',
            'name': 'name',
            'description': 'desc',
            'author': 'John Doe',
            'base_url': 'http://server.com/somescope',
            'invisible': False,
            'icon': 'http://localhost:8000/_static/somedir/foo-icon.png',
            'art': 'http://localhost:8000/_static/somedir/foo-art.png',
            'appearance': {},
        }]
        self.assertEqual(app.scope_info[None], result)

    def test_appearance(self):
        config = {
            'somescope': {
                'name': 'name',
                'description': 'desc',
                'author': 'John Doe',
                'base_url': 'http://server.com/somescope',
                'invisible': False,
                'appearance': {
                    'bgcolor': '#ffffff',
                    'page-header': {'logo': 'logo.png'},
                },
            }
        }
        with patch.object(rest_scopes, 'get_scopes_config', lambda: config):
            app = rest_scopes.App()
        result = [{
            'id': 'somescope',
            'name': 'name',
            'description': 'desc',
            'author': 'John Doe',
            'base_url': 'http://server.com/somescope',
            'invisible': False,
            'icon': None,
            'art': None,
            'appearance': {
                'bgcolor': '#ffffff',
                'page-header': {
                    'logo': 'http://localhost:8000/_static/logo.png',
                },
            },
        }]
        self.assertEqual(app.scope_info[None], result)


class HelpersTestCase(TestCase):
    """Test some helper methods."""

    def setUp(self):
        self.app = rest_scopes.App()
        self.start_response = Mock()

    def test_generic_response_sanity(self):
        resp = self.app._generic_response("200 test", self.start_response,
                                          "test message", header='extra')
        self.assertEqual(resp, ["test message"])
        should_status = "200 test"
        should_headers = [
            ('Content-type', "application/json"),
            ('Content-Length', str(len("test message"))),
            rest_scopes.REVNO_HEADER,
            ('header', 'extra'),
        ]
        self.start_response.assert_called_with(should_status, should_headers)

    def test_error_sanity(self):
        resp = self.app._error("400 test", self.start_response,
                               "test message", header='extra')
        should_error_message = json.dumps(
            {'error': '400 test', 'message': 'test message'}, indent=4)

        self.assertEqual(resp, [should_error_message])
        should_status = "400 test"
        should_headers = [
            ('Content-type', "application/json"),
            ('Content-Length', str(len(should_error_message))),
            rest_scopes.REVNO_HEADER,
            ('header', 'extra'),
        ]
        self.start_response.assert_called_with(should_status, should_headers)

    def test_get_scopes_config_simple(self):
        config_copy = rest_scopes.config.copy()
        test_fname = "test-" + str(uuid.uuid4())
        with open(test_fname, 'wt') as fh:
            fh.write("foo: bar\n")
        self.addCleanup(os.remove, test_fname)

        config_copy['scopes_config'] = test_fname
        with patch.object(rest_scopes, 'config', config_copy):
            loaded = rest_scopes.get_scopes_config()

        self.assertEqual(loaded, {'foo': 'bar'})

    def test_get_scopes_config_composed(self):
        config_copy = rest_scopes.config.copy()
        test_fname_1 = "test-" + str(uuid.uuid4())
        with open(test_fname_1, 'wt') as fh:
            fh.write("aaa: 111\nbbb: 222\n")
        self.addCleanup(os.remove, test_fname_1)
        test_fname_2 = "test-" + str(uuid.uuid4())
        with open(test_fname_2, 'wt') as fh:
            fh.write("bbb: 777\nccc: 333\n")
        self.addCleanup(os.remove, test_fname_2)

        config_copy['scopes_config'] = [test_fname_1, test_fname_2]
        with patch.object(rest_scopes, 'config', config_copy):
            loaded = rest_scopes.get_scopes_config()

        self.assertEqual(loaded, {'aaa': 111, 'bbb': 777, 'ccc': 333})
