# Copyright 2014 Canonical
# All Rights Reserved

"""Tests for translator."""

import unittest

from mock import patch, Mock

import translator


class SuperDictTestCase(unittest.TestCase):
    """Test the internationalization through the translator."""

    def test_lang_autofill(self):
        trans = translator.Translator()

        # it shouldn't be there, ask for it
        self.assertNotIn('en', trans)
        gc1 = trans['en']

        # now it *should* be there, ask for it again
        self.assertIn('en', trans)
        gc2 = trans['en']

        # it should be the same instance, from the cache
        self.assertIs(gc1, gc2)
        self.assertIsInstance(gc1, translator.GettextCache)

    def test_gettextcaching_gettextbackend(self):
        # let's get a cache letting it know that the language is present
        trans = Mock()
        ugettext = Mock()
        trans.ugettext = ugettext
        with patch.object(translator.gettext, 'translation') as mock:
            mock.return_value = trans
            gc = translator.GettextCache('en', ['locale_dir'])
            mock.assert_called_with('core', 'locale_dir', ['en'])

        # first time we call it, it should get the translation from gettext
        trans.ugettext.return_value = u'translated'
        self.assertNotIn(u'foo', gc)
        self.assertEqual(gc[u'foo'], u'translated')
        trans.ugettext.assert_called_with('foo')
        trans.ugettext.assert_calling_count(1)

        # second time it should be cached
        self.assertIn(u'foo', gc)
        self.assertEqual(gc[u'foo'], u'translated')
        trans.ugettext.assert_calling_count(1)  # not called again

    def test_gettextcaching_missinglanguage(self):
        # let's get a cache for an unknown language
        gc = translator.GettextCache('weird_lang', 'locale_dir')

        # first time we call it, just return the original string
        self.assertNotIn(u'foo', gc)
        self.assertEqual(gc[u'foo'], u'foo')

        # second time it should be cached
        self.assertIn(u'foo', gc)
        self.assertEqual(gc[u'foo'], u'foo')

    def test_gettextcaching_multipledirs_initial(self):
        # generate two translations to be returned for different dirs
        trans1 = Mock()
        trans1.ugettext = Mock()
        trans1.ugettext.return_value = u'translated'
        trans2 = Mock()
        trans2.ugettext = Mock()
        both_translators = [trans1, trans2]

        with patch.object(translator.gettext, 'translation') as mock:
            mock.side_effect = both_translators
            gc = translator.GettextCache('en', ['direct1', 'direct2'])

        # check it was called twice to get both translators
        self.assertEqual(mock.call_args_list[0],
                         (('core', 'direct1', ['en']), {}))
        self.assertEqual(mock.call_args_list[1],
                         (('core', 'direct2', ['en']), {}))

        # call it, check it only used first translator
        self.assertNotIn(u'foo', gc)
        self.assertEqual(gc[u'foo'], u'translated')
        trans1.ugettext.assert_called_with('foo')
        self.assertFalse(trans2.ugettext.called)
        self.assertIn(u'foo', gc)

    def test_gettextcaching_multipledirs_second(self):
        # generate two translations to be returned for different dirs
        trans1 = Mock()
        trans1.ugettext = Mock()
        trans1.ugettext.return_value = u'foo'
        trans2 = Mock()
        trans2.ugettext = Mock()
        trans2.ugettext.return_value = u'translated'
        both_translators = [trans1, trans2]

        with patch.object(translator.gettext, 'translation') as mock:
            mock.side_effect = both_translators
            gc = translator.GettextCache('en', ['direct1', 'direct2'])

        # check it was called twice to get both translators
        self.assertEqual(mock.call_args_list[0],
                         (('core', 'direct1', ['en']), {}))
        self.assertEqual(mock.call_args_list[1],
                         (('core', 'direct2', ['en']), {}))

        # call it, check it got to the second translator
        self.assertNotIn(u'foo', gc)
        self.assertEqual(gc[u'foo'], u'translated')
        trans1.ugettext.assert_called_with('foo')
        trans2.ugettext.assert_called_with('foo')
        self.assertIn(u'foo', gc)

    def test_gettextcaching_multipledirs_fallback(self):
        # generate two translations to be returned for different dirs
        trans1 = Mock()
        trans1.ugettext = Mock()
        trans1.ugettext.return_value = u'foo'
        trans2 = Mock()
        trans2.ugettext = Mock()
        trans2.ugettext.return_value = u'foo'
        both_translators = [trans1, trans2]

        with patch.object(translator.gettext, 'translation') as mock:
            mock.side_effect = both_translators
            gc = translator.GettextCache('en', ['direct1', 'direct2'])

        # check it was called twice to get both translators
        self.assertEqual(mock.call_args_list[0],
                         (('core', 'direct1', ['en']), {}))
        self.assertEqual(mock.call_args_list[1],
                         (('core', 'direct2', ['en']), {}))

        # call it, check it fallbacked to return key
        self.assertNotIn(u'foo', gc)
        self.assertEqual(gc[u'foo'], u'foo')
        trans1.ugettext.assert_called_with('foo')
        trans2.ugettext.assert_called_with('foo')
        self.assertIn(u'foo', gc)
