# Copyright 2014 Canonical
# All Rights Reserved

"""Translate using gettext source files but for all langs simultaneously."""

import gettext
import os
import yaml

DOMAIN = 'core'

# load general config
with open(os.environ['REST_SCOPES_CONFIG']) as fh:
    config = yaml.load(fh)


class GettextCache(dict):
    """Cache gettext results, or fake them if not available."""
    def __init__(self, lang, locale_dirs):
        super(GettextCache, self).__init__()

        # get a gettext backend if any, else fake it
        self._trans_funcs = []
        for direct in locale_dirs:
            try:
                t = gettext.translation("core", direct, [lang])
            except IOError:
                pass
            else:
                self._trans_funcs.append(t.ugettext)

        # set a default identity function at the end of the chain
        self._trans_funcs.append(lambda key: key)

    def __missing__(self, key):
        for func in self._trans_funcs:
            translated = func(key)
            if translated != key:
                break
        self[key] = translated
        return translated


class Translator(dict):
    """Handle the gettext interface for each language."""
    def __init__(self):
        super(Translator, self).__init__()
        self._locale_dirs = config['locale_dirs']

    def __missing__(self, key):
        self[key] = gc = GettextCache(key, self._locale_dirs)
        return gc

_translator = Translator()


def translator(text, locale=None):
    """Translate the text by the given locale.

    Support only receiving the text, useful for marking strings to be
    picked up by xgettext.
    """
    if locale is None:
        return text
    return _translator[locale][text]
