#!/usr/bin/python2

from __future__ import unicode_literals, print_function

import sys
import time
import urllib2

import bottlenose
import bs4
import yaml

from codecs import open
from lxml import etree

NODES_URL = (
    "http://docs.aws.amazon.com/"
    "AWSECommerceService/latest/DG/BrowseNodeIDs.html"
)

BLACKLIST = {"DigitalMusic", "MobileApps"}

SHOP_TAGS = {
    "CA": "u1treatyrslf-ca-20",
    "CN": "u1treatyrslf-23",
    "DE": "u1treatyrslf-21",
    "ES": "u1treatyrslf-es-21",
    "FR": "u1treatyrslf-fr-21",
    "IT": "u1treatyrslf-it-21",
    "JP": "u1treatyrslf-22",
    "UK": "u1treatyrslf-uk-21",
    "US": "u1treatyrslf-20",
}
AMZ_VERSION = "2011-08-01"
AMZ_NS = {'x': "http://webservices.amazon.com/AWSECommerceService/2011-08-01"}


def retrier(func):
    """A decorator that will retry calling if gets HTTP error 503."""
    delays = [1, 3, 5, 10, 30, 30, 30, None]
    def _f(*args, **kwargs):
        for delay in delays:
            try:
                return func(*args, **kwargs)
            except urllib2.HTTPError as err:
                if err.code == 503 and delay is not None:
                    time.sleep(delay)
                    continue
                raise
    return _f


class Amazon:
    """The interface to get Amazon info through the API."""

    def __init__(self, shop, tag):
        with open('configs/dev-secrets.yaml', 'rt', encoding='utf8') as fh:
            config = yaml.load(fh)['amazon']
        self.amz = bottlenose.Amazon(config['key'], config['secret'],
                                     Version=AMZ_VERSION,
                                     AssociateTag=tag, Region=shop)

    @retrier
    def get_category_info(self, cat_id):
        """Get id and name of the root of the given category."""
        raw = self.amz.BrowseNodeLookup(BrowseNodeId=cat_id,
                                        ResponseGroup='BrowseNodeInfo')
        xml = etree.XML(raw)

        # check if all is ok
        error = xml.xpath('//x:Error', namespaces=AMZ_NS)
        if error:
            err_info = {e.tag.split('}')[1]: e.text for e in error[0]}
            print("    WARNING: Amazon failed with "
                  "{Code!r} ({Message})".format(**err_info))
            return None

        ancestors = xml.xpath("//x:Ancestors", namespaces=AMZ_NS)
        if ancestors:
            # the indicated category was not a root; get last
            # ancestor, as root is deeper
            root = ancestors[-1][0]
        else:
            # the indicated category *is* a root, get its name
            root = xml.xpath("//x:BrowseNodes/x:BrowseNode",
                             namespaces=AMZ_NS)[0]

        data = {e.tag.split('}')[1]: e.text for e in root}
        return data


def main(dest_file):
    """Main entry point."""
    # get the raw categories info
    html = urllib2.urlopen(NODES_URL).read()
    soup = bs4.BeautifulSoup(html)
    table = soup.find('div', class_='informaltable')
    rows = table.find_all('tr')
    shops = [x.text.strip() for x in rows[0].find_all('th')[1:]]
    raw_info = {k: [] for k in shops}
    for row in rows[1:]:
        fields = [x.text.strip() for x in row.find_all('td')]
        cat_code = fields[0]
        if cat_code in BLACKLIST:
            continue
        assert len(shops) == len(fields) - 1
        for shop, cat_id  in zip(shops, fields[1:]):
            if cat_id:
                raw_info[shop].append((cat_code, cat_id))

    # for each shop, get the name for each category, and save it
    fh = open(dest_file, 'wt', encoding='utf8')
    for shop, categories in raw_info.items():
        try:
            tag = SHOP_TAGS[shop]
        except KeyError:
            print("WARNING: Missing tag for shop", repr(shop))
            continue
        amz = Amazon(shop, tag)

        print("Getting info for", repr(shop), categories)
        cat_info = {}
        for cat_code, cat_id in set(categories):
            print("  cat", cat_code, cat_id)
            try:
                root_data = amz.get_category_info(cat_id)
            except Exception as err:
                print("ERROR: getting info for {}:{} crashed with {}".format(
                    shop, cat_id, err))
                continue

            if root_data is not None:
                cat_info[root_data['BrowseNodeId']] = (cat_code,
                                                       root_data['Name'])

        # save the info for this shop
        fh.write("{}:\n".format(shop))
        for cat_id in sorted(cat_info, key=int):
            cat_code, cat_name = cat_info[cat_id]
            if ',' in cat_name:
                assert '"' not in cat_name
                cat_name = '"{}"'.format(cat_name)
            fh.write("    {}: [{}, {}]\n".format(cat_id, cat_code, cat_name))

    fh.close()

if __name__ == "__main__":
    if len(sys.argv) != 2:
        print("Usage: {} <config_file.yaml>".format(sys.argv[0]))
        exit()

    main(sys.argv[1])
